#! /usr/bin/env python

import os
import sys
import regex
import rfc822
import string
import getopt

def fixpath():
    """Add the directory containing the script to the front of sys.path.

    Do not do this if we're being imported as a module, or if it is
    absent or if it is the current directory, or if it is already there.

    """
    if __name__ != '__main__':
	return
    scriptname = sys.argv[0]
    dirname = os.path.dirname(scriptname)
    while os.path.islink(scriptname):
	scriptname = os.path.normpath(
	    os.path.join(dirname, os.readlink(scriptname)))
	dirname = os.path.dirname(scriptname)
    if dirname not in sys.path:
	sys.path.insert(0, dirname)
    return dirname

SCRIPTDIR = fixpath()

from FileWrapper import FileWrapper

def main():
    backup = ""
    verbose = ""
    relative = ""
    compare = "C"
    try:
	opts, args = getopt.getopt(sys.argv[1:], "vbfrt:")
    except getopt.error, msg:
	usage(msg)
    if not args:
	usage("no files specified")
    for o, a in opts:
	if o == '-v': verbose = "V"
	if o == '-b': backup = "B"
	if o == '-f': compare = ""
	if o == '-r': relative = "R"
	if o == '-t': loadtemplate(a)
    mode = "w" + backup + compare + verbose + relative
    if relative:
	# - interpret args as being wrt current directory,
	# - and translate them to be wrt www.python.org web pages root
	#   (which is presumed to be parent of script dir).
	# - and then cd to the web pages root, so we work with respect to it.
	import relpath
	rootdir = os.path.normpath(os.path.join(SCRIPTDIR, os.pardir))
	curdir = os.getcwd()
	reldir = relpath.diff(rootdir, curdir)
	for i in range(len(args)):
	    args[i] = os.path.normpath(os.path.join(reldir, args[i]))
	os.chdir(rootdir)
    dolist(args, mode)

def dosubdir(dir, mode):
    print "Checking subdirectory", dir, "..."
    files = []
    try:
	names = os.listdir(dir)
    except os.error, msg:
	print "Can't list", dir, ":", msg
	return
    for name in names:
	if name[-5:] == 'html' or name[0] == '.': continue
	file = os.path.join(dir, name)
	if file[-4:] == '.htp' or (name != 'RCS' and name != 'CVS' and
				   os.path.isdir(file) and
				   not os.path.islink(file)):
	    files.append(file)
    files.sort()
    dolist(files, mode)

def dolist(args, mode):
    for file in args:
	if os.path.isdir(file):
	    dosubdir(file, mode)
	    continue
	name, ext = os.path.splitext(file)
	if ext == '.html':
	    print "Won't overwrite", file
	    continue
	htmlfile = name + '.html'
	try:
	    fi = open(file, "r")
	except IOError, msg:
	    print "Can't open", file, ":", msg
	    continue
	try:
	    fo = FileWrapper(htmlfile, mode)
	except (os.error, IOError), msg:
	    print "Can't create", htmlfile, ":", msg
	else:
	    try:
		process(fi, fo, name, file, HEAD, TAIL)
		fo.close()
	    finally:
		fo.close("abort")
	fi.close()

def usage(msg=None):
    sys.stdout = sys.stderr
    if msg:
	print msg
    print "usage:", sys.argv[0], "[-v] [-b] [-f] file.htp ..."
    print "-v: verbose (report files written/moved/removed)"
    print "-b: make backup of old html file if overwritten"
    print "-f: force writing html file (even if no change)"
    print "-t file: use the template defined in file"
    sys.exit(2)

def loadtemplate(name):
    global HEAD, TAIL, process
    if not name: return
    globals = {}
    try:
	fp = open(name)
	exec fp.read() in globals
    except:
	print "could not load template", name
	return
    HEAD = globals.get("HEAD", HEAD)
    TAIL = globals.get("TAIL", TAIL)
    process = globals.get("process", process)


HEAD = """<HTML>
<!-- THIS PAGE IS AUTOMATICALLY GENERATED.  DO NOT EDIT. -->

<HEAD>
<TITLE>%(title)s</TITLE>
</HEAD>

<BODY BACKGROUND="%(bgimage)s"
      BGCOLOR="#FFFFFF"
      LINK="#%(linkcolor)s"
      VLINK="#%(vlinkcolor)s">

<UL><!-- Enclose the entire page in UL, so bullets don't indent. -->

<IMG SRC="%(root)spics/ArrowLeft.gif" WIDTH=11 HEIGHT=11 ALT="["><B>
<A HREF="%(root)s">Home</A> |
<A HREF="%(root)spython/">Software</A> |
<A HREF="%(root)sdoc/">Documentation</A> |
<A HREF="%(root)spsa/">PSA</A> |
<A HREF="%(root)sworkshops/">Workshops</A> |
<A HREF="%(root)ssigs/">SIGs</A> |
<A HREF="%(root)slocator/">Search</A>
</B><IMG SRC="%(root)spics/ArrowRight.gif" WIDTH=11 HEIGHT=11 ALT="]">
<HR>

"""
# ' Emacs garbage

TAIL = """

<HR>
For comments on this web page, send email to
<A HREF="mailto:webmaster@python.org"><I>webmaster@python.org</I></A>.
<BR>
For questions about Python, send email to
<A HREF="mailto:python-help@python.org"><I>python-help@python.org</I></A>.

<HR>
<IMG SRC="%(root)spics/ArrowLeft.gif" WIDTH=11 HEIGHT=11 ALT="["><B>
<A HREF="%(root)s">Home</A> |
<A HREF="%(root)spython/">Software</A> |
<A HREF="%(root)sdoc/">Documentation</A> |
<A HREF="%(root)spsa/">PSA</A> |
<A HREF="%(root)sworkshops/">Workshops</A> |
<A HREF="%(root)ssigs/">SIGs</A> |
<A HREF="%(root)slocator/">Search</A>
</B><IMG SRC="%(root)spics/ArrowRight.gif" WIDTH=11 HEIGHT=11 ALT="]">

</UL><!-- End all-enclosing UL. -->

</BODY>
</HTML>
"""

def process(fi, fo, name, file, HEAD, TAIL):
    print "Processing", file, "..."
    depth = 0
    while name[:2] == "./":
	name = name[2:]
    for c in name:
	if c == '/': depth = depth + 1
    root = "../" * depth or "./"
    head, tail = os.path.split(name)
    if tail == 'index': size = 'Long'
    else: size = 'Short'
    if head[:3] == 'doc': color = 'Red'
    elif head[:3] == 'psa': color = 'Green'
    else: color = 'Blue'

    msg = rfc822.Message(fi)
    title = msg.getheader('title') or "Untitled Python Page"
    color = msg.getheader('color') or color
    size = msg.getheader('size') or size
    bgimage = root + "pics/" + color + size + '.gif'
    linkcolor = {'Red': 'AA0000', 'Green': '00D000', 'Blue': '0000FF'}[color]
    vlinkcolor = {'Red': '906A6A', 'Green': '609060', 'Blue': '7070B0'}[color]

    fo.write(HEAD % vars())
    fo.write(fi.read())
    fo.write(TAIL % vars())

if __name__ == '__main__':
    try:
	main()
    except KeyboardInterrupt:
	print "[Interrupted]"
	sys.exit(1)
