"""Style template to drive htp2html.py for KOE documentation.
"""
__version__ = '$Revision: 1.18 $'
#  $Source: /projects/cvsroot/koe/docs/support/koe-style.py,v $


import os
import rfc822
import formatter
import htmllib
import string
from style import *

class tocifyingParser(htmllib.HTMLParser):
    def __init__(self):
	htmllib.HTMLParser.__init__(self, formatter.NullFormatter())
	self.__data = ''
	self.__saving = 0
	self.__h2names = []
	self.__lastname = None
	self.__anchortext = {}		# by name
	self.__inheader = 0

    def feed(self, data):
	self.__data = self.__data + data
	htmllib.HTMLParser.feed(self, data)

    def anchor_bgn(self, href, name, type):
	if name:
	    if self.__inheader == 2:
		self.__h2names.append((name, []))
	    elif self.__inheader == 3 and self.__h2names:
		h2name, h3names = self.__h2names[-1]
		h3names.append(name)
	    self.__lastname = name
	    self.__saving = 1
	    self.save_bgn()
	htmllib.HTMLParser.anchor_bgn(self, href, name, type)

    def anchor_end(self):
	if self.__saving:
	    data = self.save_end()
	    if self.__anchortext.has_key(self.__lastname):
		print 'Warning!  Named anchor multiply defined:', \
		      self.__lastname
	    self.__anchortext[self.__lastname] = data
	self.__saving = 0
	htmllib.HTMLParser.anchor_end(self)

    def start_h2(self, attrs):
	self.__inheader = 2
	htmllib.HTMLParser.start_h2(self, attrs)

    def end_h2(self):
	self.__inheader = 0
	htmllib.HTMLParser.end_h2(self)

    def start_h3(self, attrs):
	self.__inheader = 3
	htmllib.HTMLParser.start_h1(self, attrs)

    def end_h1(self):
	self.__inheader = 0
	htmllib.HTMLParser.end_h1(self)

    def close(self):
	html = ''
	if self.__h2names:
	    html = '<div class=PageTOC>\n  <b>Table of Contents</b>\n  <ul>\n'
	    for name, h3names in self.__h2names:
		data = self.__anchortext[name]
		html = html + '    <li><a href="#%s">%s</a>\n' % (name, data)
		if h3names:
		    html = html + '      <ul>\n'
		    for h3name in h3names:
			data = self.__anchortext[h3name]
			html = html + '        <li><a href="#%s">%s</a>\n' % \
			       (h3name, data)
		    html = html + '      </ul>\n'
	    html = html + '  </ul>\n</div>\n'
	return self.__data, html


def load_page_sequence(filename):
    try:
	fp = open(filename)
    except IOError:
	print "Failed to loaded page sequence:"
	print "   ", filename
	return []
    result = []
    basedir = os.path.dirname(filename)
    while 1:
	line = fp.readline()
	if not line:
	    break
	line = string.strip(line)
	if line and line[0] == '-':	# ignore options
	    continue
	if not line:			# ignore blanks
	    continue
	basename = os.path.basename(line)
	if basename != "blankpage.html":
	    result.append(os.path.join(basedir, line))
    return result


NAVBAR_ITEM = (
    '<a href="%s">'
    '<img src="%smanuals/icons/%s.gif" alt="[%s]" border=0>'
    '</a>\n'
    )

def get_relative_dir(dirname, relative_to):
    """Get relative path to `dirname' from `relative_to'.

    Both parameters must be directory names, not file names.
    """
    dirpath = string.splitfields(dirname, os.sep)
    relative_to = string.splitfields(relative_to, os.sep)
    changed = 0
    while dirpath and relative_to and dirpath[0] == relative_to[0]:
	del dirpath[0]
	del relative_to[0]
	changed = changed + 1
    if changed < 2:
	return dirname
    dirpath = (len(relative_to) * [os.pardir]) + dirpath
    return string.joinfields(dirpath, os.sep)


def get_relative_url(source, target):
    dirname, basename = os.path.split(target)
    dirname = get_relative_dir(dirname, os.path.dirname(source))
    return os.path.join(dirname, basename)


def process(fi, fo, name, file, HEAD, TAIL):
##     print "Processing", file, "..."
    depth = 0
    current = os.curdir + os.sep
    while name[:2] == current:
	name = name[len(current):]
    depth = map(None, os.getcwd()).count(os.sep)
    root = "../" * depth or "./"
    head, tail = os.path.split(name)

    variables, used = merge_dict({}, rfc822.Message(fi))
    p = tocifyingParser()
    p.feed(fi.read())
    bodytext, toc = p.close()
    fi.close()
    times = [mtime(file)]
    # don't allow this to be covered by parent dirs:
    if not variables.has_key("title"):
	variables["title"] = "Untitled Document"
    searchdepth = maxdepth = depth + 1
    while searchdepth:
	reldir = (os.pardir + os.sep) * (maxdepth - searchdepth)
	filename = os.path.normpath(os.path.join(head, reldir + "index.htp"))
	searchdepth = searchdepth - 1
	try:
	    fp = open(filename)
	except IOError:
	    pass
	else:
	    variables, used = merge_dict(variables, rfc822.Message(fp))
    variables, used = merge_dict(variables, DEFAULT_VARIABLES)
    times.sort()
    variables, used = merge_dict(time_dict(times[-1], "modified-"), variables)
    processed = time_dict(time.localtime(time.time()), "processed-")
    variables, used = merge_dict(processed, variables)

    # get absolute document root:
    absroot = os.path.join(os.getcwd(), variables["document-root"])
    absroot = os.path.normpath(absroot)
    page_seq = load_page_sequence(
        os.path.join(absroot, "manuals", "HTML2PS-ARGS"))
    absoutput = os.path.join(os.getcwd(), name + ".html")
    variables["leading-navbar"] = ""
    variables["trailing-navbar"] = ""
    if absoutput in page_seq:
	docroot = variables["document-root"] + os.sep
	prev_page = next_page = top_page = None
	page_pos = page_seq.index(absoutput)
	if page_pos != 0 and page_pos != len(page_seq) - 1:
	    prev_page = get_relative_url(absoutput, page_seq[page_pos - 1])
	if page_pos < len(page_seq) - 2:
	    next_page = get_relative_url(absoutput, page_seq[page_pos + 1])
	if page_pos != len(page_seq) - 1:
	    top_page = get_relative_url(absoutput, page_seq[-1])
	# build the navigation bar:
	navbar = ''
	if top_page:
	    navbar = (
		NAVBAR_ITEM % (top_page, docroot, "contents", "Contents"))
	if prev_page:
	    navbar = navbar + (
		NAVBAR_ITEM % (prev_page, docroot, "prev", "Prev"))
	if next_page:
	    navbar = navbar + (
		NAVBAR_ITEM % (next_page, docroot, "next", "Next"))
	if navbar:
	    variables["leading-navbar"] = navbar + "<hr>"
	    variables["trailing-navbar"] = "<hr>" + navbar

    # insert the leading & trailing mini-tocs in the variables dictionary:
    if toc:
	variables["leading-toc"] = toc + "<hr size=1>\n"
	variables["trailing-toc"] = "<hr size=1>\n" + toc
    else:
	variables["leading-toc"] = ""
	variables["trailing-toc"] = ""
    fo.write(HEAD % variables)
    fo.write(bodytext)
    fo.write(TAIL % variables)


HEAD = '''<!doctype html public %(doctype)s>
<html>
  <head>
    <!-- THIS PAGE IS AUTOMATICALLY GENERATED.  DO NOT EDIT. -->
    <title>%(title)s</title>
    <link rev=made href="mailto:%(author-email)s">
    <!--<link rel=STYLESHEET href="%(document-root)s/support/style.css">-->
  </head>
<BODY BGCOLOR="#ffffff">
      <h1>%(title)s</h1>
      <hr>
      %(leading-navbar)s
      %(leading-toc)s
      <div class=PageContent>
<!-- auto-generated header end -->
'''

TAIL = '''\
<!-- auto-generated footer begin -->
      </div>
      %(trailing-toc)s
      %(trailing-navbar)s
      <!-- just because I like a positive "end of page" marker -->
      <hr>
      <div align=right>
        <address>
          Copyright &copy; 1998 by the Corporation for National Research
	  Initiatives.
        </address>
      </div>
  </body>
</html>
'''
