#!/usr/bin/env python
# A handy TK GUI to show off some of the KOE testcases
# It tries to handle the paths correctly if loaded as a Grail applet
# Author: Roger E. Masse

import sys
import os
INAPPLET = 0
if __name__ == '__main__':
    import ilu_tk
    here = os.path.split(sys.argv[0])[0]
    # koe package directory.
    sys.path.append(os.path.join(here, '../..'))
    # for FontLockText.py
    sys.path.append(os.path.join(here, '../../../nondist/src/demos/medline'))
    BASEURLDIR = 'file:' + sys.argv[0] # We're a program
elif sys.path[-1][:4] == 'http' or sys.path[-1][:4] == 'file':
    BASEURLDIR = sys.path[-1]          # We're an applet
    INAPPLET = 1
else:
    BASEURLDIR = 'file:' + __file__    # We're a module

import urllib
from koe.common import KPLauncher
from koe.common import physfonts
from Tkinter import *

# KPs to demo
KPS = [{'dir' : '../../testcases/', 'filename' : 'null.py'},
       {'dir' : '../../testcases/', 'filename' : 'sleepy.py'},
       {'dir' : '../../testcases/', 'filename' : 'sleepy_suitcase.py'},
       {'dir' : '../../testcases/', 'filename' : 'migrate.py'},
       {'dir' : '../../testcases/', 'filename' : 'simple_clone.py'},
       {'dir' : '../../testcases/', 'filename' : 'clone.py'},
       {'dir' : '../../testcases/', 'filename' : '7clones.py'},
       {'dir' : '../../testcases/', 'filename' : 'wander.py'},
       {'dir' : '../../examples/',  'filename' : 'nomad.py'},
       {'dir' : '../../examples/',  'filename' : 'scramble.py'},
       {'dir' : '../../testcases/', 'filename' : 'swarm.py'},
       {'dir' : '../../examples/',  'filename' : 'borg.py'},
       {'dir' : '../../examples/',  'filename' : 'mergesort.py'}]

# Various Suitcases for the above KPs
SC = {'mergesort.py' : [{'scname' : 'unsorted', 'filename' : 'sort-small.m'},
                        {'scname' : 'unsorted', 'filename' : 'sort-medium.m'},
                        {'scname' : 'unsorted', 'filename' : 'sort-large.m'}],
      'sleepy_suitcase.py'
                     : [{'scname' : 'sleeptime', 'filename' : 'sleeptime1'},
                        {'scname' : 'sleeptime', 'filename' : 'sleeptime5'},
                        {'scname' : 'sleeptime', 'filename' : 'sleeptime10'}],
      'migrate.py'   : [{'scname' : 'sleeptime', 'filename' : 'sleeptime1'},
                        {'scname' : 'sleeptime', 'filename' : 'sleeptime5'},
                        {'scname' : 'sleeptime', 'filename' : 'sleeptime10'}]}
      
class Controller:

    def __init__(self, master):

        self.master = master
        screenheight = master.winfo_screenheight()
        if screenheight < 1000: # adjust font for smaller screens
            self.font = physfonts.BODY_B_M
        else:
            self.font = physfonts.BODY_B_L
        self.create_widgets()

    def create_widgets(self):

        self.l = Label(self.master, font=self.font,
                       text='Choose a demo Knowbot Program:')
        self.l.pack(side=LEFT, anchor=E)
        if not INAPPLET:
            self.exit = Button(self.master, text='Exit',
                               command=sys.exit, font=self.font)
            self.exit.pack(side=RIGHT)
        self.mb = Menubutton(self.master, relief=RAISED,
                             indicator=1, font=self.font)
        self.mb.pack(side=RIGHT, anchor=W)
        self.m = Menu(self.mb, font=self.font, tearoff=0)
        self.mb['menu'] = self.m
        self.l = KPLauncher.KPLauncherGUI(self.master, BASEURLDIR)
        self.l.create_launch_panel()
        self.l.start_launch_panel(deiconify=0)
        
        for kp in KPS:
                   
            shortname = os.path.splitext(kp['filename'])[0]
            self.mb.config(text=shortname)
            scf = None
            if SC.has_key(kp['filename']):
                scf = SC[kp['filename']]

            if scf is None: # No suitcase files
                
                def _launch_command(self=self, dir=kp['dir'],
                                    filename=kp['filename']):
                    shortname = os.path.splitext(filename)[0]
                    self.mb.config(text=shortname)
                    if INAPPLET:
                        path = filename
                    else:
                        path = dir + filename
                    self.l.reset_kp(BASEURLDIR, filenames=[path,])
                    self.l.deiconify()
                    
                self.m.add_command(label=shortname, command=_launch_command)
                
            else:           # Create a submenu for suitcase file selection
                m = Menu(self.m, font=self.font, tearoff=0)
                for scf in SC[kp['filename']]:

                    def _launch_command(self=self, dir=kp['dir'],
                                        filename=kp['filename'], scf=scf):
                        shortname = os.path.splitext(filename)[0]
                        self.mb.config(text=shortname)
                        if INAPPLET:
                            path = filename
                        else:
                            path = dir + filename
                        self.l.reset_kp(BASEURLDIR, filenames=[path,],
                                        suitcasefiles=[scf,])
                        self.l.deiconify()

                    m.add_command(label=scf['filename'],
                                  command=_launch_command)
                    
                self.m.add_cascade(label=shortname, menu=m)
            
            # Isn't Python cool!
            
    def __cleanup__(self):
        """Grail applet cleanup discipline.  cleanup visualizer"""
        self.l.cleanup()

if __name__ == '__main__':
    root = Tk()
    c = Controller(root)
    print 'Entering main loop...'
    try:
        root.mainloop()
    finally:
        print 'Cleaning up visualizer...',
        c.l.cleanup()
        print 'Done.'
