import bookmarks
import kp
import httplib
import random
import copy
import kptools
import StringIO
import ilutools
import string
import sys
import types
import operator

def create_kp(bookmarks, argv):
    repr = kptools.Representation(modules=('httplib', 'bookmarks'),
                                  main='kp', path=sys.path)
    bm_repr = StringIO.StringIO()
    bookmarks.dump(bm_repr)
    repr.add_file('bookmarks.pym', string=bm_repr.getvalue())
    repr.add_argv(argv)
    return repr

def average(times):
    good = filter(lambda t:type(t)==types.FloatType, times)
    if not good:
        return None
    return reduce(operator.add, good) / float(len(good))
    
    
def merge(rdict):
    """Merge the results of a split on dispatch."""
    new = bookmarks.Bookmarks()
    dest = new.entries
    tests = {}
    for kss, results in rdict.items():
        for url, entry in results.entries.items():
            if entry.has_key('kss') and entry['kss'] == kss:
                dest[url] = entry
            elif entry.has_key('timing'):
                if entry.has_key('kss'):
                    # this was a test run at an alternate kss
                    tests[url] = entry
                else:
                    # this was a timing run on a new entry
                    [(other, times)] = entry['timing']
                    avg = average(times)
                    if not avg:
                        continue
                    entry['kss'] = kss
                    entry['latency'] = avg
                    entry['tries'] = 1
                    dest[url] = entry
            else:
                # unknown entry
                print "unknown entry\n", entry
    
    for url, tentry in tests.items():
        centry = new.entries[url]
        [(other, times)] = tentry['timing']
        avg = average(times)
        if not avg:
            continue
        if avg < centry['latency']:
            print "move %s new %s %7.5f old %s %7.5f" % \
                  (url, other, avg, centry['kss'], centry['latency'])
            centry['kss'] = other
            centry['latency'] = avg
            centry['tries'] = 1
        else:
            print "test %s alt %s %7.5f cur %s %7.5f" % \
                  (url, other, avg, centry['kss'], centry['latency'])
    return new

def dispatch(B, rv=0.1, ksses=[]):
    """Divides up a group of bookmarks by kss they will be checked at.

    The random argument is a percentage controlling how many bookmarks
    will also be checked at another kss to test the current assignment
    and average time.
    """
    parts = {}
    untimed = {}
    for url, entry in B.entries.items():
        try:
            kss = entry['kss']
        except KeyError:
            untimed[url] = entry
            continue
        try:
            bm = parts[kss]
        except KeyError:
            bm = parts[kss] = bookmarks.Bookmarks()
        bm.entries[url] = entry
    for kss in parts.keys():
        if not kss in ksses:
            ksses.append(kss)
    for url, entry in untimed.items():
        kss = random.choice(ksses)
        try:
            bm = parts[kss]
        except KeyError:
            bm = parts[kss] = bookmarks.Bookmarks()
        bm.entries[url] = entry

    if rv > 0.0:
        stations = parts.keys()
        for kss, bm in parts.items():
            others = copy.copy(stations)
            others.remove(kss)
            for url, entry in bm.entries.items():
                t = random.random()
                if t < rv:
                    other = random.choice(others)
                    parts[other].entries[url] = entry
    return parts
            
# The launcher, the callback, and main use a global variable

class Launcher:

    def __init__(self):
        self.pending = []
        self.reporter = kptools.SimpleReporter()
        self.where = {}
        self.results = {}
        self.io = kptools.IOServer()

    def add(self, dest, argv, bm=None, path=None):
        if not bm:
            if path:
                bm = bookmarks.Bookmarks(file=path)
            else:
                raise ValueError, "must supply bookmark object or path"
        self.pending.append((dest, argv, bm))

    def launch(self, kp, kss, rid):
        self.reporter.register_callback(rid, self.receive)
        self.where[rid] = kss
        self.io.linktokp(kp)
        kp.submit(kosname=kss, reporter=self.reporter, kpname=rid)

    def receive(self, repid, reason, details, repr):
        i = string.find(repid, '.')
        rid = repid[:i]
        if reason:
            print "KP failed (%s): %s, %s" % (str(rid), reason,
                                              details)
            # could return it to the pending list
        else:
            where = self.where[rid]
            results = bookmarks.Bookmarks()
            results.load(StringIO.StringIO(repr.get_file('bookmarks.pym')))
            mods = string.splitfields(repr.get_file('modified'), '\n')
            self.results[where] = (mods, results)
        del self.where[rid]

    def go(self):
        for kss, argv, bmdata in self.pending:
            kp = create_kp(bmdata, argv)
            rid = kss + str(hash(bmdata))
            self.launch(kp, kss, rid)
        self.reporter.wait()
        

def initialize(bookmarks, ksses):
    launcher = Launcher()
    for kss in ksses:
        launcher.add(kss, ['-i', '3'], bookmarks)
    launcher.go()
    return launcher.results

def update(bm_path):
    launcher = Launcher()
    bm = bookmarks.Bookmarks(file=bm_path)
    for kss, bm in dispatch(bm, 0.0).items():
        launcher.add(kss, ['-u', '--stdout'], bm)
    launcher.go()
    return merge(launcher.results)

if __name__ == "__main__":
    import sys
    results = update(sys.argv[1])
    print results
    for url, dict in results.entries.items():
        if dict['last_modified'] == dict['last_visit']:
            print url
    dest = open(sys.argv[2], 'w')
    results.dump(dest)
    dest.close()
    print "results written to", sys.argv[2]
        
