import bookmarks
import time
import sys
import string
import socket
import httplib

errors = (socket.error,) + httplib.errors

class KP:

    def __init__(self):
        self.opt = KPOptions()
        self.opt.parse(sys.argv[1:])
        if self.opt.update:
            self.main = self.update
            self.samples = range(3)
        elif self.opt.init:
            self.main = self.initialize
            self.samples = range(self.opt.samples)
        else:
            raise RuntimeError, "-i or -u must be specified"
        if self.opt.verbose:
            self.__log = []

    def __main__(self, kos):
        self.kos = kos
        self.sfs = self.kos.get_suitcase()
        self.here = kos.get_kos_name()
        self.main()
        if self.opt.verbose:
            f = self.sfs.open(self.opt.verbose, 'w')
            f.write(string.joinfields(self.__log, '\n'))
            f.close()
            del self.__log   # expensive to carry home
        # remove hard to pickle things
        self.sfs = self.main = None
        self.kos.checkpoint()

    def initialize(self):
        B = self.load_bookmarks()
        self.init_timing(B)
        self.store_bookmarks(B)

    def update(self):
        B = self.load_bookmarks()
        modified = self.update_all(B)
        m = self.sfs.open('modified', 'w')
        m.write(string.joinfields(modified, '\n'))
        m.close()
        self.store_bookmarks(B)

    def update_all(self, bm):
        modified = []
        for url, entry in bm.entries.items():
            print entry['kss'], self.here
            if entry.has_key('kss') and entry['kss'] == self.here:
                if self.update_entry(entry):
                    modified.append(url)
            else:
                # this is just a test
                self.time_entry(entry)
        return modified
        
    def update_entry(self, entry):
        """Compute a new digest for an entry an initialized entry."""
        now = int(time.time())
        modified = 0
        try:
            resp, delta = bookmarks.http_req(entry)
        except errors, msg:
            resp = httplib.Response()
            resp.status = 'ERR'
        entry['last_visit'] = now
        if resp.status == '200':
            entry['last_success'] = now
            digest = bookmarks.make_dig(resp)
            if not entry.has_key('digest') or digest != entry['digest']:
                entry['last_modified'] = now
                entry['digest'] = digest
                modified = 1
            # update timing information
            tries = entry['tries']
            newavg = (entry['latency'] * tries + delta) \
                     / (tries + 1)
            entry['tries'] = tries + 1
            entry['latency'] = newavg
            if entry.has_key('error'):
                del entry['error']

            self.log('time %s %8.6f' % (entry['url'], delta))
        else:
            entry['error'] = resp.status
            self.log('err %s %s' % (entry['url'], resp.status))
        return modified

    def time_entry(self, entry):
        times = []
        for i in self.samples:
            now = int(time.time())
            try:
                resp, delta = bookmarks.http_req(entry)
            except errors, msg:
                resp = httplib.Response()
                times.append(None)
                resp.status = 'ERR'
            else:
                if resp.status == '200':
                    times.append(delta)
                else:
                    times.append(resp.status)
        self.log('test %s %s' % (entry['url'], string.join(
            map(lambda t: (t and str(t)) or '-', times))))
        try:
            entry['timing'].append((self.here, times))
        except KeyError:
            entry['timing'] = [(self.here, times)]
        if resp.status == '200':
            entry['digest'] = bookmarks.make_dig(resp)
            entry['last_visit'] = now
            entry['last_modified'] = now
            entry['last_success'] = now
        else:
            entry['last_visit'] = now
            entry['error'] = resp.status

    def init_timing(self, bm):
        for url in bm.entries.keys():
            self.time_entry(bm.entries[url])

    def log(self, str):
        if self.opt.verbose:
            self.__log.append(str)
        if self.opt.stdout:
            print str

    def load_bookmarks(self):
        bm = bookmarks.Bookmarks()
        bm.load(self.sfs.open(self.opt.source))
        return bm

    def store_bookmarks(self, bm):
        f = self.sfs.open(self.opt.dest, 'w')
        bm.dump(f)
        f.close()

class KPOptions:
    """Usage:

    -s path: source file
    -d dest: dest file
    -u: update the bookmarks
    -i n: do initial timing on bookmarks with n samples
    -v path: create a log file that has detailed information
    --stdout: print the log file to stdout
    """

    def __init__(self):
        self.source = 'bookmarks.pym'
        self.dest = 'bookmarks.pym'
        self.update = None
        self.init = None
        self.verbose = ''
        self.stdout = None

    def parse(self, argv):
        import getopt
        opts, args = getopt.getopt(argv, 's:d:ui:v:', ['stdout'])
        for k, v in opts:
            if k == '-s':
                self.source = v
            elif k == '-d':
                self.dest = v
            elif k == '-u':
                self.update = 1
            elif k == '-i':
                self.init = 1
                self.samples = string.atoi(v)
            elif k == '-v':
                self.verbose = v
            elif k == '--stdout':
                self.stdout = 1

    
