import bookmarks
import dispatcher
import kptools

import string
import StringIO
import time
import os

known_ksses = ['gather', 'hs3']

class Launcher:
    
    def __init__(self):
        self.times = {}
        self.where = {}
        self.results = []
        self.reporter = kptools.SimpleReporter()

    def launch(self, kp, kss, rid):
        self.reporter.register_callback(rid, self.receive)
        self.where[rid] = kss
        self.times[kss] = time.time()
        kp.submit(kosname=kss, reporter=self.reporter, kpname=rid)

    def receive(self, repid, reason, details, repr):
        now = time.time()
        i = string.find(repid, '.')
        rid = repid[:i]
        if not reason:
            # we succeeded
            where = self.where[rid]
            self.times[where] = time.time() - self.times[where]
            results = bookmarks.Bookmarks()
            results.load(StringIO.StringIO(repr.get_file('bookmarks.pym')))
            log = repr.get_file('log')
            self.results.append((where, log, results))

    def go(self, parts):
        argv = ['-v', 'log', '-u']
        for kss, bm in parts.items():
            if not kss:
                continue
            kp = dispatcher.create_kp(bm, argv)
            rid = kss + str(hash(bm))
            self.launch(kp, kss, rid)
        self.reporter.wait()

class Tester:

    def __init__(self, bm, logpath):
        self.bm = bm
        self.logpath = logpath
        self.round = 0
        self.id = hex(int(time.time()) % 482112000)[2:]

    def open(self, name):
        path = os.path.join(self.logpath,
                            '%s.%s.%s' % (self.id, self.round, name))
        return open(path, 'w')

    def go(self):
        self.round = self.round + 1

        # do the split version
        b = time.time()
        l = Launcher()
        l.go(dispatcher.dispatch(self.bm, 0.0, known_ksses))
        e = time.time()

        # print summary line
        print "split %7.4f " % (e - b),
        for kss, log, bm in l.results:
            print "%s %d %7.4f " % (kss, len(bm.entries), l.times[kss]),
            f = self.open('log.%s' % (kss))
            f.write(log)
            f.close()
            bm.dump(self.open('bm.%s' % (kss)))
        print

        # do two regular versions
        b = time.time()
        l = dispatcher.Launcher()
        l.add('hs3', ['-u'], self.bm)
        l.go()
        e = time.time()
        print "all hs3 %7.4f" % (e - b)

        b = time.time()
        l.add('gather', ['-u'], self.bm)
        l.go()
        e = time.time()
        print "all gather %7.4f" % (e - b)

class RandTester(Tester):

    def go(self):
        self.round = self.round + 1
        b = time.time()
        l = Launcher()
        l.go(dispatcher.dispatch(self.bm, ksses=known_ksses))
        e = time.time()
        rdict = {}
        for kss, log, bm in l.results:
            rdict[kss] = bm

        # print summary line
        print "split %7.4f " % (e - b),
        for kss, log, bm in l.results:
            print "%s %d %7.4f " % (kss, len(bm.entries), l.times[kss]),
            f = self.open('log.%s' % (kss))
            f.write(log)
            f.close()
            bm.dump(self.open('bm.%s' % (kss)))
        print

        new = dispatcher.merge(rdict)
        self.bm = new

if __name__ == "__main__":
    import sys
    t = Tester(bookmarks.Bookmarks(file=sys.argv[1]))
    for i in range(10):
        t.go()
