"""Perform an HTTP request with a KP and the HTTP extension.

The functions ending in 't' return the time and perhaps number of bytes.

Warning: The reporting station doesn't poll for keyboard interrupts,
so errors in the KP or KOS will cause you to hang.
"""

import kptools
import string
import urlopenkp
import ilu
import os
import marshal
import operator

__kos = os.uname()[1]
__rs = kptools.SimpleReporter()
 

def set_kos(kos):
    global __kos
    __kos = kos

def urlopen(url):
    "Get the contents of the resource named by the url argument."
    return urlopent(url)[0]

def urlopent(url):
    ready = 0
    env = {}
    def callback(id, reason, details, repr, env=env):
        try:
            s = repr.get_file('results')
            html, time = marshal.loads(s)
        except (IOError, EOFError):
            html = ''
            time = 0.0
        env['html'] = html
        env['time'] = time

    kp = kptools.Representation(main='urlopenkp')
    kp.add_file('url', url)
    id = str(hash(url))
    __rs.register_callback(id, callback)
    kp.submit(kosname = __kos,
              reporter = __rs,
              kpname = id)
    __rs.wait()
    return env['html'], env['time']

def prefetch(url):
    "Returns a dictionary containing url and its inline images."
    return prefetcht(url)[0]
    
def prefetcht(url):
    ready = 0
    env = {}
    def callback(id, reason, details, repr, env=env):
        try:
            raw = repr.get_file('results')
        except IOError:
            env['urls'] = {}
            env['total'] = 0.0
            env['bytes'] = 0
            return
        pages = marshal.loads(raw)
        env['urls'] = pages
        env['bytes'] = reduce(operator.add,
                              map(lambda t:len(t[0]), pages.values()))
        env['total'] = reduce(operator.add,
                              map(lambda t:t[1], pages.values()))
 
    kp = kptools.Representation(main='prefetchkp')
    kp.add_file('url', url)
    id = str(hash(url))
    __rs.register_callback(id, callback)
    kp.submit(kosname = __kos,
              reporter = __rs,
              kpname = id)
    __rs.wait()
    return env['urls'], env['bytes'], env['total']
