import time
import string
import urlparse
import regex
import marshal
 
inlinerx = regex.compile('\(<[^>]+\(BACKGROUND\|SRC\|background\|src\)\w*\=\w*\"\([^"]+\)\"\)')
# using htmllib is *WAY* too slow, like 30 times slower

def get_inline(buf, rx=inlinerx):
    links = []
    off = 0
    while 1:
        i = rx.search(buf, off)
        if i == -1:
            break
        links.append(rx.group(3))
        off = i + len(rx.group(1))
    return links

# this can get called a lot, so customizing it helps
def urljoin(base, url, allow_framents = 1, tuple_type=type(())):
    if type(base) == tuple_type:
        bscheme, bnetloc, bpath, bparams, bquery, bfragment = base
    else:
        bscheme, bnetloc, bpath, bparams, bquery, bfragment = \
                 urlparse(base, '', allow_framents)
    scheme, netloc, path, params, query, fragment = \
            urlparse.urlparse(url, bscheme, allow_framents)
    # XXX Unofficial hack: default netloc to bnetloc even if
    if scheme in urlparse.uses_netloc:
        if netloc:
            return scheme, netloc, path, params, query, fragment
        netloc = bnetloc
    if path[:1] == '/':
        return scheme, netloc, path, params, query, fragment
    if not path:
        path = bpath
        if not query:
            query = bquery
        return scheme, netloc, path, params, query, fragment
    i = string.rfind(bpath, '/')
    if i >= 0:
        path = bpath[:i] + '/' + path
    segments = string.splitfields(path, '/')
    if segments[-1] == '.':
        segments[-1] = ''
    while '.' in segments:
        segments.remove('.')
    while 1:
        i = 1
        n = len(segments) - 1
        while i < n:
            if segments[i] == '..' and segments[i-1]:
                del segments[i-1:i+1]
                break
            i = i+1
        else:
            break
    if len(segments) >= 2 and segments[-1] == '..':
        segments[-2:] = ['']
    path = string.joinfields(segments, '/')
    return scheme, netloc, path, params, query, fragment
   
class KP:

    def __init__(self):
        self.pages = {}

    def __main__(self, kos):
        self.suitcase = kos.get_suitcase()
        self.http = self.load_http(kos)
        url = self.load_url()

        # get the first page
        try:
            html, delta = self.get(url)
        except IOError:
            return
        self.save_page(url, html, delta)

        # do some prefetching
        base = urlparse.urlparse(url)
        links = get_inline(html)
        local = {}
        for link in links:
            try:
                if link[:5] != 'http:':
                    parts = urljoin(base, link)
            except (IndexError, ValueError):
                pass
            else:
                if parts[1] == base[1]:
                    local[urlparse.urlunparse(parts)] = 1

        for link in local.keys():
            try:
                img, delta = self.get(link)
            except IOError:
                pass
            self.save_page(link, img, delta)

        self.save()
        self.http.Close()

    def get(self, url):
        page = self.http.Get(url)
        s = time.time()
        page.open('r')
        #m = time.time()
        html = page.read(-1)
        f = time.time()
        return html, f - s

    def load_http(self, kos):
        factory = kos.lookup_service("HTTPAPI.Factory", "http").Open()
        return factory.GetHTTPRequester()

    def load_url(self):
        f = self.suitcase.open('url', 'r')
        url = f.read()
        f.close()
        return url

    def save_page(self, url, html, delta):
        self.pages[url] = (html, delta)

    def save(self):
        f = self.suitcase.open('results', 'w')
        f.write(marshal.dumps(self.pages))
        f.close()

