"""The decryption KP, launched by the ImageRepo.py plugin.

This KP must be loaded with the following `files' in its suitcase:

    rotorsn     - the number of rotors
    destination - rendezvous KOS

See ImageRepo.py for details.

"""

__version__ = '$Id: decrypt.py,v 1.17 1998/06/11 15:06:30 bwarsaw Exp $'

import sys
import string
import md5
import rotor
import time
from types import *
import nstools


# states
NEWBORN = 'newborn'                     # haven't gone anywhere yet
PROVIDE = 'provide'                     # provide the service
CLONED  = 'cloned'                      # cloned



class KP:
    __rotorsn = None
    __state = NEWBORN

    def __main__(self, kos):
        func = getattr(self, self.__state)
        func(kos)

    def newborn(self, kos):
        # we want to dig this information out of our suitcase before
        # we migrate.  it's probably slightly less easy to rip off
        # this information when it's in our state pickle.
        #
        suitcase = kos.get_suitcase()
        #
        # dont' wrap these in try/except. we *want* such
        # exceptions to abort us
        #
        for fn, conv in (('rotorsn',     string.atoi),
                         ('destination', lambda x: x),
                         ):
            fp = suitcase.open(fn, 'r')
            val = conv(fp.read())
            fp.close()
            suitcase.unlink(fn)
            # do not use double underscores for these attributes.  too
            # implementation specific to map to Python 1.4's private
            # attribute scheme.
            setattr(self, '_'+fn, val)
        #
        # now migrate
        #
        self.__state = PROVIDE
        kos.migrate(self._destination)
        # never returns...

    def provide(self, kos):
        # make sure there isn't already a decrypt player at this site.
        # the current demo bogusly only supports one TollPlayer
        # service per site
        try:
            player = kos.lookup_service('PlayerAPI.DecryptPlayer',
                                        'TollPlayer').Open()
            player.Close()
        except nstools.BadPathError:
            pass
        #
        # we're at our destination, so create our decrypt-and-play
        # service and wait for rendezvous
        #
        player = DecryptPlayer(kos, self._rotorsn)
        playerid = kos.bind_service('TollPlayer', player,
                                    'PlayerAPI.DecryptPlayer')
        kos.run()
##         print 'Unbinding TollPlayer service w/ id:', playerid
        kos.unbind_service(playerid)



# stand-in for un-import-able module (really just their exceptions)
PlayerAPI = None

class DecryptPlayer:
    __imagename = ''
    __imagedata = None
    __player = None
    __key = None

    def __init__(self, kos, rotorsn):
        global PlayerAPI
        self.__kos = kos
        self.__rotorsn = rotorsn
        #
        # We need to get these exceptions for later, and we aren't
        # allowed to import the PlayerAPI module.  Make it look like
        # it though... :-)
        #
        PlayerAPI = self.__kos.all_exceptions('PlayerAPI')

    #
    # API
    #

    def SetImageName(self, name):
        self.__imagename = name

    def SetImageData(self, data):
        self.__imagedata = data

    def SetImageKey(self, key):
        self.__key = key

    def Play(self):
        if self.__key is None:
            raise PlayerAPI.AuthorizationFailed, 'Decrypt key required'
        if self.__imagedata is None:
            raise PlayerAPI.ImageError, 'No image data'
        #
        # get the lower level image display service
        #
        service = self.__kos.lookup_service(
            'PlayerAPI.PlayerFactory', 'PlayerFactory').Open()
        self.__player = player = service.GetNewPlayer()
        #
        # perform the decryption
        #
        r = rotor.newrotor(self.__key, self.__rotorsn)
        data = r.decrypt(self.__imagedata)
        #
        # now play it through the low level interface
        #
        player.SetImageName(self.__imagename)
        player.SetImageData(data)
        player.Play()

    def Close(self):
##      if self.__player:
##          self.__player.Close()
        self.__kos.stop()
