"""Slogan object and database parser."""

__version__ = '$Revision: 1.1 $'


import Rebuilder                        # koe.tools.xml.Rebuilder
import stop                             # koe.common.stop
import string


d = {}
for w in stop.sw:
    d[w] = w
is_stopword = d.has_key


def fromxml((gi, attrs, content)):
    text = author = email = ''
    for gi, attrs, content in content:
        if gi == "text":
            text = flatten(content)
        elif gi == "author":
            author = flatten(content)
            email = string.strip(attrs.get("email"))
    return Slogan(text, author, email)

def flatten(content):
    s = ''
    for stuff in content:
        if type(stuff) is type(''):
            s = s + stuff
        else:
            s = s + flatten(stuff[-1])
    return s


class Slogan:
    __indexing = None
    __normalized = None

    def __init__(self, text, author, email=''):
        self.text = text
        self.author = author
        self.email = email

    def get_index_info(self):
        if self.__indexing is None:
            self.__generate_index_info()
        return self.__indexing

    def get_normalized_phrase(self):
        if self.__normalized is None:
            self.__generate_normalized_phrase()
        return self.__normalized

    def tohtml(self):
        """Generate HTML text for this item."""
        s = ("      <li> %s<br>\n"
             "           &nbsp;&nbsp;&nbsp;<em>"
             % self.text)
        if self.email:
            s = '%s<a href="mailto:%s">%s</a>' % (s, self.email, self.author)
        else:
            s = s + self.author
        return s + "</em>\n"

    def toxml(self):
        email_attrs = {}
        if self.email:
            email_attrs["email"] = self.email
        content = [("text", {}, [self.text]),
                   ("author", email_attrs, [self.author])]
        return ("slogan", {}, content)

    # internal methods

    puncts = ",-&@=!><?/()[]{}"
    __tr = string.maketrans(puncts + string.uppercase,
                            ' '*len(puncts) + string.lowercase)
    del puncts

    def __generate_index_info(self):
        d = {}
        s = string.translate(self.text, self.__tr)
        words = string.split(s)
        for w in words:
            while w and w[-1] == '.':
                w = w[:-1]
            if not is_stopword(w):
                d[w] = w
        self.__indexing = d

    def __generate_normalized_phrase(self):
        s = string.translate(self.text, self.__tr)
        words = string.split(s)
        # this word list needs work:
        if words[0] in ('a', 'an', 'the', 'it'):
            del words[0]
        for i in range(len(words)):
            w = words[i]
            while w and w[-1] == '.':
                w = w[:-1]
            words[i] = w
        words = filter(lambda w: len(w) > 1, words)
        self.__normalized = string.join(words)


def byauthor(l, r):
    return cmp(l.author, r.author)


class SloganDatabaseParser(Rebuilder.XMLRebuilder):
    def __init__(self, ):
        Rebuilder.XMLRebuilder.__init__(self, contentsof=("text", "author"))
        self.__slogans = []

    def rebuild_slogan(self, xml):
        slogan = fromxml(xml)
        self.__slogans.append(slogan)
        return slogan

    def get_slogans(self):
        return self.__slogans
