"""Sloganizer KP."""

__version__ = '$Revision: 1.4 $'


import cgi
import getopt
import nstools
import whrandom
import string
import sys

import Slogan


class KP:
    def __init__(self):
        self.__state = "initialize"
        self.__verbose = 0

    def __main__(self, kos):
        self.cr_api = kos.all_exceptions("ContentRegistrarAPI")
        self.map_api = kos.all_exceptions("Mapping")
        methodname = "do_" + self.__state
        method = getattr(self, methodname)
        method(kos)

    def do_initialize(self, kos):
        self.__homebase = kos.get_kos_name()
        opts, args = getopt.getopt(sys.argv[1:], "v", ["verbose"])
        if opts:
            self.__verbose = 1
        if args:
            self.__target = args[0]
        else:
            self.__target = self.__homebase
        self.__state = "startup"
        if self.__homebase == self.__target:
            self.do_startup(kos)
        else:
            self.migrate(self.__target)

    def do_startup(self, kos):
        self.registrar = kos.lookup_service(
            "ContentRegistrarAPI.ContentRegistrar", "Registrar").Open()
        sfs = kos.get_suitcase()
        self.load_database(sfs)
        # To create the ILU true object, bind the service and then open the
        # "opaque" id to get a reference to it.
        id = kos.bind_service("my-content-provider", self,
                              "ContentProviderAPI.ContentProvider")
        self.kos = kos
        self.register(sfs, id.Open())
        if self.__verbose:
            print "providing service at", kos.get_kos_name()
        kos.run()
        kos.unbind_service(id)
        self.unregister()
        # fall through and exit KP

    def register(self, sfs, provider):
        """Register our ContentProvider interface with the content manager."""
        try:
            self.id = self.registrar.RegisterProvider("slogans")
        except self.cr_api.NonUniqueNameError, err:
            self.id = err
        self.registrar.RegisterLiveContent(self.id, provider)

    def unregister(self):
        """Remove registration for dynamic content."""
        self.registrar.UnRegisterLiveContent(self.id)
        self.registrar.UnRegisterProvider(self.id)

    def remove_static_files(self):
        sfs = self.kos.get_suitcase()
        for filename in sfs.listdir("/"):
            if sfs.path.isfile(filename):
                print "Discarding suitcase file", filename
                sfs.unlink(filename)

    def load_database(self, sfs):
        try:
            self.__database = self.read_database(sfs, "var/slogans.xml")
        except IOError:
            self.__database = []
        try:
            self.__suggestions = self.read_database(sfs, "var/suggestions.xml")
        except IOError:
            self.__suggestions = []

    def read_database(self, sfs, filename):
        parser = Slogan.SloganDatabaseParser()
        parser.feed(sfs.open(filename).read())
        parser.close()
        return parser.get_slogans()

    __modified = 1
    def save_database(self, sfs=None):
        if sfs is None:
            sfs = self.kos.get_suitcase()
        fp = sfs.open("var/slogans.xml", "w")
        self.__write_database(fp, self.__database)
        fp.close()
        self.__modified = 1

    def save_suggestions(self, sfs=None):
        if sfs is None:
            sfs = self.kos.get_suitcase()
        fp = sfs.open("var/suggestions.xml", "w")
        self.__write_database(fp, self.__suggestions)
        fp.close()

    def __write_database(self, fp, database, dbattrs={}):
        content = []
        xml = ("slogans", dbattrs, content)
        for slogan in database:
            content.append(slogan.toxml())
        fp.write("<!DOCTYPE slogans SYSTEM 'slogans.dtd'>\n")
        pprint_xml(fp, xml)

    def __write_html(self, fp, database, description=""):
        strings = []
        for slogan in database:
            strings.append(slogan.tohtml())
        vars = {
            "data": string.join(strings, ''),
            "description": description,
            }
        data = self.format_template("slogans.html,t", vars)
        fp.write(data)

    def save_html(self):
        fp = self.kos.get_suitcase().open("slogans.html", "w")
        self.__write_html(fp, self.__database)
        fp.close()
        self.__modified = 0

    __tr = string.maketrans("./", "__")
    def GetContent(self, url, env, fp):
        """Provide content in response to a user request."""
        if self.__verbose:
            print "URL:", url
        while url and url[0] == '/':
            url = url[1:]
        parts = string.split(url, '/')
        url = string.join(parts[1:], '/')
        try:
            http_method = string.lower(env.GetValue("REQUEST_METHOD"))
        except self.map_api.KeyError:
            self.http_error(fp, 400, "No HTTP method specified")
            fp.flush()
            return
        methodname = "%s_%s" % (http_method, string.translate(url, self.__tr))
        try:
            method = getattr(self, methodname)
        except AttributeError:
            self.http_error(fp, 404, "No such resource")
        else:
            method(env, fp, self.kos)
        fp.flush()

    def get_(self, env, fp, kos):
        self.dump_template(fp, "index.html,t", "text/html")

    def get_slogans_xml(self, env, fp, kos):
        self.dump_file(fp, "var/slogans.xml", "text/xml")

    def get_slogans_html(self, env, fp, kos):
        if self.__modified:
            self.save_html()
        self.dump_file(fp, "slogans.html", "text/html")

    def get_byauthor_xml(self, env, fp, kos):
        slogans = self.__database[:]
        slogans.sort(Slogan.byauthor)
        fp.write("Content-Type: text/xml\n\n")
        self.__write_database(fp, slogans, {"sort-key": "author"})

    def get_byauthor_html(self, env, fp, kos):
        slogans = self.__database[:]
        slogans.sort(Slogan.byauthor)
        fp.write("Content-Type: text/html\n\n")
        self.__write_html(fp, slogans,
                          description="Slogans sorted by contributor.")

    def get_random_xml(self, env, fp, kos):
        if self.__database:
            slogan = whrandom.choice(self.__database)
            fp.write("Content-Type: text/xml\n\n")
            self.__write_database(fp, [slogan])
        else:
            self.dump_template(fp, "norandom.html,t", "text/html")

    def get_random_html(self, env, fp, kos):
        if self.__database:
            slogan = whrandom.choice(self.__database)
            vars = {
                "title": "Random Slogan",
                "data": slogan.tohtml(),
                "description": "",
                }
            self.dump_template(fp, "slogans.html,t", "text/html", vars)
        else:
            self.dump_template(fp, "norandom.html,t", "text/html")

    def get_slogans_dtd(self, env, fp, kos):
        self.dump_file(fp, "slogans.dtd", "text/xml")

    def get_admin(self, env, fp, kos):
        self.dump_template(fp, "admin.html,t", "text/html")

    def post_admin(self, env, fp, kos):
        fields = self.parse_form(env, fp)
        userid = string.strip(fields.get("userid", [''])[0])
        passwd = fields.get("passwd", [''])[0]
        operations = self.check_access(userid, passwd)
        if not operations:
            self.http_error(fp, 403, "Unauthorized user")
            return
        prelude = ''
        found_op = 0
        if fields.has_key("quit"):
            if "quit" in operations:
                self.dump_file(fp, "quitting.html", "text/html")
                kos.stop()
            else:
                self.http_error(fp, 403, "Operation not allowed")
            return
        if not fields.has_key("reload"):
            for k in fields.keys():
                parts = string.split(k)
                if len(parts) == 2 and parts[0] in ("accept", "remove"):
                    if parts[0] not in operations:
                        self.http_error(fp, 403, "Operation not allowed")
                        return
                else:
                    continue
                slogan = None
                for s in self.__suggestions:
                    sid = `id(s)`
                    if sid == parts[1]:
                        slogan = s
                        break
                # This is allowed.
                if parts[0] == "accept":
                    self.__suggestions.remove(slogan)
                    self.__database.append(slogan)
                    sfs = self.kos.get_suitcase()
                    self.save_database(sfs)
                    self.save_suggestions(sfs)
                    prelude = SLOGAN_HAS_BEEN_ADDED % slogan.tohtml()
                    break
                elif parts[0] == "remove":
                    self.__suggestions.remove(slogan)
                    self.save_suggestions()
                    prelude = SLOGAN_HAS_BEEN_REMOVED % slogan.tohtml()
                    break
                found_op = 1
            else:
                if found_op:
                    prelude = ADMIN_UNKNOWN_SLOGAN
        if self.__suggestions:
            s = ''
            for slogan in self.__suggestions:
                sid = `id(slogan)`
                s = (s +
                     "      <hr>\n"
                     "      <table width='100%'>\n"
                     "        <tr><td>\n"
                     "              <ul>\n" +
                     slogan.tohtml() +
                     "              </ul>\n"
                     "            </td>\n"
                     "            <td align=right>\n")
                if "accept" in operations:
                    s = (s +
                         "          <input type=submit value='Accept' name='accept " +
                         sid + "'>\n")
                if "remove" in operations:
                    s = (s +
                         "          <input type=submit value='Remove' name='remove " +
                         sid + "'>\n")
                s = (s +
                     "            </td>\n"
                     "          </tr>\n"
                     "      </table>\n")
        else:
            s = NO_SUGGESTED_SLOGANS
        vars = {"prelude": prelude,
                "body": s,
                "passwd": passwd,
                "userid": userid}
        self.dump_template(fp, "admindata.html,t", "text/html", vars)

    def get_suggest(self, env, fp, kos):
        self.dump_template(fp, "suggest.html,t", "text/html",
                           {"prelude": DEFAULT_SUGGEST_PRELUDE,
                            "author": '',
                            "email": '',
                            "text": ''})

    def post_suggest(self, env, fp, kos):
        fields = self.parse_form(env, fp)
        text = string.strip(fields.get("text", [''])[0])
        author = string.strip(fields.get("author", [''])[0])
        email = string.strip(fields.get("email", [''])[0])
        if not (text and author and email):
            prelude = """\
The following fields were not filled in, but are required:
    </p>
    <ul>
"""
            if not text:
                prelude = prelude + "      <li> The slogan!\n"
            if not author:
                prelude = prelude + "      <li> Your name!\n"
            if not email:
                prelude = prelude + "      <li> Your email address!\n"
            prelude = prelude \
                      + "    </ul>\n    <p> Please provide this information."
            vars = {"prelude": prelude,
                    "text": text,
                    "author": author,
                    "email": email}
            self.dump_template(fp, "suggest.html,t", "text/html", vars)
            return
        if string.count(email, "@") != 1 or \
           not string.count(email[string.find(email, "@"):], "."):
            prelude = BAD_EMAIL_ADDRESS
            vars = {"prelude": prelude,
                    "text": text,
                    "author": author,
                    "email": email}
            self.dump_template(fp, "suggest.html,t", "text/html", vars)
            return
        slogan = Slogan.Slogan(text, author, email)
        problem = self.suggestion_problems(slogan)
        if problem:
            prelude = (
                "Your suggestion:\n    </p>\n    <ul>\n"
                + slogan.tohtml()
                + "    </ul>\n"
                + "    <p> cannot be added to the database.\n"
                + "      "
                + problem
                )
            vars = {"prelude": prelude,
                    "text": text,
                    "author": author,
                    "email": email}
        else:
            self.__suggestions.append(slogan)
            self.save_suggestions()
            prelude = (
                "Thank you for your suggestion:\n    </p>\n    <ul>\n"
                + slogan.tohtml()
                + "    </ul>\n"
                + "    <p> Please feel free to make additional suggestions.")
            vars = {"prelude": prelude,
                    "text": '',
                    "author": author,
                    "email": email}
        self.dump_template(fp, "suggest.html,t", "text/html", vars)

    # general helpers for the resource handlers:

    def parse_form(self, env, fp):
        clen = int(env.GetValue("CONTENT_LENGTH"))
        data = fp.read(clen)
        return cgi.parse_qs(data)

    def suggestion_problems(self, slogan):
        new_phrase = slogan.get_normalized_phrase()
        # should add a check for obscenities first.
        for slogan in self.__database:
            phrase = slogan.get_normalized_phrase()
            if phrase == new_phrase:
                return ("The new slogan is too similar to"
                        " an existing database entry:\n"
                        "    <ul>\n" +
                        slogan.tohtml() +
                        "    </ul>\n"
                        "    <p>\n")
        for slogan in self.__suggestions:
            phrase = slogan.get_normalized_phrase()
            if phrase == new_phrase:
                return ("The new slogan is too similar to"
                        " a recent suggestion.")
        return None

    def http_error(self, fp, errno, msg):
        if self.__verbose:
            print "ERROR:", errno, msg
        s = HTTP_ERROR_TEMPLATE % {"errno": errno, "message": msg}
        fp.write(s)

    def check_access(self, userid, passwd, operation=None):
        fp = self.kos.get_suitcase().open("access-list")
        tr = string.maketrans(",", " ")
        while 1:
            line = fp.readline()
            if not line:
                break
            u, p, options = string.split(line, None, 2)
            if u == userid and p == passwd:
                options = string.translate(options, tr)
                options = string.split(options)
                if not operation:
                    return options
                if operation in options:
                    return options
                return 0
        return 0

    def dump_file(self, fp, filename, ctype):
        sfs = self.kos.get_suitcase()
        data = sfs.open(filename).read()
        fp.write("Content-Type: %s\nContent-Length: %d\n\n"
                 % (ctype, len(data)))
        fp.write(data)

    def dump_template(self, fp, filename, ctype, vars={}):
        data = self.format_template(filename, vars)
        fp.write("Content-Type: %s\nContent-Length: %d\n\n"
                 % (ctype, len(data)))
        fp.write(data)

    __vars = {
        "title":  "Python Marketing Slogans",
        "dbname": "Python Advocacy",
        }

    def format_template(self, filename, vars={}):
        myvars = self.__vars.copy()
        myvars.update(vars)
        sfs = self.kos.get_suitcase()
        data = sfs.open(filename).read()
        return data % myvars


def format_stag(gi, attrs, etag=0):
    s = "<" + gi
    for k, v in attrs.items():
        if "'" in v:
            if '"' in v:
                # need to escape " chars
                parts = string.split(v, '"')
                v = string.join(parts, "&#%s;" % ord('"'))
            s = '%s %s="%s"' % (s, k, v)
        else:
            s = "%s %s='%s'" % (s, k, v)
    return s + (etag and "/>" or ">")


def format_etag(gi):
    return "</%s>" % gi


def pprint_xml(fp, xml, depth=0):
    if type(xml) is type(''):
        fp.write(xml)
        return
    fp.write(depth * "  ")
    gi, attrs, content = xml
    if content:
        fp.write(format_stag(gi, attrs))
        if len(content) == 1 and type(content[0]) is type(''):
            fp.write(content[0])
            fp.write(format_etag(gi))
        else:
            fp.write("\n")
            for x in content:
                pprint_xml(fp, x, depth + 1)
            fp.write(depth * "  ")
            fp.write(format_etag(gi))
    else:
        fp.write(format_stag(gi, attrs, 1))
    fp.write("\n")



HTTP_ERROR_TEMPLATE = '''\
Content-Type: text/html

<html>
  <head>
    <title>%(message)s</title>
  </head>
  <body bgcolor="#ffffff">
    <h1>%(message)s</h1>

    <p> Error <b>%(errno)d</b> generated within content provision KP.
    </p>
  </body>
</html>
'''

DEFAULT_SUGGEST_PRELUDE = """\
Please suggest new marketing slogans for inclusion in the
      <strong><em>Python Advocacy</em></strong> database.  Your suggestions 
      will be considered for inclusion."""

SLOGAN_HAS_BEEN_ADDED = """\
    <p> This slogan has been added to the database:
    </p>

    <ul>
%s    </ul>
"""
SLOGAN_HAS_BEEN_REMOVED = """\
    <p> This slogan has been removed from the list of suggestions:
    </p>

    <ul>
%s    </ul>
"""

BAD_EMAIL_ADDRESS = """The email address you provided does not appear to be
      valid, please check it carefully.  Contact the administrator if this
      system will not accept your correct address."""

ADMIN_UNKNOWN_SLOGAN = """\
    <p> Unknown slogan; perhaps another administator
      is working on the database as the same time?
    </p>
"""

NO_SUGGESTED_SLOGANS = """    <hr>
    <p> There are no suggestions in the database.
    </p>
"""
