"""Extract links from Web pages"""

import string
import re
import urlparse
import urllib

def parseSchemeHostPort(url):
    parts = urlparse.urlparse(url)
    host = parts[0]
    if host[-3:] == ':80':
	host = host[:-3]
    path = parts[2]
    while len(path) > 1 and path[1] == '/':
	path = path[1:]
    return host, string.lower(parts[1]), path

# Is this the best I can do for recognizing absolute URLs?
_rx = re.compile('^\S+:')

def isRelative(url):
    mo = _rx.match(url)
    if mo:
        return 0
    else:
        return 1

def canonicalizeURL(url):
    # XXX This is probably very expensive
    scheme, netloc, url, params, query, frag = list(urlparse.urlparse(url))
    netloc = string.lower(netloc)
    url = urllib.quote(url)
    params = urllib.quote(params)
    query = urllib.quote(query)
    return urlparse.urlunparse((scheme, netloc, url, params, query, ''))

class LinkExtractor:
    # XXX Doesn't use htmllib because it's way too slow.  Perhaps the
    # XML SIG's new C extensions will help.

    def __init__(self, url, body):
        self.body = body
        self.url = url
        self.links = None
        self.labels = {}
        self.schemeHostPort = parseSchemeHostPort(url)

    def extractLinks(self):
        if self.links:
            return self.links
        parts = filter(None,
                       map(string.strip,
                           string.split(self.body, '<')))
        self.links = links = []
        # The next 3 assignments result in a 60% speedup
        find = string.find
        join = string.join
        split = string.split
        for part in parts:
            c = part[0]
            if c == 'a' or c == 'A':
                smushed = join(split(part), '')
                i = find(smushed, 'HREF=')
                if i == -1:
                    i = find(smushed, 'href=')
                    if i == -1:
                        continue
                smushed = smushed[i+6:]
                i = find(smushed, '"')
                links.append(smushed[:i])
        return links

    def extractFullLinks(self):
        start = self.extractLinks()
        links = []
        for link in start:
            if link[:5] == 'http:':
                links.append(link)
            else:
                links.append(urlparse.urljoin(self.url, link))
        return map(canonicalizeURL, links)

    # XXX should really store the results of these operations.  if we
    # know a page is local, then we also know it isn't external.
                
    def extractLocalLinks(self):
        start = self.extractLinks()
        links = []
        for link in start:
            if isRelative(link):
                links.append(urlparse.urljoin(self.url, link))
            else:
                t = parseSchemeHostPort(link)
                if t == self.schemeHostPort:
                    links.append(link)
        return map(canonicalizeURL, links)
   
    def extractExternalLinks(self):
        start = self.extractLinks()
        links = []
        for link in start:
            if not isRelative(link):
                t = parseSchemeHostPort(link)
                if t != self.schemeHostPort:
                    links.append(link)
        return map(canonicalizeURL, links)
        
