import nstools
import random

def median(seq):
    seq.sort()
    return seq[len(seq)/2]

class KP:

    def __init__(self):
        self.graph = {}
        self.kernels = []
        self.tried = None
        self.todo = []
        self.hosts = {}
        self.ready = None

    def bootstrap(self, where):
        world = nstools.WorldOpen('kos')
        self.kernels = world.List()
        if self.tried is None:
            self.tried = self.kernels[:]
        self.todo = self.kernels[:]
        ns = self.get_service('Network.NameService')
        if not ns:
            self.tried.remove(where)
            if self.tried:
                self.kos.migrate(random.choice(self.tried))
            else:
                raise RuntimeError, "No KSSs are running the Network plugin"
        for k in self.kernels:
            self.graph[k] = {}
            for j in self.kernels:
                self.graph[k][j] = None
            self.hosts[k] = ns.GetKernelHost(k)
        self.ready = 1

    def __main__(self, kos):
        self.kos = kos
        home = kos.get_kos_name()
        if not self.ready:
            self.bootstrap(home)
        ping = self.get_service('Network.Latency')
        if ping:
            for k in self.kernels:
                if k == home:
                    self.graph[home][k] = 0.0
                else:
                    pings = ping.Ping(self.hosts[k], 8)
                    self.graph[home][k] = median(pings)
        self.todo.remove(home)
        if self.todo:
            self.kos.migrate(random.choice(self.todo))
        else:
            for k in self.kernels:
                print k, "->"
                for dest, dist in self.kernels[k].items():
                    print "\t", dest, dist

    def get_service(self, interface):
        servers = self.kos.list_services(interface)
        if servers:
            ns_desc = self.kos.lookup_service(interface, servers[0])
            return ns_desc.Open()
        else:
            return None
        
