"""A KP class for using the SimpleIO extension.

The class implements a write method and several helper functions that
allow KPs to call the write method without doing any explicit
management.

The class implements an __init__ and a __setup__.

Needs:
- should have methods for safe pickling
"""

class KP:

    def __init__(self):
        self._kos = None
        self._iof = None
        self._exc = { 'KOSKernel.NotFound' : None,
                      }

    def __setup__(self, kos):
        self._kos = kos
        self._get_exceptions()

    def __getstate__(self):
        exc_names = {}
        for exc in self._exc.keys():
            exc_names[exc] = None
        return { '_kos' : None,
                 '_iof' : None,
                 '_exc' : exc_names,
                 }

    def _get_exceptions(self):
        for exc in self._exc.keys():
            try:
                val = self._kos.exception(exc)
            except (AttributeError, ValueError):
                pass
            else:
                self._exc[exc] = val

    def _open_iof(self):
        if self._kos:
            try:
                conndesc = self._kos.lookup_service \
                           ('SimpleIOAPI.SimpleIOFactory',
                           'simpleio')
            except self._exc['KOSKernel.NotFound']:
                return
            iofactory = conndesc.Open()
            self._iof = iofactory.GetNewSimpleIO()
            
    def write(self, str):
        if not self._iof:
            self._open_iof()
        if self._iof:
            self._iof.write("%s\n" % (str))
