"""Administrative interface for the KOS.

The whole interface seems to be a bit kludgy. Hence, I'm adding my
interface for inquiring about processes being run by a kernel here.

"""

import os
from koe.interfaces.stubs import AdministratorAPI, AdministratorAPI__skel
from koe.interfaces.stubs import ReportingAPI, ReportingAPI__skel
from koe.interfaces.stubs import KOS
from koe.common import RS
from koe.common import nstools
from koe.common import access

MAX_VISUALIZERS = 12

class Administrator(AdministratorAPI__skel.Administrator):

    def __init__(self, kos):
        self.kos = kos
        self.kos_name = kos.name
        self.reporter = Visualizer(())
        self.sbh_cache = {}
        self.id_cache = {}
        self.kos_pid = os.getpid()

    def get_reporter(self):
        return self.reporter

    def bindnames(self, kosns):
        desc = nstools.Descriptor(typename='AdministratorAPI.Administrator',
                                  reference=self.IluSBH())
        kosns.Bind('administrator', desc)

    def withdraw_kos(self):
        for sbh, watch in self.sbh_cache.items():
            self.reporter.remove_station(sbh)

    def GetProcessInfo(self):
        """Return list of records describing processes run by KOS.

        This is a first-cut, perhaps kludgy interface. Want to try
        using it and see what happens.

        TYPES = kernel, kpstarter, plugin, kp
        """
        ProcessInfo = AdministratorAPI.ProcessInfo
        KOSProcessType = AdministratorAPI.KOSProcessType

        kernel = ProcessInfo(KOSProcessType.kos, self.kos_name,
                             self.kos_pid, "", "") 
        procs = [kernel]
        if self.kos.receiver.kpstarter_pid:
            starter = ProcessInfo(KOSProcessType.kp_starter,
                                  self.kos_name,
                                  self.kos.receiver.kpstarter_pid, "",
                                  "")
            procs.append(starter)
        # do the plugins
        for pid, entry in self.kos.receiver.ext_entries.items():
            # no info here
            info = ProcessInfo(KOSProcessType.extension, entry.name,
                               pid, entry.kpid, ""); 
            procs.append(info)
        # do the KPs
        for pid, entry in self.kos.receiver.kp_entries.items():
            info = ProcessInfo(KOSProcessType.kp, entry.reporting_id,
                               pid, entry.kpid, "") 
            procs.append(info)

        return procs

    lastid = 0
    def RegisterVisualizer(self, watch):
        if self.reporter.get_station_count() >= MAX_VISUALIZERS:
            raise KOS.ResourceError, "too many visualizers active"
        sbh = watch.IluSBH()
        try:
            xx = self.sbh_cache[sbh]
        except KeyError:
            self.sbh_cache[sbh] = watch
        else:
            raise KOS.AuthorizationError, "Visualizer must register only once."
        id = self.lastid + 1
        self.lastid = id
        id = watch._visualizer_id = `id`
        self.id_cache[id] = sbh
        self.reporter.add_station(watch, Deleter(self.tossit, id))
        return id

    def UnregisterVisualizer(self, id):
        if self.id_cache.has_key(id):
            self.reporter.remove_station(self.id_cache[id])
            # reporter will call the deletion callback  -->

    def Shutdown(self):

        # Insure requestor is authorized
        try:
            if access.ilu_allow_access('kosshutdown'):
                self.kos.shutdown()
        except access.ResourceError, msg:
            print 'Error obtaining access information:', msg

    def tossit(self, id):
        sbh = self.id_cache[id]
        del self.sbh_cache[sbh]
        del self.id_cache[id]


class Visualizer(ReportingAPI__skel.Visualizer, RS.MulticastRS):
    pass


class Deleter:
    def __init__(self, func, id):
        self.func = func
        self.id = id

    def __call__(self):
        self.func(self.id)
        del self.func
