"""KOS kernel communications link with a KOS subprocess."""

verbose = 0

import sys

from koe.interfaces.stubs import KOS
from koe.interfaces.stubs import KOSKernel__skel, KOSKernel
import CnManager
    
# Common counter provides kpids distinct over plugins *and* KPs.
process_counter = 0

class ProcessEntry(KOSKernel__skel.ProcessEntry):
    """KOS kernel communications link with a KOS subprocess."""

    proctype = "Other"

    def __init__(self, kernel):

        global process_counter
        process_counter = process_counter + 1

        self.kernel = kernel
        self.kpid = `process_counter`           # Kernel process id
        
        self.pid = None                 # Underlying-OS process id
        self.processlink = None         # RPC link back to Supervisor

        self._Note('instantiating...')

    def SetPID(self, pid):
        """Register underlying-OS process id."""
        self._Note("Underlying OS process started, pid %d", pid)
        self.pid = pid

    def EstablishProcessLink(self, processlink):
        "KP has created supervisor object; passes link for KOS to use."

        # TBD: There are two different places we could
        # store the mapping from pid to process link. It
        # could be stored as part of the process entry or
        # it could be stored in the kernel (or
        # Receiver). If we store it in the process entry,
        # then we need to get the process entry from the
        # receiver and use it to get the process link.

        # The whole implementation needs to be cleaned up
        # so I'll use the kernel and expect some changes.

        self.processlink = processlink
        self.kernel.processlinks[self.kpid] = processlink

    # These methods provide Connection Manager services.

    def RegisterConnector(self, instance, type, name):
        """Create an entry for a connector."""
        self.kernel.RegisterConnector(self.kpid, instance, type, name)

    def UnregisterConnector(self, instance):
        """Remove an entry for a connector."""
        self.kernel.UnregisterConnector(self.kpid, instance)

    def LookupService(self, type, name):
        """Find a registered connector."""
        return self.kernel.LookupService(self.kpid, type, name)

    def ListServices(self, type):
        """Find all registered connectors of specific type."""
        return self.kernel.ListServices(self.kpid, type)

    def ListServiceTypes(self):
        """Return the type names used by any connectors."""
        return self.kernel.ListServiceTypes(self.kpid)

    # These methods serve as an interface between a user process
    # (specifically, its CnBroker) and the kernel's TriggerManager.

    # The kernel methods may raise exceptions specified in the ISL.

    def CreateCondition(self):
        var = self.kernel.CreateCondition()
        return var

    def Signal(self, var):
        self.kernel.Signal(var)

    def Broadcast(self, var):
        self.kernel.Broadcast(var)

    def RegisterTrigger(self, var):
        self.kernel.RegisterTrigger(var, self.kpid)

    def UnregisterTrigger(self, var):
        self.kernel.UnregisterTrigger(var, self.kpid)

    # Debugging support methods.

    def _Note(self, msg, *args):
        """Print MSG (with optional ARGS) to stderr iff verbose operation."""
        # Note that _Note depends on self.kpid having a setting.
        if verbose:
            sys.stderr.write("(%s Process %s: %s)\n" %
                             (self.proctype, self.kpid, msg % args))

    def _ForceNote(self, msg, *args):
        """Unconditionally print MSG (with optional ARGS) to stderr."""
        global verbose
        was_verbose, verbose = verbose, 1
        apply(self._Note, (msg,)+args)
        verbose = was_verbose

    def _Where(self, msg, *args):
        """Print MSG (with optional ARGS) to stderr and append stack trace."""
        apply(self._ForceNote, (msg,)+args)
        import traceback
        sys.stderr.write("--------------------STACK--------------------\n")
        traceback.print_stack()
        sys.stderr.write("---------------------------------------------\n")
