"""Utilities assisting in KOS plugin bootup."""

import os
from koe.pythonlib import ConfigParser            # Until Python 1.5.2
from koe.common import pathhack

PLUGINDIR = "plugins"                   # Relative to kos root.
SETUPDIR = "config"                     # Relative to kos root.
SETUPFN = "plugins"
LOCALSETUPFN = "plugins.local"



class PluginOpts(ConfigParser.ConfigParser):
    """Load plugin setup options from setup files.

    The setup files all reside in the plugins directory.  All
    applicable setup files are read; values set in later files
    override those set in earlier files.  The files are, in the order
    in which they are read:

        plugins -- complete setup from the distribution
        plugins.local -- local overrides
        plugins.<kernelname> -- overrides specific to one kernel

    In order to enable or disable a particular plugin in the local or
    kernel specific setup file, only the run-at-boot flag needs to be
    specified for that plugin in the specific setup file.


    """

    def __init__(self, kosname, kosroot=None):
        kosroot = kosroot or pathhack.kos_root()
        plugindir = os.path.join(kosroot, PLUGINDIR)
        setupdir = os.path.join(kosroot, SETUPDIR)
        setuppath = [os.path.join(setupdir, SETUPFN),
                     os.path.join(setupdir, LOCALSETUPFN),
                     ]
        if kosname:
            setuppath.append(os.path.join(setupdir, SETUPFN + '.%s' % kosname))

        # A few intrinsic values for DEFAULT section
        defaults = {'kosroot': kosroot,
                    'root'   : plugindir,
                    'kosname': kosname,
                    }

        ConfigParser.ConfigParser.__init__(self, defaults)
        self.read(setuppath)
