"""Implementation of a mapping between file-like Python objects and
the File interface in File.isl.
"""

from koe.interfaces.stubs import FileAPI, FileAPI__skel



class FileWrapper(FileAPI__skel.File):
    """Wrapper around a specific file opener.

    Instantiate with a function that can be called by passing in only
    the opening mode, and which returns a file-like object.
    """
    def __init__(self, openfunc):
        self._openfunc = openfunc
        self._fp = None

    def open(self, mode):
        try:
            self._fp = self._openfunc(mode)
        except IOError:
            raise FileAPI.FileIOError

    def close(self):
        if self._fp:
            try: self._fp.close()
            except IOError: raise FileAPI.FileIOError
            self._fp = None
        else:
            raise FileAPI.FileIOError

    def flush(self):
        if self._fp:
            try: self._fp.flush()
            except IOError: raise FileAPI.FileIOError
        else:
            raise FileAPI.FileIOError

    def read(self, size):
        if self._fp:
            try: return self._fp.read(size)
            except IOError: raise FileAPI.FileIOError
        raise FileAPI.FileIOError

    def readline(self):
        if self._fp:
            try: return self._fp.readline()
            except IOError: raise FileAPI.FileIOError
        raise FileAPI.FileIOError

    def readlines(self):
        if self._fp:
            try: return self._fp.readlines()
            except IOError: raise FileAPI.FileIOError
        raise FileAPI.FileIOError

    def seek(self, offset, whence=0):
        if self._fp:
            try: self._fp.seek(offset, whence)
            except IOError: raise FileAPI.FileIOError
        else:
            raise FileAPI.FileIOError

    def tell(self):
        if self._fp:
            try: return self._fp.tell()
            except IOError: raise FileAPI.FileIOError
        raise FileAPI.FileIOError

    def write(self, text):
        if self._fp:
            try: self._fp.write(text)
            except IOError: raise FileAPI.FileIOError
        else:
            raise FileAPI.FileIOError

    def writelines(self, lines):
        if self._fp:
            try: self._fp.writelines(lines)
            except IOError: raise FileAPI.FileIOError
        else:
            raise FileAPI.FileIOError
