#! /usr/bin/env python
    
import os
import sys

if __name__ == '__main__':
    import ilu_tk
    # Munge the path if we're an application
    here = os.path.split(sys.argv[0])[0]
    # koe package directory.
    sys.path.append(os.path.join(here, '../..'))
    # for FontLockText.py
    sys.path.append(os.path.join(here, '../../../nondist/src/demos/medline'))

import time
import marshal
import urllib
import urlparse
import string
import socket
import random
from Tkinter import *
from koe.common import kptools
from koe.common import nstools
from koe.common import tktools
from koe.common import physfonts
from koe.common import RS
from koe.tools.visu import Controller
from koe.interfaces.stubs import KOS

DEFAULTRIGGER = "TEST_TRIGGER"
SERVICES = {}

def fetch_url(url):
    """Return a string object with the lines of the file minus the
       HTTP headers
    """
    try:
        f = urllib.urlopen(url)
    except IOError, v:
        # try the url without the leading path components
        urlparts = urlparse.urlparse(url)
        shorturl = urlparts[0] + ':'
        if urlparts[1]:
            shorturl = shorturl + '//' + urlparts[1] + '/'
        shorturl = shorturl + os.path.basename(urlparts[2])
        try:
            f = urllib.urlopen(shorturl)
        except IOError, v2:
            raise "Tried: %s: %s\n  and: %s: %s" % (url, v[1], shorturl, v2[1])
    # Handle potential Python 1.4 httplib - Apache incompatability
    lines = []
    line = f.readline()
    while line:
        if line == 'Content-Type: text/plain\015\012':
            f.readline() # last ctrl-M
            lines = []
        else:
            lines.append(line)
        line = f.readline()
    return string.join(lines, '')

class KPCreater:
    """Fetch and maintain the source code for a module that is all or
       part of a KP
    """
    def __init__(self, name=None, url=None, read_only=0):
        self.url = url
        if not name:
            raise TypeError, "Must specify a name"
        self.name = name
        self.read_only = read_only
        self.kp_src = ''
        self.var_dict = {}

    def add_var(self, name, value, type=None):
        """Add a new variable to the variable dictionary"""
        if type == 'string':
            value = '"' + str(value) + '"'
        self.var_dict[name] = value

    def fetch_src(self):
        """get the source corresponding to the URL"""
  
        if self.url:
            self.kp_src = fetch_url(self.url)
            
    def src(self, add_vars=0):
        """generate the source for the KP"""
        kp_src = ''
        if add_vars:
            kp_src = self.var_src()
        if len(self.kp_src) == 0:
            self.fetch_src()
        kp_src = kp_src + self.kp_src
        return kp_src

    def var_src(self):
        """Fetch just the added variables as a string."""
        var_src = ''
        for var in self.var_dict.keys():
            var_src = var_src + var + ' = ' + str(self.var_dict[var]) + '\n'
        return var_src

class KPLauncherGUI:
    """
    Basic usage:
    
        l = KPLauncher.KPLauncherGUI(master_tk_frame, self, base_url_dir)
        l.create_launch_panel()
        l.start_launch_panel()
        l.reset_kp(base_url_dir, shortname, filename)

    """
    
    def __init__(self, master, baseurldir, font=None, delegate=None,
                 rsmodule=sys.modules[__name__], rsname='MyRS',
                 main_mod_name=None, filenames=None, klass='KP',
                 suitcasefiles=None, addquit=0, host=None):
        """Create a top-level widget to act as a launching platform for KPs"""
        self.master = master
        self.delegate = delegate
        self.baseurldir = baseurldir
        self.screenheight = self.master.winfo_screenheight()
        self.screenwidth = self.master.winfo_screenwidth()
        if font is None:
            if self.screenheight < 1000: # adjust font for smaller screens
                self.font = physfonts.BODY_B_M
            else:
                self.font = physfonts.BODY_B_L
        else:
            self.font = font
        self.rsname = rsname
        self.rsmodule = rsmodule
        self.filenames = filenames
        self.suitcasefiles = suitcasefiles
        self.klass = klass
        self.addquit = addquit
        self.main_mod_name = self.get_main_mod_name(main_mod_name)
        if host is not None:
            self.host = host
        else:
            self.host = self.getmyhost()
            
    def get_main_mod_name(self, main_mod_name):
        """Determine the main module name for the current KP"""
        
        if main_mod_name is None and self.filenames is not None:
            if len(self.filenames) > 1:
                raise TypeError, "Too many KP files, must specify main module"
            # If one file and main_module unspecified, assume they are the same
            return os.path.basename(os.path.splitext(self.filenames[0])[0])

        return main_mod_name
        
    def create_launch_panel(self):
        """Create all the controls for the Toplevel widget by calling
        create_widgets, then it initializes the startup values for them."""

        self.kp_modules = self.create_modules()
        self.create_widgets()
               
        # Create Applet reporting station for Status Buffer
        klass = getattr(self.rsmodule, self.rsname)
        self.rs = klass(self, self.delegate, self.kp_text_status)

        # Create a local IO Server for Status Buffer
        self.kpio = MyIO(self.kp_text_status)
        self.ioserver = kptools.IOServer(w=self.kpio)
        
        # Set inital state for the widgets
        self.kp_kss_entry.delete(0, END)
        self.kp_kss_entry.insert(0, self.host)
        self.edit_var.set('None')
        self.display_kp()
        
        # Create a visualizer
        quiet = 0
        self.visu = Controller.Controller(None, quiet, width=750,
                                          height=self.screenheight/1.6,
                                          master=self.kp_visu_frame)

    def getmyhost(self):
        return socket.gethostname()

    def create_modules(self):
        if self.filenames is None or self.baseurldir is None:
            return []

        kpmodules = []
        for filename in self.filenames:
            url = urllib.basejoin(self.baseurldir, filename)
            name = os.path.splitext(os.path.basename(filename))[0]
            kp = KPCreater(name=name, url=url)
            kpmodules.append(kp) 
        return kpmodules

    def reset_kp(self, baseurldir, filenames=None, main_mod_name=None,
                 suitcasefiles=None):
        """Reset the Knowbot Program related ivars"""

        self.baseurldir = baseurldir
        self.filenames = filenames
        self.suitcasefiles = suitcasefiles
        self.main_mod_name = self.get_main_mod_name(main_mod_name)
        self.kp_modules = self.create_modules()
        self.configure_edit_menu()
        
    def start_launch_panel(self, deiconify=0):

        self.update_vars_on_deiconify()
        self.update_kp()

        # Start a visualizer, but don't view it by default
        self.visu.start()
        self.view_neither_window()
        if deiconify:
            self.deiconify()
        else:
            self.iconify()

    def deiconify(self):
        self.kp_panel.deiconify()

    def iconify(self):
        self.kp_panel.iconify()

    def update_vars_on_deiconify(self):
        pass

    def cleanup(self):
        self.visu.cleanup()
    
    def create_widgets(self):
        """Create all the controls for the Toplevel widget."""
        
        self.kp_panel = tktools.make_toplevel(self.master,
                         title='Knowbot Program Launcher')
        self.kp_panel.iconify()
        self.kp_frame = Frame(self.kp_panel)

        # KP class edit buffer
        self.kp_edit_frame = Frame(self.kp_frame)
        self.kp_edit_label = Label(self.kp_edit_frame, text='Edit Buffer',
                                      font=self.font)
        self.kp_text_frame = Frame(self.kp_edit_frame, relief=SUNKEN)

        # Try and use FontLockText for Python syntax coloring
        try:
            import FontLockText
            self.kp_text, self.kp_text_box_frame = \
                          self.make_text_box(self.kp_text_frame, hbar=1,
                          width=100, height=self.screenheight/30, wrap=NONE)
        except ImportError:
            self.kp_text, self.kp_text_box_frame = \
                          tktools.make_text_box(self.kp_text_frame, hbar=1,
                          width=100, height=self.screenheight/30, wrap=NONE)
        
        # Status reporting or visualizer
        self.reporting_var = StringVar()
        self.top_menu_frame = Frame(self.kp_frame)
        self.clear_status_button = Button(self.top_menu_frame, text='Clear',
                                   command=self.clear_status, font=self.font)
        self.reporting_menubutton = Menubutton(self.top_menu_frame,
                                               relief=RAISED,
                                               indicator=1,
                                               font=self.font)
        self.reporting_menu = Menu(self.reporting_menubutton, tearoff=0,
                                   font=self.font)
        self.reporting_menubutton['menu'] = self.reporting_menu
        self.reporting_menubutton.pack(side=LEFT)
        self.reporting_menu.add_command(label='Status Buffer',
                         command=self.view_status_window)
        self.reporting_menu.add_command(label='Visualizer',
                         command=self.view_visu_window)
        self.reporting_menu.add_command(label='No Reporting',
                         command=self.view_neither_window)
        self.kp_reporting_frame = Frame(self.kp_frame)
        self.kp_reporting_frame.pack(side=TOP, fill=BOTH, expand=1)
        self.kp_status_frame = Frame(self.kp_reporting_frame)
        self.kp_text_status, self.kp_text_frame_status = \
                         tktools.make_text_box(self.kp_status_frame,
                         height=self.screenheight/30, width=100,
                         font=self.font)
        
        self.kp_visu_frame = Frame(self.kp_reporting_frame)
        
        #
        # Controls
        self.kp_bottom_frame = Frame(self.kp_frame)
        self.kp_kss_frame = Frame(self.kp_frame, border=1, relief=SUNKEN)
        self.kp_kss_rframe = Frame(self.kp_kss_frame, border=1, relief=RAISED)
        self.kp_kss_lframe = Frame(self.kp_kss_frame, relief=RAISED)
        self.kp_bottom_button_frame = Frame(self.kp_frame)

        # KSS
        self.kp_kss_entry = Entry(self.kp_kss_rframe,
                                  text=self.host, font=self.font)
        self.kp_kss_label = Label(self.kp_kss_lframe,
                         text='    My Knowbot Service Station:',
                         font=self.font)
        # Edit
        self.edit_label = Label(self.kp_kss_lframe, text='Edit Module:',
                         font=self.font)
        self.edit_button_frame = Frame(self.kp_kss_rframe)
        self.edit_var = StringVar()
        self.edit_menubutton = Menubutton(self.edit_button_frame,
                                          relief=RAISED, indicator=1)
        self.suit_case_label = Label(self.edit_button_frame, font=self.font)
        self.edit_menu = Menu(self.edit_menubutton, tearoff=0, font=self.font)
        self.edit_menubutton['menu'] = self.edit_menu      

        # Edit font
        self.font_label = Label(self.kp_kss_lframe, text='Edit Font Size:',
                         font=self.font)
        self.font_button_frame = Frame(self.kp_kss_rframe)
        self.font_menubutton = Menubutton(self.font_button_frame,
                                          text='medium', font=self.font,
                                          relief=RAISED, indicator=1)
        self.font_menu = Menu(self.font_menubutton, tearoff=0,
                              font=self.font)
        self.font_menubutton['menu'] = self.font_menu
        self.edit_width = self.kp_text['width']
        self.edit_height = self.kp_text['height']

        
        # TT_B_S = "-*-courier-bold-r-normal-*-*-100-*-*-*-*-*-*"
        # TT_B_M = "-*-courier-bold-r-normal-*-*-120-*-*-*-*-*-*"
        # TT_B_L = "-*-courier-bold-r-normal-*-*-140-*-*-*-*-*-*"
        # TT_B_XL = "-*-courier-bold-r-normal-*-*-180-*-*-*-*-*-*"
        for font in [('small', physfonts.TT_B_S, 1.0, 0.9),
                     ('medium', physfonts.TT_B_M, 1.0, 0.9),
                     ('large', physfonts.TT_B_L, 0.9, 0.9),
                     ('jumbo', physfonts.TT_B_XL, 0.7, 0.9)]:
            def _font_command(self=self, font=font):
                self.kp_text.config(font=font[1])
                newwidth = int(float(self.edit_width) * font[2])
                newheight = int(float(self.edit_height) * font[3])
                self.kp_text.config(width=newwidth, height=newheight)
                self.font_menubutton.config(text=font[0])
            self.font_menu.add_command(label=font[0], command=_font_command)
            if self.font_menubutton['text'] == font[0]:
                _font_command()
            
        # Bottom row buttons
        if self.addquit:
            self.kp_close_button = Button(self.kp_bottom_button_frame,
                                      text='Exit',
                                      command=sys.exit,
                                      font=self.font)
        else:
            self.kp_close_button = Button(self.kp_bottom_button_frame,
                                      text='Close',
                                      command=self.launch_panel_close,
                                      font=self.font)
            
        self.kp_submit_button = Button(self.kp_bottom_button_frame,
                                      text='Submit KP',
                                      command=self.launch_panel_submit,
                                      font=self.font)
       
        # pack em up
        self.kp_frame.pack(fill=BOTH, expand=1)
        self.top_menu_frame.pack(side=TOP, fill=NONE, expand=0)
        self.kp_bottom_button_frame.pack(side=BOTTOM, expand=0)
        self.kp_edit_frame.pack(expand=1, side=BOTTOM, fill=BOTH)
        self.kp_edit_label.pack(side=TOP)
        self.kp_text.pack(side=BOTTOM, fill=BOTH, expand=1)
        self.kp_bottom_frame.pack(side=BOTTOM, fill=NONE, expand=0)
        self.kp_kss_frame.pack(side=BOTTOM, fill=NONE, expand=0)
        self.kp_kss_lframe.pack(side=LEFT, fill=NONE, anchor=E, expand=0)
        self.kp_kss_rframe.pack(side=RIGHT, expand=0, fill=NONE, anchor=W)
        self.kp_kss_label.pack(anchor=E)
        self.kp_kss_entry.pack(anchor=W, expand=0, fill=NONE)
        self.edit_label.pack(anchor=E)
        self.edit_button_frame.pack(anchor=W, expand=1)
        self.font_menubutton.pack(side=LEFT)
        self.suit_case_label.pack(side=RIGHT)
        self.edit_menubutton.pack()
        self.kp_close_button.pack(side=RIGHT, anchor=W)
        self.kp_submit_button.pack(side=LEFT, anchor=E)

        self.add_extra_controls()
        self.configure_edit_menu()

    def add_extra_controls(self):
        pass

    def clear_status(self, event=None):
        """Clear the status Text widget"""
        self.kp_text_status.delete("0.0", END)
        
    def configure_edit_menu(self):
        """Reconfigure the edit menu and menubutton"""

        # delete the old menu
        self.edit_menu.delete(0, index2=len(self.kp_modules))
        
        # add a selection for no editing
        self.edit_menu.add_command(label='None', command=self.no_edit)
        self.edit_menubutton.config(text="None", font=self.font)

        # If there are no modules to be loaded, create an empty workspace
        if len(self.kp_modules) == 0:
            self.kp_modules.append(KPCreater(name='New KP'))
            self.main_mod_name = 'New KP'

        # add a selection for each module
        for module in self.kp_modules:

            # try *this* in C...
            def _edit_command(self=self, module=module):
                # update the kp object with the source from the text field
                # in case the user edited the buffer
                
                #self.update_kp()
                self.edit_var.set(module.name)
                return self.display_kp()
            
            if module.read_only:
                menu_text = module.name + ' (RO)'
            else:
                menu_text = module.name
           
            self.edit_menu.add_command(label=menu_text, command=_edit_command)

        # If we were previously editing and we just reset, display new KP 
        if self.edit_var.get() <> "None":
            name = self.kp_modules[0].name
            self.edit_var.set(name)
            self.edit_menubutton.config(text=name, font=self.font)
            _edit_command(self=self, module=self.kp_modules[0])
        else:
            self.edit_var.set("None")
            self.view_neither_window()

    def make_text_box(self, parent, width=0, height=0, hbar=0, vbar=1,
                  fill=BOTH, expand=1, wrap=WORD, pack=1, font=None,
                  class_=None, name=None, takefocus=None):

        """Subroutine to create a Font locking/syntax colored Text box.
           Adapted from tk_tools.make_text_box

        Create:
        - a both-ways filling and expanding frame, containing:
            - a text widget on the left, and
            - possibly a vertical scroll bar on the right.
            - possibly a horizonta; scroll bar at the bottom.

        Return the text widget and the frame widget.

        """
        hbar, vbar, frame = tktools.make_scrollbars(parent,
                                            hbar, vbar, pack,
                                            class_=class_, name=name,
                                            takefocus=takefocus)
        import FontLockText
        widget = FontLockText.FontLockText(frame, wrap=wrap,
                                           name="fontLockText", font=font)
        if width: widget.config(width=width)
        if height: widget.config(height=height)
        widget.pack(expand=expand, fill=fill, side=LEFT)

        tktools.set_scroll_commands(widget.text, hbar, vbar)

        return widget, frame

    def view_status_window(self):
        """Callback to show the Status window view with the line oriented
        reporting station and the IOServer output"""
        self.kp_visu_frame.forget()
        self.kp_status_frame.pack(fill=BOTH, expand=1)
        self.reporting_menubutton.config(text='Status Buffer',
                         font=self.font)
        self.kp_reporting_frame.pack(side=TOP, fill=BOTH, expand=1)
        self.clear_status_button.pack(side=RIGHT)
        
        # Hide the edit window if it's visible to make room
        self.edit_var.set('None')
        self.display_kp()
        
    def view_visu_window(self):
        """Callback to show the visualizer window"""
        self.kp_status_frame.forget()
        self.kp_visu_frame.pack(fill=BOTH, expand=1)
        self.reporting_menubutton.config(text='Visualizer',
                         font=self.font)
        self.kp_reporting_frame.pack(side=TOP, fill=BOTH, expand=1)
        self.clear_status_button.forget()

        # Hide the edit window if it's visible to make room
        self.edit_var.set('None')
        self.display_kp()
        
    def view_neither_window(self):
        """Callback to disable the viewing of any status windows"""
        self.kp_reporting_frame.forget()
        self.clear_status_button.forget()
        self.reporting_menubutton.config(text='No Reporting',
                         font=self.font)

    def get_module(self, name):
        """Return the KPCreater instance corresponding to name"""
        for module in self.kp_modules:
            if module.name == name:
                return module
        raise AttributeError, "No module named: %s" % name
            
                
    def display_kp(self):
        """Callback for the edit radio buttons"""
        module_name = self.edit_var.get()
        if not module_name: return
        if module_name == 'None':
            self.kp_text_frame.forget()
            self.kp_edit_frame.forget()
            self.font_label.forget()
            self.font_button_frame.forget()
            self.edit_menubutton.config(text="None")
            self.suit_case_label.config(text="")
            return
        
        module = self.get_module(module_name)
        if module.read_only:
            menu_text = module_name + ' (RO)'
            text = 'Edit Buffer: ' + module_name + ' (Read Only)'
            state = DISABLED
        else:
            menu_text = module_name
            text = 'Edit Buffer: ' + module_name
            state = NORMAL
        self.edit_menubutton.config(text=module_name)
        self.set_text(self.kp_text, module.src(add_vars=1), state=state)
        self.kp_edit_label.config(text=text)
        self.kp_text_frame.pack(expand=1, fill=BOTH)
        self.kp_edit_frame.pack(expand=1, side=BOTTOM, fill=BOTH)
        self.font_label.pack(anchor=E)
        self.font_button_frame.pack(anchor=W, expand=1)
        if self.suitcasefiles is None:
            self.suit_case_label.config(text="")
        else:
            txt = "Suitcase: "
            for scf in self.suitcasefiles:
                txt = txt + scf['scname'] + " "
            self.suit_case_label.config(text=txt)
            
        # Hide the Visualizer or Status buffer to make room
        self.view_neither_window()
        
    def no_edit(self):
        """Callback for the edit mode where no editing is desired"""
        self.edit_var.set("None")
        self.display_kp()
      
    def set_text(self, text, stuff, state=NORMAL):
        """Insert stuff in text leaving text state in accordance with state."""
        text.config(state=NORMAL)
        text.delete('1.0', END)
        text.insert(END, stuff)
        text.config(state=state)
        
    def get_kp(self, add_vars=1):
        """Get the initial set of KP modules (from the URL with added
        variables if add_vars is set) associated with this search.
        The result is a dictionary of the form {'modulename' : 'contents'}.
        """
        kp = {}
        for kpcreater in self.kp_modules:
            kp[kpcreater.name] = kpcreater.src(add_vars=add_vars)
        return kp

    def update_kp(self):
        """Update current kpcreater object with the contents of the text widget
        """
        try:
            module_name = self.edit_var.get()
        except AttributeError:
            return
        if module_name == 'None':
            return

        curr_module = self.get_module(module_name)
        if not curr_module.read_only:
            curr_module.kp_src = self.kp_text.get("0.0", END)
        
    def display_log(self, log):
        """Display the contents of the log file in the logfile text field"""
        self.kp_text_status.delete('1.0', END)
        self.kp_text_status.insert(END, log)
        self.kp_text_status.update_idletasks()
 
    def launch_panel_close(self):
        """Callback for the knowbot panel close button.  This method
           only iconifies the Knowbot launchpanel"""
        self.kp_panel.iconify()

    def launch_panel_submit(self):
        """Submit the KP to the specified KOS"""
            
        self.get_ready_for_submit()
            
        # Update the kp object with the source from the text field
        # in case the user edited the buffer and to update the
        # variable just added.  TODO: Develop a
        # framework for updating and maintaining persistently between
        # sessions, any updated code.  Currently, changes only
        # remain in effect in the applets memory, not on disk.
        self.update_kp()
        
        kp = self.get_kp()
        if not len(kp):
            self.display_log("Error - Knowbot Program Not Specified")
            self.set_cursor()
            return
        
        self.kprepr = kptools.Representation()
        self.kprepr.add_main_module(self.main_mod_name, kp[self.main_mod_name],
                              klass=self.klass)
        if len(kp.keys()) > 1:
            for module_name in kp.keys():
                if module_name != self.main_mod_name:
                    self.kprepr.add_module(module_name, kp[module_name])
                    
        self.kpid = self.main_mod_name + '.' + str(random.random())

        # Add out I/O server to the KP
        self.ioserver.linktokp(self.kprepr)
        
        try:
            kss = self.kp_kss_entry.get()
            self.host = kss
        except AttributeError:
            kss = self.host

        # fetch suitcase files (if there are any) and add to Representation
        # self.sutcasefiles is either None or a list of dictionaries whos
        # keys are 'scname' (for the name the file should have in the
        # suitcase filesystem) and 'filename' (the name the file has
        # as the last component of the URL which we use to fetch it
        if self.suitcasefiles is not None:
            for scf in self.suitcasefiles:

                # Look in the directory that the modules live in and
                # the 'current' directory for the the suitcase file
                url = urllib.basejoin(self.baseurldir, scf['filename'])
                lpath = urllib.basejoin(self.filenames[0], scf['filename'])
                url = urllib.basejoin(self.baseurldir, lpath)
                sc_file = fetch_url(url)
                self.kprepr.add_file(scf['scname'], string=sc_file)

        # Switch to visualizer if we're currently editing
        if self.edit_var.get() <> 'None':
            self.view_visu_window()
            time.sleep(1)

        # While the KP is alive, change the submit button to be a kill button
        self.kp_submit_button.config(command=self.killkp, text='Kill KP')
        
        # Submittal with a reporting station
        #self.rs.message(self.host, 'Submiting KP "%s" to kss: "%s"' \
        #                % (self.kpid, kss), foreground='Green')
        try:    
            self.kprepr.submit(kosname=kss, reporter=self.rs, repid=self.kpid)
        except nstools.BadPathError:
            self.rs.message(self.host, 
                  "Error - There does not appear to be a KOS on host: %s" \
                            % kss, foreground='Red')
            

    def killkp(self):
        """Find the KP corresponding to self.kpid in the namespace
           and ask the hosting kernel to kill it"""
        koslayer = nstools.WorldOpen('kos')
        kernels = koslayer.List()
        for kernel in kernels:
            path = "%s/kps/%s" % (kernel, self.kpid)
            # print 'Trying: ', path
            try: 
                nstools.Lookup(koslayer, path)
            except nstools.BadPathError:
                continue
            receiver = nstools.WorldOpen("kos/%s/receiver" % kernel)
            self.rs.message(self.host,
                           "Killing '%s' on kernel '%s'" % (self.kpid, kernel),
                            foreground='Green')
            try:
                    receiver.KillKP(self.kpid, 15, "HaHaHa") # SIGTERM
            except KOS.AuthorizationError, msg:
                self.rs.message(self.host, "Permission Denied: %s" % msg,
                                foreground='Red')
            except KOS.ResourceError, msg:
                self.rs.message(self.host, "Resource Error: %s" % msg,
                                foreground='Red')
            except:
                self.rs.message(self.host, "Unexpected Error:%s:%s" \
                                % (str(sys.exc_type), sys.exc_value),
                                foreground='Red')
            # Reset the submit button directly even tho it will
            # eventually be done via the reporting station
            self.kp_submit_button.config(text='Submit KP',
                              command=self.launch_panel_submit) 
            return
        self.rs.message(self.host,
                        "Could not find KP '%s' in '%s'" \
                        % (self.kpid, str(kernels)), foreground='Red')
        self.kp_submit_button.config(text='Submit KP',
                              command=self.launch_panel_submit)
        
    def get_ready_for_submit(self):
        pass
    
    def set_cursor(self, cursor=''):
        """ set the cursor, default is the default Tk cursor."""
        self.kp_frame.config(cursor=cursor)
        self.kp_label_status.config(cursor=cursor)
        self.kp_text_status.config(cursor=cursor)
        self.kp_text.config(cursor=cursor)
        self.kp_frame.update_idletasks()

class MyRS(RS.NullRS):

    def __init__(self, launch_panel, delegate, rs_text):
        """The delegate object is the Controller class is expected to
        respond to a call to search_controller which operates on
        a list of dictionarys placed in delegate.recs"""
        RS.NullRS.__init__(self)
        self.launch_panel = launch_panel
        self.delegate = delegate
        self.rs_text = rs_text
        rs_text.tag_config('Red', foreground='Red')
        rs_text.tag_config('Blue', foreground='Blue')
        rs_text.tag_config('Green', foreground='#0a0')
        rs_text.tag_config('Black', foreground='Black')
        self.ids = {}

    def KPTerminated(self, id, reason, details, kprepr):
        """Cleanup and reconigure for the next KP launch"""
        self.launch_panel.kp_submit_button.config(text='Submit KP',
                              command=self.launch_panel.launch_panel_submit)
        self.message(id, "KPTerminated", foreground='Blue')
        if reason:
            from koe.interfaces.stubs import KPRepr
            state = kprepr.GetState()
            try:
                trace = state.GetData("traceback")
            except KPRepr.PartNameError:
                trace = None
            self.exception(id, reason, details, trace)
            msg = "terminated by %s exception" % reason
            foreground = 'Red'
        else:
            msg = "exited with status %s" % details
            foreground = 'Blue'
        self.message(id, msg, foreground=foreground)
        if not reason:
            self.process_results(id, reason, details, kprepr)
            
    def process_results(self, id, reason, details, kprepr):
        pass
    
    def KPMigrationReceived(self, id, idTo, migratingTo):
        self.message(id, "KPMigrationReceived, migrating to: %s" % migratingTo,
                     foreground='Blue')

    def exception(self, id, reason, details, trace=None):
        self.message(id, "Exception Occurred:", foreground='Red')
        if trace:
            self.message(id, "Traceback (innermost last):\n" + trace,
                         foreground='Red')
        self.message(id, "%s: %s" % (reason, details), foreground='Red')
        self.process_exception(reason)

    def process_exception(self, reason):
        pass
    
    def AllocateID(self, prefix):
        id = RS.NullRS.AllocateID(self, prefix)
        self.addid(id)
        return id
    
    def addid(self, id):
        self.ids[id] = id

    def message(self, id, message, foreground='Black'):

        self.rs_text.insert(END, "%s: %s\n" % (id, message), foreground)
        self.rs_text.yview(END)
        
class MyIO:
    """stdin/stdout style IO object that redirects to a TK text widget"""
    def __init__(self, text):
        self.text = text
    def write(self, *args):
        self.text.insert(END, args[0])
        self.text.yview(END)
    def flush(self): pass

def usage():
    print 'Usage: KPLauncher.py [-m main_module_name]'
    print '                     [-f file1.py,file2.py...]'
    print '                     [-s suitcasefile1,suitcasefile2...]'
    print '                     [-S scfile1:filename1,scfile2:filename2...]'
    print '                     [-t] est mode\n'
    print '\tThe argument to -m is the module name that is the main module'
    print '\tof the KP.  If you have only one module, -m can be ommitted.\n'
    print '\tArguments to -f are expected to be full pathname to the'
    print '\tactual Python files that contain the KP modules.\n'
    print '\tArguments to -s are expected to be file system names for suitcase'
    print '\tfiles that will have the same name in the suitcase filesystem\n'
    print '\tArguments to -S are expected to be comma separated pairs'
    print '\t(where the pairs are separated by colons) where the first in the'
    print '\tpair is the name the file will have in the suitcase filesystem'
    print '\tand the second of the pair is the name of the filesystem file.\n'
    print '\tThe -t flag simply bypasses this usage message and tests the GUI'
    print '\tThis method can be used to create & launch KPs on-the-fly.'
    
    sys.exit(1)


if __name__ == '__main__':

    import getopt

    opts, args = getopt.getopt(sys.argv[1:], 'tm:f:s:S:')
    filenames = suitcasefiles = main_mod_name = None

    if len(opts) == 0:
        usage()
        
    for o, a in opts:
        if o == '-m':
            main_mod_name = a
            continue
        if o == '-f':
            filenames = []
            for file in string.split(a,','):
                filenames.append(file)
            continue
        if o == '-s':
            suitcasefiles = []
            for file in string.split(a,','):
                suitcasefiles.append({'scname' : file, 'filename' : file})
            continue
        if o == '-S':
            suitcasefiles = []
            for files in string.split(a,','):
                sc_file = string.split(files, ':')
                if len(sc_file) <> 2:
                    usage()
                suitcasefiles.append({'scname' : sc_file[0],
                                      'filename' : sc_file[1]})
            continue
        if o <> '-t':
            usage()

    baseurldir = 'file:' + sys.argv[0]
    root = Tk()
    root.iconify()
    kpg = KPLauncherGUI(root, baseurldir, addquit=1,
                        main_mod_name=main_mod_name,
                        filenames=filenames,
                        suitcasefiles=suitcasefiles)
    kpg.create_launch_panel()
    kpg.start_launch_panel(deiconify=1)
   
    print 'Entering main loop...'
    try:
        root.mainloop()
    finally:
        print 'Cleaning up visualizer...',
        kpg.cleanup()
        print 'Done.'
