"""Testing mid-level KP MIMEifying and unMIMEifying"""

from KPMimeParser import KPMimeParser
from KPMimeWriter import KPMimeWriter
from types import *
import sys



def mime_test(args=None):
    """Mime-ifying test program."""
    if not args:
        args = sys.argv[1:]
    if args:
        fp = open(args[0], 'w')
    else:
        fp = sys.stdout
    w = KPMimeWriter(fp)
    # Package
    part = w.currentpart()
    part.addheader("From", "bwarsaw@cnri.reston.va.us")
    part.addheader("Date", "Mon Feb 12 17:21:48 EST 1996")
    part.addheader("To", "kss-submit@cnri.reston.va.us")
    part.addheader("MIME-Version", "1.0")
    w.start_kp(boundary='801spam999')
    # Metadata
    w.start_metadata(boundary='802spam999')
    w.simple_metadata(
        # metadata
        {'interpreter': 'python',
         'interpreter-version': '1.3',
         'owner-name': 'Barry Warsaw',
         'owner-rendezvous': 'bwarsaw@cnri.reston.va.us',
         'home-KSS': 'kss.cnri.reston.va.us',
         'identifier': 'hdl://cnri.kss/my_first_knowbot',
         'launch-date': 'Mon Feb 12 16:39:03 EST 1996',
         },
        # context
        {'access': 'read-only'}
        )
    w.start_complex_metadata(
        # key
        'connection',
        # context
        {'access': 'read-only',
         'connection-description': "Barry's Big Bass Business",
         'connection-id': 'B4',
         'connection-direction': 'client'
         })
    bodyfp = w.currentpart().startbody('text/isl')
    bodyfp.write(SELLER_ISL)
    w.end_complex_metadata()
    # complex metadata included by reference
    w.start_complex_metadata(
        # key
        'generic-interface',
        # context
        {'access': 'read-only',
         'connection-description': 'Generic Interface for All Knowbots',
         'connection-id': 'generic-kp',
         'connection-direction': 'client'
         },
        # url
        'hdl://cnri.kss/generic-knowbot')
    bodyfp = w.currentpart().startbody('text/isl')
    w.end_complex_metadata()
    w.end_metadata()
    # code
    w.start_code(boundary='803spam999')
    w.start_code_source('BuyerKP')
    bodyfp = w.currentpart().startbody('text/python')
    bodyfp.write(BUYERKP_PY)
    w.end_code_source()
    w.code_object(_f.func_code)
    w.code_object(_f.func_code, 'hdl://cnri.kss/sample-kp/_f')
    w.code_object(None, 'hdl://cnri.kss/sample-kp/_g', 108)
    w.end_code()
    # state
    w.start_state(boundary='804spam999', mainmodule='main')
    w.start_state_source('main')
    bodyfp = w.currentpart().startbody('text/python')
    bodyfp.write(MAIN_PY)
    w.end_state_source()
    w.state_pickle(_p)
    w.end_state()
    w.end_kp()
    fp.flush()
    if fp <> sys.stdout:
        fp.close()


def unmime_test(args=None):
    """Un-mime-ifying test program."""
    if not args:
        args = sys.argv[1:]
    if args:
        fp = open(args[0])
    else:
        fp = sys.stdin
    kp = KPMimeParser(fp)
    print '---------- metadata ----------'
    md = kp.metadata()
    for mdkey in md.keys():
        mdtype, val, context, url = md[mdkey]
        print 'Key: ', mdkey
        print '    Type :', mdtype
        if url:
            print '    Url  :', url
        print '    Cntxt:', context
        print '    Value:', 
        if type(val) == StringType:
            print val
        elif val:
            ctype, hdrs, body = val
            print
            print '        content-type:', ctype
            for h in hdrs.keys():
                print '        %s: %s' % (h, hdrs[h])
            print
            while 1:
                line = body.readline()
                if not line: break
                print '        %s' % line[:-1]
        else:
            print '[no value]'
    print '---------- code ----------'
    main, dict, cobjs = kp.code()
    print 'Main module:', main
    for mdkey in dict.keys():
        ctype, sourcefp = dict[mdkey]
        print 'Module:', mdkey
        print 'Content-Type:', ctype
        while 1:
            line = sourcefp.readline()
            if not line: break
            print '    %s' % line[:-1]
    print 'There are', len(cobjs), 'code objects'
    for c in cobjs:
        if type(c) == CodeType:
            print '    [inline code object]'
        else:
            print '    [ref: %s, len: %d]' % c
    print '---------- state ----------'
    main, dict, pickles = kp.state()
    print 'Main module:', main
    for mdkey in dict.keys():
        ctype, sourcefp = dict[mdkey]
        print 'Module:', mdkey
        print 'Content-Type:', ctype
        while 1:
            line = sourcefp.readline()
            if not line: break
            print '    %s' % line[:-1]
    print 'There are', len(pickles), 'objects'
    for p in pickles:
        if type(p) == TupleType:
            print '    [ref: %s, len: %d]' % c
        else:
            print '    [inline unpickled object]'
    print '---------- suitcase ----------'
    suitcase = kp.suitcase()
    for skey in suitcase.keys():
        print 'Content-Id:', skey
    # TBD: we should further decode the suitcase data
    print '---------- end of kp ----------'
    if fp <> sys.stdin:
        fp.close()



SELLER_ISL = '''\
INTERFACE Seller-1;

TYPE Seller = OBJECT
    DOCUMENTATION "A simple Seller interface to test ILU"
    METHODS
            price():INTEGER,
    END;
'''

BUYERKP_PY = '''\
class Buyer:
    def __setup__(self, maxprice):
        self._maxprice = maxprice

    def __main__(self, kos):
        """Entry point upon arrival at a new KOS."""
        broker = kos.broker()
        # B4 == Barry\'s Big Bass Business :-)
        seller = broker.lookup(\'Seller_1.Seller\', \'B4\')
        if seller:
            price = seller.price()
            print \'Seller wants $\', price, \'... \'
            if price > self._maxprice:
                print \'too much!\'
            else:
                print \"I\'ll take it!\"
        else:
            print \'no seller found here\'
'''

MAIN_PY = '''\
# instantiate a buyer instance and put it in a magic place for the KOS
# to find.
__kp__ = Buyer()
__kp__.__setup__(500)
'''

def _f():
    hw = 'Hello World'
    print hw, 'is', len(hw), 'characters long'

class _P:
    def __init__(self):
        self._a = 8
        self._b = .01
        self._c = 'Spam is Yummy!'
_p = _P()



if __name__ == '__main__':
    if len(sys.argv) < 2:
        argv = ['testkp.txt']
    else:
        argv = sys.argv[1:]
    mime_test(argv)
    unmime_test(argv)
