"""Hackery for timing ILU calls.

Interpose this code between iluPr and iluRt to use. In iluRt commment out the 
`from iluPr import *' and replace with `from iluProf import *'.

"""

from iluPr import *
import time
import os
import sys
import marshal
import regex

iluProf = 1        # can check for this var to determine if we're profiling

# log this stuff to disk
# let's intuit the filename for the log based on sys.argv
# what is a good regexp to match filenames?
name = regex.compile('/?\([A-Za-z_\-]+\)\.py')
if name.search(sys.argv[0]) != -1:
    progname = name.group(1)
    logfile = "/tmp/iluprof.%s.%s" % (progname,
                                      str(int(time.time())))
else:
    logfile = "/tmp/iluprof.%s.%s" % (str(os.getpid()),
                                      str(int(time.time())))
logfd = None

def set_logfile(path):
    dir, file = os.path.split(path)
    if os.path.exists(dir):
        global logfile, logfd
        if logfd:
            logfd.close()
            os.rename(logfile, path)
            logfd = None
        logfile = path

def exitfunc():
    if logfd:
        logfd.close()

sys.exitfunc = exitfunc

# Let's store some information about the current run and system
import socket

nodename = os.uname()[1]
hostinfo = socket.gethostbyaddr(nodename)
sysinfo = (('__sys__', '', sys.argv, hostinfo),
           [])

# want to grab all the neat class info

_FormClassRecord = FormClassRecord

classes = {}

class IluProfileClass:
    """
    This is what the stub code passes:

    ('Stress.Server', '', 'ilu:lEfXYWpML+XBEja2SgQrLll2cOE', None, 0, 0, None, (('Pong', 1, 0, 0, ()),), ())
    """
        
    def __init__(self, args):
        self.name = args[0]
        self.methods = args[7]
        self.nummeth = len(self.methods)

    def get_method(self, i):
        if i > self.nummeth or i <= 0:
            raise ValueError, "index out of range"
        return self.methods[i-1]

def FormClassRecord(*args):
    global classes
    myclass = IluProfileClass(args)
    ret = apply(_FormClassRecord, args)
    classes[ret] = myclass
    return ret

# creater timers, which are lists of events and times

timers = {}

class Timer:

    def __init__(self):
        # need to do something with call info
        self.events = []
        self.id = None
        self.info = None  # 3-tuple of (Class, method, id)

    def set_id(self, callid):
        self.id = callid

    def record(self, func, dir):
        self.events.append(func.__name__, dir, time.time())

    def dump(self):
        if self.info == None:
            raise RuntimeError, "shouldn't be dumping" + self.__class__
        return marshal.dumps((self.__class__ == ClientTimer,
                              self.info, self.events))

class ClientTimer(Timer):

    def __init__(self, callinfo):
        Timer.__init__(self)
        if callinfo:
            global classes
            inst, iluclass, funcid = callinfo
            cl = classes[iluclass]
            self.info = (cl.name, cl.get_method(funcid)[0], `inst`)
        else:
            self.info = ('Null Timer', 0)

class ServerTimer(Timer):

    def __init__(self, *args):
        Timer.__init__(self)
        call, true, klass = args[0]
        self.info = None
        self.klass = klass._IluClass.name()

    def get_skel_info(self, method, id):
        self.info = (self.klass, method, id)

    def dump(self):
        if not self.info:
            print "warning: no skel info for", self.klass
            self.info = (self.klass, 'unknown', '')
        return Timer.dump(self)

def store_timer(timer, callid):
    timers[callid] = timer
    timer.set_id(callid)

def get_timer(callid, null=Timer()):
    try:
        return timers[callid]
    except KeyError:
        # we don't want to have to worry about errors inside the func
        return null

def free_timer(timer):
    global timers
    try:
        global logfd, logfile
        if logfd is None:
            logfd = open(logfile, 'a')
            marshal.dump(sysinfo, logfd)
        logfd.write(timer.dump())
        del timers[timer.id]
    except KeyError:
        pass
    

# function to replace a routine with an instrumented versoin

_BeginCall = BeginCall
_FinishCall = FinishCall

CallBegin = 1
CallEnd = 2

def BeginCall(*args):
    # should probably grab more caller info from these args
    # we should be able to say who is calling and what method it is
    t = ClientTimer(args)
    t.record(_BeginCall, CallBegin)
    ret = apply(_BeginCall, args)
    t.record(_BeginCall, CallEnd)
    store_timer(t, ret)
    return ret

def FinishCall(*args):
    timer = get_timer(args[0])
    timer.record(_FinishCall, CallBegin)
    apply(_FinishCall, args)
    timer.record(_FinishCall, CallEnd)
    free_timer(timer)
    
_BeginRequest = BeginRequest
_FinishRequest = FinishRequest
_GetReply = GetReply
_ReplyRead = ReplyRead

def BeginRequest(*args):
    timer = get_timer(args[0])
    timer.record(_BeginRequest, CallBegin)
    ret = apply(_BeginRequest, args)
    timer.record(_BeginRequest, CallEnd)
    return ret
    
def FinishRequest(*args):
    timer = get_timer(args[0])
    timer.record(_FinishRequest, CallBegin)
    ret = apply(_FinishRequest, args)
    timer.record(_FinishRequest, CallEnd)
    return ret
    
def GetReply(*args):
    timer = get_timer(args[0])
    timer.record(_GetReply, CallBegin)
    ret = apply(_GetReply, args)
    timer.record(_GetReply, CallEnd)
    return ret
    
def ReplyRead(*args):
    timer = get_timer(args[0])
    timer.record(_ReplyRead, CallBegin)
    ret = apply(_ReplyRead, args)
    timer.record(_ReplyRead, CallEnd)
    return ret
    
# server methods

_InputObjectID = InputObjectID
_FinishReply = FinishReply
_NoReply = NoReply
_RequestRead = RequestRead
_BeginReply = BeginReply

class SkelWrapper:

    def __init__(self, obj, call, timer):
        self.obj = obj
        self.call = call
        self.timer = timer
        self._IluClass = obj._IluClass
        self._IluInstVars = obj._IluInstVars

    def __getattr__(self, attr):
        self.timer.get_skel_info(attr, `self.obj`)
        return getattr(self.obj, attr)

def InputObjectID(*args):
    t = get_timer(args[0])
    if t.__class__ == ClientTimer:
        t.record(_InputObjectID, CallBegin)
        ret = apply(_InputObjectID, args)
        t.record(_InputObjectID, CallEnd)
        return ret
    else:
        t = ServerTimer(args)
        t.record(_InputObjectID, CallBegin)
        ret = apply(_InputObjectID, args)
        t.record(_InputObjectID, CallEnd)
        store_timer(t, args[0])
        return SkelWrapper(ret, args[0], t)

def RequestRead(*args):
    timer = get_timer(args[0])
    timer.record(_RequestRead, CallBegin)
    apply(_RequestRead, args)
    timer.record(_RequestRead, CallEnd)

def BeginReply(*args):
    timer = get_timer(args[0])
    timer.record(_BeginReply, CallBegin)
    apply(_BeginReply, args)
    timer.record(_BeginReply, CallEnd)

def FinishReply(*args):
    timer = get_timer(args[0])
    timer.record(_FinishReply, CallBegin)
    apply(_FinishReply, args)
    timer.record(_FinishReply, CallEnd)
    free_timer(timer)
    
def NoReply(*args):
    timer = get_timer(args[0])
    timer.record(_NoReply, CallBegin)
    apply(_NoReply, args)
    timer.record(_NoReply, CallEnd)
    free_timer(timer)
    
