"""Uniform configurable IO mechanism, using simpleio when possible."""

import sys
import nstools

def get_io_mechanism(kos, use_dev=None, height=0, width=0):
    """Establish the IO device.

    use_dev
        Either 'X', 'tty', or 'null'."""

    if use_dev and use_dev == 'tty':
        return tty_io_mechanism()
    elif use_dev and use_dev == 'null':
        return null_io_mechanism()
    else:
        try:
            sio_descr = kos.lookup_service('SimpleIOAPI.SimpleIOFactory',
                                           'simpleio')
        except nstools.BadPathError:
            print "get_io_mechanism: No X device access (simpleio), using tty"
            return tty_io_mechanism()
        else:
            sio_factory = sio_descr.Open()
            return sio_factory.GetNewSimpleIOHW(height, width)

class tty_io_mechanism:
    """stdin/stdout style IO object"""
    def read(self, *args):  return apply(sys.stdin.read, args)
    def readline(self, *args): return apply(sys.stdin.readline, args)
    def readlines(self, *args): return apply(sys.stdin.readlines, args)
    def write(self, *args): apply(sys.stdout.write, args)
    def writelines(self, *args): apply(sys.stdout.writelines, args)
    def close(self): pass


class null_io_mechanism:
    """null IO object..."""
    def read(self, *args): return ''
    def readline(self, *args): return ''
    def readlines(self, *args): return []
    def write(self, *args): pass
    def writelines(self, *args): pass
    def close(self): pass
