"""Physical font definitions.

Export a number of symbolic names for fonts, to help solve the problem
of fonts unavailable on a particular system.  The value of the name is
a string usable as a Tk font name (i.e. an X11 font wildcard
specification).

ALL OTHER CODE SHOULD USE THE FONT NAMES EXPORTED BY THIS MODULE ONLY!

If you need more fonts, you have to add them here.

Font names have the form <family>_<style>_<size>.

<family> is either BODY or TT; BODY is a proportional font usable
for displaying labels, running text, etc., while TT is a fixed-width
typewriter style font usable for displaying program output which might
contain tabular data.

<style> is R, I, B or BI, for Roman, Italic, Bold or Bold-Italic.

<size> is S, M or L for Small, Medium or Large.  The normal size is M.

In the future, X11 resources may be used to customize this module.

The test() program is run when this module is executed as a main
program; it displays a window containing table of all fonts defined.

XXX Question: what if initializing this module were to require making
calls to Tk?  (Well, it could always open a toplevel and close it
again without ever running a main loop.)

"""

BODY_R_S = "-*-helvetica-medium-r-normal-*-*-100-*-*-*-*-*-*"
BODY_R_M = "-*-helvetica-medium-r-normal-*-*-120-*-*-*-*-*-*"
BODY_R_L = "-*-helvetica-medium-r-normal-*-*-140-*-*-*-*-*-*"

BODY_I_S = "-*-helvetica-medium-o-normal-*-*-100-*-*-*-*-*-*"
BODY_I_M = "-*-helvetica-medium-o-normal-*-*-120-*-*-*-*-*-*"
BODY_I_L = "-*-helvetica-medium-o-normal-*-*-140-*-*-*-*-*-*"

BODY_B_S = "-*-helvetica-bold-r-normal-*-*-100-*-*-*-*-*-*"
BODY_B_M = "-*-helvetica-bold-r-normal-*-*-120-*-*-*-*-*-*"
BODY_B_L = "-*-helvetica-bold-r-normal-*-*-140-*-*-*-*-*-*"

BODY_BI_S = "-*-helvetica-bold-o-normal-*-*-100-*-*-*-*-*-*"
BODY_BI_M = "-*-helvetica-bold-o-normal-*-*-120-*-*-*-*-*-*"
BODY_BI_L = "-*-helvetica-bold-o-normal-*-*-140-*-*-*-*-*-*"

TT_R_S = "-*-courier-medium-r-normal-*-*-100-*-*-*-*-*-*"
TT_R_M = "-*-courier-medium-r-normal-*-*-120-*-*-*-*-*-*"
TT_R_L = "-*-courier-medium-r-normal-*-*-140-*-*-*-*-*-*"

TT_I_S = "-*-courier-medium-o-normal-*-*-100-*-*-*-*-*-*"
TT_I_M = "-*-courier-medium-o-normal-*-*-120-*-*-*-*-*-*"
TT_I_L = "-*-courier-medium-o-normal-*-*-140-*-*-*-*-*-*"

TT_B_S = "-*-courier-bold-r-normal-*-*-100-*-*-*-*-*-*"
TT_B_M = "-*-courier-bold-r-normal-*-*-120-*-*-*-*-*-*"
TT_B_L = "-*-courier-bold-r-normal-*-*-140-*-*-*-*-*-*"
TT_B_XL = "-*-courier-bold-r-normal-*-*-180-*-*-*-*-*-*"

TT_BI_S = "-*-courier-bold-o-normal-*-*-100-*-*-*-*-*-*"
TT_BI_M = "-*-courier-bold-o-normal-*-*-120-*-*-*-*-*-*"
TT_BI_L = "-*-courier-bold-o-normal-*-*-140-*-*-*-*-*-*"

def test():
    import Tkinter
    root = Tkinter.Tk()
    g = globals()
    names = filter(lambda x: x[:2] in ('TT', 'BO'), g.keys())
    names.sort()
    labels = []
    for name in names:
        value = g[name]
        l = Tkinter.Label(root,
                          text="%s = %s" % (name, `value`),
                          font=value,
                          )
        l.pack(anchor='w')
    quit = Tkinter.Button(root, text="Quit", command=root.quit)
    quit.pack(anchor='w')
    root.mainloop()

if __name__ == '__main__':
    test()
