(* Administration interface for KOS kernels
 *
 * $Revision: 1.5 $
 * $Source: /projects/cvsroot/koe/src/koe/interfaces/AdministratorAPI.isl,v $
 *)

INTERFACE AdministratorAPI IMPORTS ReportingAPI, KOS END;


(* typedefs *)

TYPE String = ilu.CString;
TYPE KOSProcessType = ENUMERATION kos, kp, extension, kp-starter END;

TYPE ProcessInfo = RECORD
    "type" : KOSProcessType,
    name : String,
    pid : INTEGER,
    kid : String,
    label : String    (* might not need this for anything *)
END;

(* Probably don't want all of these to be strings, but I'm not sure
 * what they could be instead. Unions seem like a hassle.
 *)
TYPE ProcessList = SEQUENCE of ProcessInfo;

TYPE Visualizer = ReportingAPI.Visualizer;
TYPE VisualizerID = ReportingAPI.VisualizerID;


TYPE Administrator = OBJECT
    DOCUMENTATION
        "Administrative interface to the KOS."

    METHODS
	GetProcessInfo(): ProcessList
	    "Get information about all process run by the KOS.",

        RegisterVisualizer(watch: Visualizer) : VisualizerID
	    RAISES KOS.AuthorizationError,
		   KOS.ResourceError
	    END
	    "Register a new visualizer with the KOS kernel.",

        ASYNCHRONOUS
	UnregisterVisualizer(id: VisualizerID)
	    "Withdraw a previously registered visualizer object.",

	ASYNCHRONOUS
	Shutdown()
	    "Gracefully shuts down the KOS kernel."

    END; (* Administrator *)
