(* Preliminary interface specification for an Elhill communications
 * KOS extension
 *
 * Recent changes:
 *
 * $Version$
 * $Source: /projects/cvsroot/koe/src/koe/interfaces/ElhillController.isl,v $
 *)

INTERFACE ElhillController;

TYPE String = ilu.CString;

EXCEPTION BadService : String
	"Invalid service specified";

EXCEPTION BadDatabase : String
	"Invalid database specified";

EXCEPTION NoFields : String
	"No fields found from print full";

EXCEPTION LoginFailed : String
	"Elhill login failed after five attempts";

EXCEPTION UnexpectedLine : String
	"Unexpected Line";

EXCEPTION NoUI : String	
	"Missing Unique ID from result";

EXCEPTION NonNumericUI : String
	"Nonnumeric UI";

EXCEPTION NoTitle : String
	"Missing Title from result";

EXCEPTION BadTerm: String
	"Badly formed search term";

TYPE LOGIN = SHORT SEQUENCE OF String;
TYPE LOGINS = SHORT SEQUENCE OF LOGIN;

TYPE SearchTerm = RECORD
	negated : BOOLEAN,
	term : String,
	field : String
END;

TYPE ANDExpression = SEQUENCE OF SearchTerm;

TYPE ORExpression = SEQUENCE OF ANDExpression;

TYPE SearchResult = RECORD
	Exceptional : INTEGER,
	NumFound : INTEGER
END;

TYPE LongRec = RECORD
	TI : String,
	AU : String,
	AD : String,
	JL : String,
	YR : String,
	AB : String,
	MH : String,
	UI : String,
	RN : String,
	LA : String,
	PT : String,
	SO : String,
	Exceptional : INTEGER
END;

TYPE ShortRec = RECORD
	TI : String,
	UI : String,
	Exceptional : INTEGER
END;

TYPE MEDLINE = OBJECT

	METHODS

	connect(Logins : LOGINS) : BOOLEAN
	  RAISES LoginFailed, BadService, BadDatabase END
	  "Connect to the database",

	close()
	  "Closes the database connection",

	search(Expression : ORExpression) : SearchResult
	  RAISES LoginFailed, BadTerm,
		BadService, BadDatabase END
	  "Perform an Elhill search based on SearchExpression",

	fetchRecord(RecNum : INTEGER) : LongRec
	  RAISES LoginFailed, UnexpectedLine, NoFields,
		BadService, BadDatabase END
	  "Fetch a single record from a search",

	fetchUITI(RecNum : INTEGER) : ShortRec
	  RAISES LoginFailed, NoUI, NonNumericUI, NoTitle,
		BadService, BadDatabase END
                  "Fetch a UI and TI from a single record from a search"

	END;

