(* Knowbot Operating System interface. *)

(* $Id: KOS.isl,v 1.29 1997/04/22 22:00:37 jhylton Exp $ *)
(* $Source: /projects/cvsroot/koe/src/koe/interfaces/KOS.isl,v $ *)

INTERFACE KOS IMPORTS KPRepr END;

TYPE String = ilu.CString;
TYPE BinaryString = SEQUENCE of BYTE;
TYPE SBH = String;

TYPE KPID	= String;		(* Kernel Process Idenitifier *)
TYPE KOSAddress	= String;		(* Address of a KOS *)
TYPE SubmissionType = String;           (* Type of KP submission format *) 

CONSTANT AlphaFormat : String = "Alpha System Submission Format";

EXCEPTION EncodingError : String
	"Error in the MIME-data packaging a KP.";

EXCEPTION MetadataError : String
	"Error in metadata content or missing metadata.";

EXCEPTION AuthorizationError : String
	"User is not authorized to use the KOS.";
	(* or some particular requested resource. *)

EXCEPTION ResourceError : String
	"Resource req. declared in the metadata cannot be met.";
	(* resource is not on KOS or is already obligated. *)

EXCEPTION CommunicationError : String
	"KOS could not support the requested connection.";
	(* or that a transport error occurred during communication. *)

EXCEPTION NotFound : String
	"Designated item not found.";


TYPE Receiver =
	OBJECT

	DOCUMENTATION
	  "Receive KP submissions and plugin registration requests."

	METHODS

	  SubmitKP("type": SubmissionType, repr: BinaryString) : KPID
	    RAISES EncodingError,		(* MIME-encoding error	     *)
		   MetadataError,		(* Missing information	     *)
		   AuthorizationError,		(* User/KP authorization     *)
		   ResourceError		(* Resources unavail.	     *)
	    END
	    "Submit a KP in a representation defined by the type parameter",

	  SubmitRepr(repr: KPRepr.Representation) : KPID
	    RAISES MetadataError,		(* Missing information	     *)
		   AuthorizationError,		(* User/KP authorization     *)
		   ResourceError		(* Resources unavail.	     *)
	    END,

	  KillKP(id: KPID, signal: INTEGER, password: String)
	    RAISES AuthorizationError, (* Bad password *)
		   ResourceError       (* No such KP *)
	    END
	    "Send a Unix signal to a KP.  XXX Interim interface.",
	    (* Currently the password is any non-empty string. *)

	  RegisterPlugin(pid: INTEGER) : SBH
	    "Allocate an PluginEntry for a KOS plugin process."
	    (* XxX Would like to return KOSKernel.PluginEntry, *but*
	       that would require mutual imports with KOSKernel.isl,
	       which bollixes the ILU stubber, sigh.  Rather than
	       totally restructure the two, we'll return SBH and
	       manually construct the ILU object. 
	       This call is a security leak, because the ingenious KP
	       programmer could create a PLuginEntry object with this
	       call. It doesn't introduce *new* risks, however. *)   

	END;	(* Receiver *)
