INTERFACE MarketAPI;

(* Various market interfaces, simple to arcane.

 * $Revision: 1.13 $
 * $Source: /projects/cvsroot/koe/src/koe/interfaces/MarketAPI.isl,v $
*)

TYPE String = ilu.CString;

TYPE Cred = String;

TYPE ItemDescription = RECORD	(* Item's static identifying characteristics *)
    name : String,
    brief : String,
    details : String,		(* XxX --> list of strings so needn't parse *)
    owner : String
END;

(* Reveal merchandise selection sans price, particularly price range *)
TYPE ItemDescriptions = SHORT SEQUENCE of ItemDescription;

(* Enumeration's too clunky for this.  We're using a string, instead. *)
(*TYPE LotReportType ENUMERATION LOT, DONE, PENDING, SETTLED, TABLED END;*)

TYPE AuctionItemBid = RECORD
    desc : ItemDescription,
    current-bid : INTEGER,
    start-bid : INTEGER
END;

TYPE Date = String;

TYPE Receipt = RECORD
    desc : ItemDescription,
    buyercred : Cred,		(* XxX Separate into buyer id and cred *)
    payment : INTEGER,
    date : Date,
    validation : String		(* pgp-stamped with seller signature *)
END;

EXCEPTION DataUnavailable: String
	"Data Unavailable";

EXCEPTION NotFound : String
	"Requested object not found.";

EXCEPTION BadValue : String
	"Argument out of range.";

TYPE SimpleStore = OBJECT
    DOCUMENTATION "Simplistic selling interface."

    METHODS

	Wares() : ItemDescriptions
	    "Report the kinds of items currently for sale.",

	Price(itemname: String) : INTEGER
	    RAISES NotFound END
	    "Report the price for a particular item.",

	Purchase(itemname: String, payment: INTEGER, 
		 buyercred: Cred) : Receipt
	    RAISES NotFound END
	    "Fulfill a purchase, returning validation receipt for item."

    END;


TYPE BasicAuctioneer = OBJECT
    DOCUMENTATION "Basic auction protocol."

    METHODS

	ASYNCHRONOUS Register(who: Cred, bidder: AuctionBidder)
	    "Register to participate in a session.",

	(* When() : INTEGER
	    "Time in seconds until the auction session.", *)
	
	Lot() : AuctionLot
	    "Items on the block in current auction",

	CurrentBid(): AuctionItemBid
	    RAISES DataUnavailable END 
	    "Report the current bid on the item."

    END;

TYPE AuctionBidder = OBJECT
    DOCUMENTATION "Mechanism by which auction round bids are solicited."

    METHODS

	ASYNCHRONOUS NextItem(what: ItemDescription, remaining: INTEGER)
	    "Notify bidders of start of new round.",

	(* XxX This will also be asynch when we go to asynch method *)
	SolicitBid(asking: INTEGER, what: ItemDescription,
		   currhigh: String) : BOOLEAN
	    "Obtain a bid on the current item.  Returning 0 means no bid.",

	ASYNCHRONOUS Award(receipt: Receipt)
	    "Award receipt to high bidder, receiving price amt as ack.",

	ASYNCHRONOUS Settled(item: ItemDescription, to: String,
			     price: INTEGER, remaining: INTEGER)
	    "Notify of item sale.",

	ASYNCHRONOUS Tabled(item: ItemDescription, remaining: INTEGER)
	    "Notify no-sale - item received no satisfactory bid.",

	ASYNCHRONOUS Lot(lot: AuctionLot)
	    "Verify registration by reporting auction lot."

    END;

TYPE AuctionLot = OBJECT
    DOCUMENTATION "Bidder view of items on the auction block."
    METHODS

	CurrentItem() : ItemDescription
	    "Return the description of the current item.",

	Report(selector: String) : ItemDescriptions
	    RAISES BadValue END
	    "List of items according to selector."

    END;


(* Wakeup stuff should eventually be done as part of trigger language *)
TYPE WakeupService = OBJECT
    DOCUMENTATION "Provide trigger vars for requested alarms time."

    METHODS
		
	Register(a: INTEGER, b: BOOLEAN)
		(* secs: INTEGER, callback: WakeupObject, arg: String) *)
	    "In number of seconds invoke Wakeup method of callback."

    END;

TYPE WakeupObject = OBJECT
    DOCUMENTATION "Pass as callbackup object for WakeupService"

    METHODS

	WakeupCall(arg: String)
	    "Callback for invocation by WakeupService."

    END;
