(* Specifications for Namespace types; generic namespace traversal
 *
 * A Descriptor is a simple record containing two string fields.  The
 * first is a type name (i.e. Python class name), and the second is an
 * object reference (i.e. an SBH).  Given these two data, a surrogate
 * for the object can be instantiated by using ObjectOfSBH().
 *
 * A Context is a location in the Namespace graph which can resolve
 * relative atomic names.  Resolving a name in a context always
 * returns a Descriptor which can be used to instantiate a surrogate
 * object.  If a name resolves to a subcontext, the descriptor will
 * always reference another Context object.
 *)

(* $Id: Namespace.isl,v 1.6 1997/01/02 19:40:00 guido Exp $ *)
(* $Source: /projects/cvsroot/koe/src/koe/interfaces/Namespace.isl,v $ *)

INTERFACE Namespace;

TYPE String = ilu.CString;


(* Exceptions *)

EXCEPTION ResolutionError : String
    "Lookup failed.";

EXCEPTION BindError : String
    "Binding of a name to an object failed.";


(* Type definitions *)

TYPE Descriptor = RECORD
    (* typename is the name of the type of object that the descriptor
     * points to; e.g. the first argument to ObjectOfSBH()
     *
     * reference is the opaque reference to the object, currently an SBH
     *)
    typename : String,
    reference : String
END;

TYPE Strings = SEQUENCE OF String;


(* Object definitions *)

TYPE Context = OBJECT
	METHODS
 	    Lookup1(component: String) : Descriptor
	    	RAISES ResolutionError END
	        "Returns a descriptor for the specified object

		Raises a ResolutionError if the specified component
		doesn't exist.",

	    List() : Strings
	        "Returns a list of strings of the context's names.
		Order is not defined.",

	    Bind(name: String, desc: Descriptor)
	        RAISES BindError END
		"Binds a name to a descriptor in the current context.
		If the name is already bound, raises BindError.",

	    Rebind(name: String, desc: Descriptor)
	        RAISES BindError END
		"Binds a name to a descriptor in the current context.
		If the name is already bound, it is replaced atomically.",

	    Unbind(name: String)
	        RAISES BindError END
		"Unbinds a name in the current context.",

	    GetDescriptor() : Descriptor
	        "Get descriptor for this context."

	END;
