INTERFACE Network;

TYPE string = ilu.CString;

TYPE Time = OPTIONAL INTEGER;
(* times returned by this interface are milliseconds,
   or NULL when a timeout or other failure occurs *)

TYPE Times = SEQUENCE of Time;

TYPE RouteTime = RECORD
	host : string,
        delays : Times
END;	

TYPE RouteDelays = SEQUENCE of RouteTime;

TYPE SeqString = SEQUENCE of string;

(* Host names and addresses are both represented as strings. Thus, the
   host 'www.cnri.reston.va.us' has address '132.151.1.49'. You can use
   string.split and string.atoi to turn this into a numerical address
   if you want to. 
 *)

TYPE HostInfo = RECORD
	name : string,
	aliases : SeqString,
	addresses : SeqString
END;

EXCEPTION HostNotFound;
EXCEPTION KernelNotFound;
EXCEPTION Error; (* unspecified network errors *)

TYPE Latency = OBJECT
	METHODS
	  Ping(host: string, samples: INTEGER): Times
	  RAISES HostNotFound, Error END
	END;

TYPE Route = OBJECT
	METHODS
	  TraceRoute(host: string, samples: INTEGER): RouteDelays
	  RAISES HostNotFound, Error END
	END;

TYPE NameService = OBJECT
	METHODS
	  GetHostAddress(host: string): string
	  RAISES HostNotFound END,

	  GetHostInfo(host: string): HostInfo
	  RAISES HostNotFound END,

	  GetKernelHost(kernel: string): string
	  RAISES KernelNotFound, HostNotFound END
	END;

