(* A RAP interface for Knowbots. First draft!
 *
 * $Revision: 1.7 $
 * $Source: /projects/cvsroot/koe/src/koe/interfaces/RAPK.isl,v $
 *)

INTERFACE RAPK
    IMPORTS Mapping END;


(* Types *)

TYPE String  = ilu.CString;
TYPE Strings = SEQUENCE OF String;
TYPE Handle  = String;
TYPE Handles = SEQUENCE OF Handle;

TYPE CardinalOrNull = OPTIONAL CARDINAL;
TYPE Iterator = CARDINAL;

TYPE TermsAndConditions = Mapping.StringMap;
TYPE Metadata = Mapping.StringMap;


(* exceptions *)

EXCEPTION TypeError: String
    "The digital object cannot be disseminated in the desired type.";

EXCEPTION NotFound: String
    "Could not find a digital object with that handle.";

EXCEPTION LastDocument: String
    "No more documents to iterate over.";

EXCEPTION BadIterator: String
    "Iterator is not valid.";

EXCEPTION ReadError: String
    "Problem reading from a Dissemination.";

EXCEPTION TermsAndConditionsFailure : String
    "Ts and Cs negotiation not complete.";


(* a lot of these methods can be functional, right? *)

TYPE DemoManager = OBJECT
    DOCUMENTATION
	"Just for demo purposes. Need a simple way to add new
	 documents to a repository."
    METHODS
	NewObjects(name: String, newpath: String)
    END;


TYPE Repository = OBJECT
    METHODS
	GetRepoMetadata() : Metadata
	"Get the repository's metadata.",

	Scan() : Handles
	"Return a list of all registered handles.",

	GetMetadata(hdl: Handle) : Metadata
	    RAISES NotFound END
	    "Get the metadata for a specific digital object named by `hdl'.",

	GetDO(hdl: Handle) : DigitalObject
	    RAISES NotFound END
	    "Get the digital object named by `hdl'.",

	GetTermsAndConditions(hdl: Handle) : TermsAndConditions
	    RAISES NotFound END
	    "Return the Ts & Cs for DO named by `hdl'.",

	CreateIterator(): Iterator
	    "Create an iterator over the handles in the repository.",

	Iterate(token: Iterator) : Handle
	    RAISES LastDocument, BadIterator END
	    "Using the iterator, select the next indexed handle."

    END;


(* Disseminations *)

TYPE Dissemination = OBJECT
    COLLECTIBLE
    METHODS
        GetTermsAndConditions() : TermsAndConditions
	    "Return the terms and conditions for the digital object.",
	    
	Read(Size: CardinalOrNull) : String
	    RAISES ReadError, TermsAndConditionsFailure END
	    "Read some data from the dissemination.

	    Can fail with a TermsAndConditionsFailure if the rights to
	    the dissemination have not been cleared.
	    ",

	Close()
    END;


(* Terms and conditions must be negotiated for a dissemination before
 * that dissemination can be retrieved.  Not all disseminations
 * require T&C negotiation, but all disseminations have an interface
 * for negotiating T&C (some transactions may be null)
 *
 * T&C negotiation is currently very simple.  To find out what terms
 * you must satisfy, use the GetTermsAndConditions() method, which
 * returns a Mapping object.  The keys of this Mapping object are the
 * terms which must be satisfied or conditions which can be queried.
 * The values are either values used by clients negotiating the T&Cs,
 * or are expected to be set by the clients in order to satisfy a
 * condition.  The actual set of T&C keys and the expected values are
 * implementation dependent.
 *)

TYPE DigitalObject = OBJECT
    COLLECTIBLE
    METHODS
	GetMetadata() : Metadata
	    "Return the object's metadata.",

        GetTermsAndConditions() : TermsAndConditions
	    "Return the terms and conditions for the digital object.",

	GetDissemination(dtype: String) : Dissemination
	    RAISES TypeError, TermsAndConditionsFailure END
	    "Get a dissemination of the specified type.

	    Can fail with a TermsAndConditionsFailure if the rights to
	    the dissemination have not been cleared.
	    "
    END;
