(* Reporting Station interface
 *
 * Used by the KOS, Visualizers, etc.
 *
 * $Revision: 1.3 $
 * $Source: /projects/cvsroot/koe/src/koe/interfaces/ReportingAPI.isl,v $
 *)

INTERFACE ReportingAPI IMPORTS KOS, KPRepr END;


(* typedefs *)

TYPE String = ilu.CString;
TYPE VisualizerID = String;


(* The ReportingStation is the fundamental interface for communication
 * of KP state changes to the originator (e.g. launcher) of a KP.
 * Other entities in the system may be interested in migration and
 * termination state changes for particular KPs, and they would also
 * use a ReportingStation object to gather this information.
 *)

TYPE ReportingStation = OBJECT
    DOCUMENTATION
        "This is the object to which the KOS reports KP status."

    (* reportingID is considered the `name' of the KP.  It should be
     * assumed that this name is a globally unique pure name, although
     * the current implementation may not strictly guarantee this.
     *
     * idTo is the KPid of the migrating KP, local to the receiving KOS.
     *
     * migratingTo is the name of the receiving KOS.
     *)

    METHODS
        KPTerminated(reportingID: String,
		     reason:      String,
		     details:     String,
		     repr:        KPRepr.Representation)
	    "Reports non-migration related termination of KP.",

        ASYNCHRONOUS
	KPMigrationReceived(reportingID: String,
			    idTo:        KOS.KPID,
			    migratingTo: KOS.KOSAddress)
	    "Reports acceptance of a migrating KP.",

	AllocateID(prefix: String): String
            RAISES KOS.AuthorizationError END
	    "Allocate a new reporting ID, e.g. for a clone."

    END; (* ReportingStation *)


(* The Visualizer interface is used for GUI objects that act like
 * ReportingStations.  This is probably obsolete.
 *
 * TBD: the only difference is the Shutdown() method.  Should this be
 * migrated back up into the ReportingStation?
 *)

TYPE Visualizer = OBJECT SUPERTYPES ReportingStation END
    METHODS

	ASYNCHRONOUS
	Shutdown()
	    "Gracefully shuts down the Visualizer"

    END; (* Visualizer *)
