(* Worldroot replication API *)

(* $Id: WorldAPI.isl,v 1.3 1997/01/31 00:31:37 bwarsaw Exp $ *)
(* $Source: /projects/cvsroot/koe/src/koe/interfaces/WorldAPI.isl,v $ *)

INTERFACE WorldAPI IMPORTS Namespace END;

TYPE String = Namespace.String;
TYPE OptionalDescriptor = OPTIONAL Namespace.Descriptor;
TYPE SBH = String;

TYPE Event = RECORD
	(* Admin overhead: *)
	originator: String,	(* Originating site *)
	uniqueid:   String,	(* For discarding duplicate events *)
	generation: INTEGER,	(* For discarding old updates *)
	timestamp:  INTEGER,	(* For real time logging/tracking *)

	(* Contents: *)
	dbname: String,		(* Database name *)
	action: String,		(* "Bind", "Rebind", "Unbind" *)
	key:    String,		(* Record key *)
	value:  OptionalDescriptor	(* Record value, unless Unbind *)
	END;

TYPE EventList = SEQUENCE OF Event;

TYPE Connection = OBJECT
	DOCUMENTATION "Connection between two peer worldroot servers"
	METHODS
		ASYNCHRONOUS
		DispatchEvent(event: Event)
		"Dispatch an update event"
	END;

TYPE Worldroot = OBJECT
	DOCUMENTATION "Inter-Worldroot API"
	METHODS
		Connect(peername: String,
			peer: Worldroot,
			conn: Connection): Connection
		"Set up a peer-to-peer relationship",

		GetEvents(lower: INTEGER, upper: INTEGER): EventList
		"Get all events with lower <= generation <= upper"
		(* And set the current generation to at least upper,
		   so no new events in this range will ensure.
		   Special case: if upper == 0, return all events with
		   lower <= generation *),

		PutEvents(generation: INTEGER, events: EventList)
		"Pushes the results of a sweep"
		(* And set the current generation to at least generation *),

		ASYNCHRONOUS
		Shutdown()
		"Gracefully shuts down the Worldroot server"
	END;
