#! /usr/bin/env python

"""Set up ISL file dependencies for the koe/interfaces/stubs/Makefile

This script is intended to be run (manually) from the koe/interfaces/stubs
directory whenever a new interface (e.g .isl) file is added to
koe/interfaces.  Run the script like so:

    $ ./isldepend.py `(cd ..; ls *.isl)`

This will add the proper dependency targets to Makefile (generated from
Makefile.in), overwriting Makefile.
"""

import sys
import os
import string
import re

def process(base):
    stopper = string.whitespace + ',;'
    imports = []
    ifile = os.path.join(os.pardir, base + '.isl')
    print 'Processing:', ifile
    fp = open(ifile)
    buf = fp.read()
    i = 0
    eob = len(buf)
    while i < eob:
        if buf[i:i+2] == '(*':
            i = i + 2
            while i < eob:
                if buf[i:i+2] == '*)':
                    i = i + 2
                    break
                else:
                    i = i + 1
        elif buf[i] == '"':
            i = i + 1
            while i < eob and buf[i] <> '"':
                i = i + 1
        elif string.lower(buf[i:i+7]) == 'imports' and \
             buf[i+7:i+8] in string.whitespace:
            # then
            i = i + 8
            while 1:
                while buf[i] in string.whitespace:
                    i = i + 1
                j = i
                while buf[j] not in stopper:
                    j = j + 1
                interface = string.strip(buf[i:j])
                if string.lower(interface) == 'end' and buf[j] == ';':
                    break
                print '    Found import:', interface
                imports.append(interface)
                i = j + 1
            # XXX: break out of this function here, since we're done
            return imports
        else:
            i = i + 1
    # No imports found
    return []


def writedeps(fp, target, depends):
    fp.write(target + '.py:')
    # recursive
    def recurse(target, tdone, recurse, depends=depends, fp=fp):
        if target not in tdone:
            fp.write(' ' + target + '.isl')
            tdone.append(target)
            imports = depends.get(target, [])
            for i in imports:
                recurse(i, tdone, recurse)
    recurse(target, [], recurse)
    fp.write('\n')



def main():
    depends = {}
    # calculate all dependencies
    for file in sys.argv[1:]:
        base, ext = os.path.splitext(file)
        depends[base] = process(base)
    # sort targets
    targets = depends.keys()
    targets.sort()
    # munge Makefile.in
    infp = open('Makefile.in')
    outfp = open('Makefile', 'w')
    outfp.write(infp.read())
    infp.close()
    for t in targets:
        writedeps(outfp, t, depends)
    outfp.close()



if __name__ == '__main__':
    main()
