"""Mainloop controller for the visualizer application"""

import sys
try:
    import signal
except ImportError:
    signal = None

from koe.common import nstools
import Alarm
import Visualizer
import Events
import KOE
import App



class Controller:
    def __init__(self, name, quiet, master=None, width=500, height=500):
        self.__name = name
        self.__quiet = quiet
        self.__animation_alarm = Alarm.Alarm()
        self.__nsscan_alarm = Alarm.Alarm(15.0)
        self.__e = Events.EventManager()
        self.__k = KOE.KOE()
        self.__v = Visualizer.Visualizer(master=master, width=width,
                                         height=height)
        self.__bindname()
        App.CONTROLLER = self
        if master:
            self.__in_applet = 1
        else:
            self.__in_applet = 0

    def __bindname(self):
        if not self.__name:
            # don't bind name if it hasn't been set
            return
        # bind ourself into the world's tools/ namespace
        toolsns = nstools.WorldOpen('tools')
        # see if name is already bound in the world root.  if so quit
        # now, 'cause we don't allow that.  however, if its bound but
        # unresponsive, then re-bind it with a warning
        other = None
        found = None
        if self.__name in toolsns.List():
            found = 1
            try:
                other = toolsns.Lookup1(self.__name).Open()
                # I'm assuming IluPing() should be as responsive as a
                # Pong() method in the interface.
                if other and not other.IluPing():
                    other = None
            except nstools.BadPathError, msg:
                other = None
        if other:
            print 'Error: a visualizer named "%s" is already running' \
                  % self.__name
            print 'Error: this is not allowed... exiting.'
            sys.exit(-1)
        if found:
            # another visualizer was found but it was unresponsive
            if not self.__quiet:
                print 'Warning: a visualizer named "%s" was found' \
                      % self.__name
                print 'Warning: but it was unresponsive... rebinding.'
            toolsns.Unbind(self.__name)
        desc = nstools.Descriptor(typename='ReportingAPI.Visualizer',
                                  reference=self.__v.IluSBH())
        toolsns.Bind(self.__name, desc)

    def __stop(self, signum=None, frame=None, *extras):
        self.__v.stop()

    def visualizer(self):
        return self.__v

    def start(self):
        try:
            self.__nsscan_alarm.register(self.__k.scan)
            self.__animation_alarm.register(self.__e.step)
            if signal:
                signal.signal(signal.SIGINT, self.__stop)
                signal.signal(signal.SIGTERM, self.__stop)
            self.__animation_alarm.arm()
            self.__nsscan_alarm.arm()
            self.__k.scan()
            self.__v.start()
        finally:
            # Applets call this differently - visu.cleanup()
            if not self.__in_applet:
                self.cleanup()

    def cleanup(self):
        self.__animation_alarm.close()
        self.__nsscan_alarm.close()
        self.__k.close()

    def close(self):
        # unbind our name
        if self.__name:
            nstools.WorldOpen('tools').Unbind(self.__name)

    def reset_scan_alarm(self):
        self.__nsscan_alarm.close()
        self.__nsscan_alarm.arm()
