"""The image of a KP."""

import traceback
import time
import App
from Tkinter import *
from Constants import *



class KP:
    __rendered_p = 0
    __hidden_p = 0

    def __init__(self, name, age=0):
        self.__name = name
        self.__id = `time.time()`
        self.__tag = 'KP:' + self.__id
        self.__age = age
        self.__circid = None

    def render_at(self, x, y):
        if self.__rendered_p:
            self.move(x, y)
            return
        self.__rendered_p = 1
        self.__x = x
        self.__y = y
        tags = (self.__tag,)
        self.__circid = App.CANVAS.create_oval(
            x+3, y+7, x+20, y+24,
            fill=self.color(),
            tags=tags)
        self.__hatid1 = App.CANVAS.create_line(
            x+3, y, x+20, y+5,
            tags=tags)
        self.__hatid2 = App.CANVAS.create_line(
            x+3, y+5, x+20, y+5,
            tags=tags)
        self.__textid = App.CANVAS.create_text(
            x + KP_ICON_WIDTH, y + 15,
            anchor=W,
            text=self.__name,
            fill=KP_NAME_COLOR,
            font=KP_NAME_FONT,
            tags=tags)

    def add_tag(self, tag):
        App.CANVAS.addtag_withtag(tag, self.__tag)

    def remove_tag(self, tag):
        App.CANVAS.dtag(self.__tag, tag)

    def get_idtag(self):
        return self.__tag

    def hide(self):
        invisible = App.CANVAS['background']
        for item in (self.__circid, self.__hatid1,
                     self.__hatid2, self.__textid):
            App.CANVAS.itemconfigure(item, fill=invisible)
        App.CANVAS.itemconfigure(self.__circid, outline=invisible)

    def show(self):
        App.CANVAS.itemconfigure(self.__hatid1, fill='black')
        App.CANVAS.itemconfigure(self.__hatid2, fill='black')
        App.CANVAS.itemconfigure(self.__textid, fill=KP_NAME_COLOR)
        App.CANVAS.itemconfigure(self.__circid,
                                 fill=self.color(),
                                 outline='black')

    def get_origin(self):
        return self.__x, self.__y

    def get_centroid(self):
        if not self.__circid:
            return 0, 0
        ulx, uly, lrx, lry = App.CANVAS.bbox(self.__circid)
        return (lrx - ulx) * 0.5, (lry - uly) * 0.5

    def get_center(self):
        bbox = None
        if self.__circid:
            bbox = App.CANVAS.bbox(self.__circid)
        if not bbox:
            return 0, 0
        ulx, uly, lrx, lry = bbox
        return (lrx + ulx) * 0.5, (lry + uly) * 0.5

    def get_name(self):
        return self.__name

    def move(self, absx, absy):
        """Translate absolution coords to relative coords.
        Very bogus that canvas takes relative movement coordinates only!
        """
        if not self.__rendered_p:
            self.render_at(absx, absy)
        else:
            App.CANVAS.move(self.__tag, absx - self.__x, absy - self.__y)
            self.__x = absx
            self.__y = absy

    def close(self):
        App.CANVAS.delete(self.__tag)

    #
    # aging is useful in some demos
    #
    def set_age(self, age):
        self.__age = age
        App.CANVAS.itemconfigure(self.__circid, fill=self.color())

    def get_age(self):
        return self.__age

    def age(self, incr=1):
        self.set_age(self.__age + incr)

    def color(self):
        if self.__age < 0 or self.__age >= len(KP_AGE_COLORS):
            return KP_ICON_COLOR
        else:
            return KP_AGE_COLORS[self.__age]
