"""A KOS representation."""

import time
import ilu
import Visualizer
import KP
import App
from Tkinter import *
from Constants import *
from koe.common import nstools


_LOSTTIME = 10                          # seconds



class KOS:
    """Object to handle a collection of object on a display."""
    __lockp = None
    __livep = 1
    __kpcnt = 0

    def __init__(self, name, x=-100.0, y=-100.0):
        self.__name = name
        self.__extensions = []
        self.__tag = 'KOS:' + name
        self.__vstag = 'VITALSIGN:'+name
        self.__x = x
        self.__y = y
        #
        # add the KOS image to the canvas
        #
        tags = (self.__tag, self.__vstag)
        self.__triId = App.CANVAS.create_polygon(
            x+(KOS_ICON_WIDTH * 0.5), y,
            x+KOS_ICON_WIDTH, y+KOS_ICON_HEIGHT,
            x, y+KOS_ICON_HEIGHT,
            fill=KOS_ICON_COLOR,
            outline='black',
            tags=tags)
        #
        # add the kos name tag
        #
        self.__textId = App.CANVAS.create_text(
            x + 0.5 * KOS_ICON_WIDTH, y,
            anchor=S,
            fill=KOS_NAME_COLOR,
            text=name,
            tags=tags)
        #
        # add the KP count label
        #
        self.__kpcntId = App.CANVAS.create_text(
            x+25, y+KOS_ICON_HEIGHT,
            anchor=SE,
            fill=KOS_KPCNT_FONT_COLOR,
            text='0',
            tags=tags)
        #
        # bind useful events
        #
        App.CANVAS.tag_bind(self.__tag, '<ButtonPress-1>',
                            self.__button_press_event)
        App.CANVAS.tag_bind(self.__tag, '<ButtonRelease-1>',
                            self.__button_release_event)
        App.CANVAS.tag_bind(self.__tag, '<B1-Motion>',
                            self.__button_motion_event)

    def __cmp__(self, other):
        return cmp(self.__name, other.__name)

    #
    # mouse events
    #
    def __button_press_event(self, event=None):
        ulx, uly, lrx, lry = App.CANVAS.bbox(self.__tag)
        tag = 'BOX:' + self.__name
        boxid = App.CANVAS.create_rectangle(
            ulx, uly, lrx, lry,
            outline='', fill=DRAG_ITEM_BACKGROUND,
            tags=(self.__tag, tag))
        App.CANVAS.lower(boxid, self.__tag)
        self.__ptrx = event.x - self.__x
        self.__ptry = event.y - self.__y

    def __button_motion_event(self, event=None):
        self.move(event.x - self.__ptrx, event.y - self.__ptry)
        self.lock()

    def __button_release_event(self, event=None):
        App.CANVAS.delete('BOX:' + self.__name)
        del self.__ptrx
        del self.__ptry

    #
    # other utils
    #

    def close(self):
        App.CANVAS.tag_bind(self.__tag, '<ButtonPress-1>')
        App.CANVAS.tag_bind(self.__tag, '<ButtonRelease-1>')
        App.CANVAS.tag_bind(self.__tag, '<B1-Motion>')
        App.CANVAS.delete(self.__tag)

    def move(self, absx, absy):
        """Translate absolution coords to relative coords.
        Very bogus that canvas takes relative movement coordinates only!
        """
        App.CANVAS.move(self.__tag, absx - self.__x, absy - self.__y)
        self.__x = absx
        self.__y = absy

    #
    # periodic indicators
    #

    def marklive(self):
        if not self.__livep:
            App.CANVAS.itemconfigure(self.__triId,
                                     fill=KOS_ICON_COLOR,
                                     outline='black')
            App.CANVAS.itemconfigure(self.__textId, fill=KOS_NAME_COLOR)
            App.CANVAS.itemconfigure('ALLEXT:'+self.__name,
                                     fill=EXT_COLOR)
            App.CANVAS.itemconfigure(self.__kpcntId,
                                     text=`self.__kpcnt`,
                                     fill=KOS_KPCNT_FONT_COLOR)
        self.__livep = 1

    def markdead(self):
        if self.__livep:
            App.CANVAS.itemconfigure(self.__vstag, fill=KOS_DEAD_COLOR)
            App.CANVAS.itemconfigure(self.__triId, outline=KOS_DEAD_COLOR)
            App.CANVAS.delete('ALLEXT:'+self.__name)
            self.__extensions = []
            self.__kpcnt = 0
        self.__livep = 0

    def scan(self):
        # Scan the namespace for extensions that have come or gone.
        try:
            ns = nstools.WorldOpen(
                'kos/%s/plugins' % self.__name,
                typeobj='Namespace.Context')
            found = ns.List()
        except (nstools.BadPathError, ilu.IluGeneralError):
            return
        # add the newly discovered extensions
        for extname in found:
            if extname not in self.__extensions:
                x = self.__x
                y = self.__y
                nexty = (SPACING + EXT_HEIGHT) * len(self.__extensions)
                hpos = x + KOS_ICON_WIDTH
                vpos = y + KOS_ICON_HEIGHT + nexty + SPACING
                tag = 'EXT:' + extname
                tags = (self.__tag, tag, self.__vstag, 'ALLEXT:'+self.__name)
                textid = App.CANVAS.create_text(
                    hpos, vpos, anchor=NE,
                    text=extname,
                    fill=EXT_COLOR,
                    font=EXT_FONT,
                    tags=tags)
                self.__extensions.append(extname)
                nexty = nexty + SPACING + EXT_HEIGHT
        adjusty = 0
        for extname in self.__extensions[:]:
            tag = 'EXT:' + extname
            if extname not in found:
                self.__extensions.remove(extname)
                adjusty = adjusty - (EXT_HEIGHT + SPACING)
                App.CANVAS.delete(tag)
            else:
                App.CANVAS.move(tag, 0, adjusty)

    #
    # adding and deleting components
    #

    def mark_kp_as_owned(self, kp):
        kp.add_tag('KOS:' + self.__name)
        self.__kpcnt = self.__kpcnt + 1
        App.CANVAS.itemconfig(self.__kpcntId, text=`self.__kpcnt`)

    def unmark_kp_as_owned(self, kp):
        kp.remove_tag('KOS:' + self.__name)
        self.__kpcnt = self.__kpcnt - 1
        App.CANVAS.itemconfig(self.__kpcntId, text=`self.__kpcnt`)

    def slot_xy(self, kpname):
        """Return the X and Y location of the specified slot."""
        kpnames = App.KOE.get_kpnames_for_kos(self.__name)
        slotno = kpnames.index(kpname)
        x = self.__x + KOS_ICON_WIDTH + SPACING
        y = self.__y - KOS_NAME_HEIGHT + (KP_ICON_HEIGHT + SPACING) * slotno
        return x, y

    #
    # getting useful information
    #

    def get_centroid(self):
        """Return the centroid of the KOS icon."""
        ulx, uly, lrx, lry = App.CANVAS.bbox(self.__triId)
        return (lrx - ulx) * 0.5, (lry - uly) * 0.5

    def get_center(self):
        """Return the center of the KOS icon."""
        ulx, uly, lrx, lry = App.CANVAS.bbox(self.__triId)
        return (lrx + ulx) * 0.5, (lry + uly) * 0.5

    def get_name(self):
        return self.__name

    def get_lock(self):
        return self.__lockp

    def lock(self):
        if not self.__lockp:
            tag = 'LOCK:' + self.__name
            tags = (self.__tag, tag)
            App.CANVAS.create_text(
                self.__x+20, self.__y+5,
                text=LOCK_INDICATOR,
                fill=LOCK_FONT_COLOR,
                font=LOCK_FONT,
                tags=tags)
            App.CANVAS.tkraise(tag, 'KOS:' + self.__name)
        self.__lockp = 1

    def unlock(self):
        self.__lockp = None
        App.CANVAS.delete('LOCK:' + self.__name)
