"""Subclasses of the Outliner classes that allow subclasses to add methods
which 'rebuild' the just-closed elements using methods named rebuild_<gi>.
The rebuild_<gi> methods should accept the structural definition of the
element, (<gi>, attrs, content), as the only parameter and return a
replacement object.  This can be used to build semantic objects from
structured data more easily than having to walk the stucture after the fact.
"""
__version__ = '$Revision: 1.1 $'


import Outliner


class Rebuilder:
    def rebuild(self, tag):
        context = self.get_current_context()
        methodname = "rebuild_" + tag
        if hasattr(self, methodname):
            method = getattr(self, methodname)
            context[-1] = method(context[-1])


class XMLRebuilder(Rebuilder, Outliner.XMLOutliner):
    def unknown_endtag(self, tag):
        Outliner.XMLOutliner.unknown_endtag(self, tag)
        self.rebuild(tag)


class SGMLRebuilder(Rebuilder, Outliner.SGMLOutliner):
    def unknown_endtag(self, tag):
        Outliner.SGMLOutliner.unknown_endtag(self, tag)
        self.rebuild(tag)


class HTMLRebuilder(Rebuilder, Outliner.HTMLOutliner):
    def unknown_endtag(self, tag):
        Outliner.HTMLOutliner.unknown_endtag(self, tag)
        self.rebuild(tag)
