# Implementation of an GeeHill.MEDLINE interface which uses the
# generated GDBM files instead of the real Elhill.

import sys, os; sys.path.insert(0, os.path.join(sys.path[0], "../../"))
from koe.common import pathhack

import sys
import getopt
import gdbm
from koe.common import pluginlib
from koe.interfaces.stubs import GeeHill
from koe.interfaces.stubs import GeeHill__skel
import dbindex
import string


# this is the path to the medline gdbm files.  note that this should be
# the absolute file name sans the extension, which dbindex.Index fills in
# automatically.  overridable with the -i option
IDXNAME = '/export/ags/MEDLINE'



# Plugin boilerplate
def main():
    idxname = IDXNAME
    # pluginlib.register() doesn't give a conveninent way to deal with
    # arguments it doesn't know (just to ignore them)
    kos, ns = pluginlib.register('Geehill', command_args='i:')
    argc = 1
    while argc < len(sys.argv):
        if sys.argv[argc] == '-i':
            idxname = sys.argv[argc+1]
            argc = argc + 1
        argc = argc + 1

    print "Geehill starting with", `idxname`, "..."
    try:
        medline = MEDLINEImpl(idxname)
    except gdbm.error, msg:
        print "Can't open database %s: %s" % (`idxname`, str(msg))
        sys.exit(1)
    svcid = kos.bind_service('Geehill', medline, 'GeeHill.MEDLINE')
    print "Geehill started."
    kos.run()
    kos.unbind_service(svcid)
    sys.exit(0)



class MEDLINEImpl(GeeHill__skel.MEDLINE):
    def __init__(self, idxname):
        self.__index = dbindex.Index(idxname, 'r')

    def _search(self, expression):
        # Rather than try to translate the normalized expression
        # format into dbindex's whacked format, we'll just unravel
        # the expression and apply the boolean operations directly
        orhits = []
        for ANDexpr in expression:
            andhits = None
            nothits = []
            for term in ANDexpr:
                if andhits == []:
                    # Stop if result is already empty
                    break
                try:
                    hits = self.__index.search1(term.field, term.term)
                except ValueError, msg:
                    raise GeeHill.BadTerm, str(msg)
                # Carry our the Boolean combinations right away, so we
                # can stop early if the result is empty for this ANDexpr
                if not term.negated:
                    if andhits is None:
                        if nothits:
                            # Carry out postponed "and not" operation
                            andhits = dbindex.difference([hits, nothits])
                            nothits = []
                        else:
                            andhits = hits
                    else:
                        andhits = dbindex.intersection([andhits, hits])
                else:
                    if andhits is None:
                        # Negated term before other terms: postpone
                        if not nothits:
                            nothits = hits
                        else:
                            nothits = dbindex.union([nothits, hits])
                    elif hits:
                        andhits = dbindex.difference([andhits, hits])
            if andhits is None:
                # No positive terms -- this is illegal!
                raise GeeHill.BadTerm, "ANDexpr needs a positive term"
            if orhits:
                orhits = dbindex.union([orhits, andhits])
            else:
                orhits = andhits
        return orhits

    def search(self, expression):
        return map(str, self._search(expression))

    def searchCount(self, expression):
        return len(self._search(expression))

    def fetchRecord(self, recid):
        recno = recid_to_recno(recid)
        try:
            dict = eval(self.__index.lookup(recno), {}, {})
        except KeyError:
            raise GeeHill.UnknownRecord, recid
        return GeeHill.LongRec(
            dict.get('TI', ''),
            dict.get('AU', ''),
            dict.get('AD', ''),
            dict.get('JL', ''),
            dict.get('YR', ''),
            dict.get('AB', ''),
            dict.get('MH', ''),
            dict.get('UI', ''),
            dict.get('RN', ''),
            dict.get('LA', ''),
            dict.get('PT', ''),
            dict.get('SO', ''))

    def fetchUITI(self, recid):
        recno = recid_to_recno(recid)
        try:
            dict = eval(self.__index.lookup(recno), {}, {})
        except KeyError:
            raise GeeHill.UnknownRecord, recid
        return GeeHill.ShortRec(
            dict.get('TI', ''),
            dict.get('UI', ''))


def recid_to_recno(recid):
    try:
        return string.atoi(recid)
    except ValueError:
        raise GeeHill.UnknownRecord, recid



if __name__ == '__main__':
    main()
