#! /usr/bin/env python

import sys
import string
import dbindex
import stop
from koe.common.tokenizer import getTokens

MAXRECORDS = None
dbindex.CACHESIZE = 40000

import time


def parseRecord(lines):
    dict = {}
    field = None
    val = ''
    for line in lines:
        if line[:2] == '*F':
            if field:
                dict[field] = val
            field = string.strip(line[2:])
            val = ''
        else:
            val = val + line
    # be sure to get the last field
    if field:
        dict[field] = val
    return dict



def parseFile(f, func):
    recs = 0
    buf = []
    while 1:
        line = f.readline()
        if line == '':
            break
        if line[:2] == '*R':
            if buf:
                tmp = parseRecord(buf)
                func(tmp, string.join(buf))
                if not (recs % 1000):
                    print 'added record #%d' % recs
                recs = recs + 1
            buf = []
        else:
            buf.append(line)
        if MAXRECORDS is not None and recs > MAXRECORDS:
            break
    # be sure to get the last record
    if buf:
        t0 = time.time()
        tmp = parseRecord(buf)
        t1 = time.time()
        func(tmp)
        t2 = time.time()
        recs = recs + 1
        print "added rec #%d, %f+%f" % (recs, round(t1 - t0, 3),
                                        round(t2 - t1, 3))
    return recs


if __name__ == "__main__":
    global MAXRECORDS

    idxName = sys.argv[1]
    idx = dbindex.BulkIndex(idxName, 'nf', ['AU', 'AB', 'TI', 'MH'])

    try:
        MAXRECORDS = int(sys.argv[2])
    except IndexError:
        pass

    class Foo:
        pass
    obj = Foo()
    obj.id = 0

    def addEntry(rec, buf, idx=idx, obj=obj):
        # make an entry
        entry = {}
        for field in idx.fields:
            if rec.has_key(field):
                entry[field] = getTokens(rec[field])
        if entry:
            idx.addEntry(obj.id, entry, rec)
            obj.id = obj.id + 1

    t0 = time.time()
    try:
        recs = parseFile(sys.stdin, addEntry)
    except KeyboardInterrupt:
        recs = obj.id - 1
    t1 = time.time()
    print 'Records:', recs
    print 'Parse:', round(t1-t0, 3), 'sec/rec:', round((t1-t0)/recs, 4)
    idx.sync()
    t2 = time.time()
    print 'Sync :', round(t2-t1, 3), 'sec/rec:', round((t2-t1)/recs, 4)
    print 'Total:', round(t2-t0, 3), 'sec/rec:', round((t2-t0)/recs, 4)
