#! /usr/bin/env python

"""A very simple image player.

Conforms to the PlayerAPI.isl interface.
"""

from Tkinter import *
import os
import sys
import string
import getopt
import imghdr
import tempfile
from koe.common import pluginlib
from koe.interfaces.stubs import PlayerAPI, PlayerAPI__skel

_FILTER = 'djpeg -gif'



def main():
    """Main for simple image player plugin."""
    kos, ns = pluginlib.register('ImagePlayer')
    # we're only binding a simple image player
    service_id = kos.bind_service('PlayerFactory',
                                  PlayerFactory(),
                                  'PlayerAPI.PlayerFactory')
    kos.run()
    kos.unbind_service(service_id)
    sys.exit(0)



class PlayerFactory:
    __tk = None
    __root = None

    def get_toplevel(self, name, tmpfile):
        if not self.__tk:
            tk = self.__tk = Tk(className='KOEImagePlayer')
            tk.withdraw()
        return PlayerFactory.PhotoWin(self.__tk, name, tmpfile)

    class PhotoWin:
        def __init__(self, master, name, tmpfile):
            self.__root = Toplevel(master)
            self.__root.title(name)
            self.__root.iconname(name)
            self.__root.protocol("WM_DELETE_WINDOW", self.quit)
            error = None
            if tmpfile:
                try:
                    image = PhotoImage(file=tmpfile)
                    self.__image = image
                    imglabel = Label(self.__root, image=image)
                    imglabel.pack()
                    os.unlink(tmpfile)
                except TclError, msg:
                    error = 1
            else:
                error = 1
                msg = 'tmpfile was None'
            if error:
                badlabel = Label(self.__root,
                                 text='Bad Image Data',
                                 foreground='red')
                badlabel.pack()
            namelabel = Label(self.__root, text=name)
            namelabel.pack()
            quitbtn = Button(self.__root, text='Close', command=self.quit)
            quitbtn.pack()
            if error:
                raise PlayerAPI.ImageError, msg

        def quit(self, event=None):
            self.__root.destroy()
            self.__image = None

    #
    # ISL interface
    #
    def GetNewPlayer(self):
        return ImagePlayer(self)


class ImagePlayer(PlayerAPI__skel.ImagePlayer):
    __imagename = '<unnamed image>'
    __imagedata = ''
    __top = None

    def __init__(self, factory):
        self.__factory = factory

    def __gif(self, imagedata):
        try:
            tmp = tempfile.mktemp()
            fp = open(tmp, 'w')
            fp.write(imagedata)
            fp.close()
            return tmp
        except IOError, v:
            print 'GIF translation failed:', v
            os.unlink(tmp)
            return None

    def __jpeg(self, imagedata):
        try:
            tmp = tempfile.mktemp()
            fp = os.popen(_FILTER + '>' + tmp, 'wb')
            fp.write(imagedata)
            fp.close()
            return tmp
        except IOError, v:
            print 'JPG translation failed:', v
            os.unlink(tmp)
            return None

    #
    # ISL interface
    #

    def SetImageName(self, name):
        self.__imagename = name

    def SetImageData(self, image):
        self.__imagedata = image

    def Play(self):
        filetype = imghdr.what(None, self.__imagedata)
        #
        # TBD: Tk's PhotoImage does not support creating a GIF or PPM
        # file from a byte stream.  Why?  Because you can't pass
        # strings with null bytes around in Tcl.  So although the
        # interface is described in the Tk Photo manpage, it isn't
        # supported.  <sigh>
        #
        tmpfile = None
        if filetype == 'gif':
            tmpfile = self.__gif(self.__imagedata)
        elif filetype == 'jpeg':
            tmpfile = self.__jpeg(self.__imagedata)
        # get a viewer
        self.__top = self.__factory.get_toplevel(self.__imagename, tmpfile)

    def Close(self):
        if self.__top:
            self.__top.quit()


if __name__ == '__main__':
    main()
##     w = ImageWindow(sys.argv[1])
##     w.run()
