"""Simple I/O plugin for interactive KPs."""

import sys
import os
import ilu
from koe.common import pluginlib
from koe.interfaces.stubs import FileAPI
from koe.interfaces.stubs import SimpleIOAPI

program = 'tkstdio.py'



def main():
    """Main for the Stdio plugin."""
    kos, ns = pluginlib.register('simpleio')
    factory = SimpleIOFactory()
    serviceid = kos.bind_service('simpleio',
                                 factory,
                                 'SimpleIOAPI.SimpleIOFactory')
    try:
        kos.run()
    finally:
        kos.unbind_service(serviceid)



class SimpleIOFactory:
    def GetNewSimpleIO(self, height=None, width=None):
        args = ()
        if height:
            args = args + ('-H', `height`)
        if width:
            args = args + ('-W', `width`)
        # TBD: Do not assume `.' is on os.environ['PATH'], however
        # this does assume that the kernel chdir()s to the directory
        # containing this plugin (which is current practice)
        path = os.path.join(os.getcwd(), program)
        try:
            (sbh, pid, pids) = pluginlib.spawnserver(path, args)
            if sbh:
                return ilu.ObjectOfSBH(FileAPI.File, sbh)
            else:
                raise SimpleIOAPI.SpawnError, path
        except os.error, msg:
            raise SimpleIOAPI.SpawnError, msg

    def GetNewSimpleIOHW(self, height, width):
        "Like GetNewSimpleIO, but with designated height and width."
        return self.GetNewSimpleIO(height, width)



if __name__ == '__main__':
    main()
