"""Extended pickler class and support (with kludges!)."""

import sys
import pickle
import copy
from types import InstanceType, StringType

# XXX I'm not sure that the following comment still applies to Python 1.5.  We
# should test this and remove it if necessary.  -BAW

# KLUDGE ALERT!  Okay, so we could pickle surrogates with the changes
# in ExtendedPickler.  But because the pickler runs in restricted
# execution, we couldn't unpickle the most trivial classes in the most
# trivial modules, if those modules came out of e.g. KOSROOT/common!
# The reason was because the standard pickle module uses sys.modules,
# which doesn't contain, e.g. iotools.  See the testcases/iotest.py
# for a failure case without this patch (only if it uses the tty
# version!).
#
# Guido sez: the *right* approach would be to change
# pickle.whichmodule() so that its a method on the Pickler class, and
# instead of hardcoding its use of sys.modules, it would use a class
# instance variable `modules'.  This would be initialized to
# sys.modules.  Then ExtendedPickler could just override this instance
# variable with rexec.modules.
#
# However, Guido doesn't want to make this change now because its too
# late to fold it into Python1.4's pickle module and he doesn't want
# to distribute the KOSROOT/pythonlib directory with the alpha system.

classmap = {}
smart_modules = None

def smart_whichmodule(cls, clsname=None):
    # XXX This is copied from module pickle
    """Figure out the module in which a class occurs.

    Search sys.modules for the module.
    Cache in classmap.
    Return a module name.
    If the class cannot be found, return __main__.
    """
    if classmap.has_key(cls):
        return classmap[cls]
    if clsname is None:
        # 1.4/1.5 incompatibility: in 1.4 pickle, whichmodule()
        # takes only a cls argument.  In 1.5, the clsname is added.
        # We support both by defaulting clsname to cls.__name__.
        clsname = cls.__name__
    moditems = smart_modules.items() + sys.modules.items()
    for name, module in moditems:
        if name != '__main__' and \
           hasattr(module, clsname) and \
           getattr(module, clsname) is cls:
            break
    else:
        name = '__main__'
    classmap[cls] = name
    return name
pickle.whichmodule = smart_whichmodule


class ExtendedPickler(pickle.Pickler):
    dispatch = copy.copy(pickle.Pickler.dispatch)

    def __init__(self, file, kosbastion, rexec):
        global smart_modules
        pickle.Pickler.__init__(self, file)
        self.kosbastion = kosbastion
        smart_modules = rexec.modules

    def save_inst(self, object):
        if object is self.kosbastion:
            self.write(pickle.MARK + pickle.INST + "KOS\n__KOS_Bastion__\n")
        # catch IluObjSurr types -- ILU surrogate objects. this is
        # necessary because such objects are not unpicklable in
        # restricted execution env.  see class RExec in KPRexec.py for
        # the other end of this process
        elif hasattr(object, '_IluClass'):
            # TBD: kludgery based on observation and reading
            # pickle.py. Guido probably has a much cleaner way of
            # doing this!!
            sbh = object.IluSBH()
            self.write(pickle.MARK)
            self.save(smart_whichmodule(object.__class__))
            self.save(object.__class__.__name__)
            self.save(sbh)
            self.write(pickle.INST + "SURR\n__birth__\n")
            # yes this is redundant, but we have a new and improved
            # iluRt.py that supports __getstate__/__setstate__
            # discipline
            self.save({'sbh': sbh})
            self.write(pickle.BUILD)
        else:
##             if not self.__picklable(object):
##                 print `object`
##                 raise pickle.PicklingError, \
##                       "Won't pickle un-unpicklable instance of %s" %\
##                       (object.__class__)
            pickle.Pickler.save_inst(self, object)
    dispatch[InstanceType] = save_inst

    def __picklable(self, object):
        """Return true if the object appears picklable (and unpicklable).

        This test may not be foolproof, but it detects an __init__
        requiring arguments without a matching __getinitargs__.

        """
        if hasattr(object, "__getinitargs__"):
            # Assume it's okay
            return 1
        try:
            constructor = object.__class__.__init__
        except AttributeError:
            # No constructor -- we're safe
            return 1
        try:
            function = constructor.im_func
        except AttributeError:
            # Constructor doesn't appear to be a method -- very doubtful
            return 0
        code = function.func_code
        argcount = code.co_argcount
        defaults = function.func_defaults
        if defaults:
            argcount = argcount - len(defaults)
        return argcount <= 1            # Allow for 'self'
