"""Allow KPs access to WWW information using urllib.

The current implementation is a placeholder for a later version that
will make it easy for KOS operators to restrict the sites to which the 
KPS can connect.  For example, it should allow the operator to prevent 
access to private Web sites that use IP addresses to limit access.
"""

from koe.pythonlib.Bastion import StickyBastion
from koe.common import access, pathhack

import urlparse

from urllib import *
# only export these names to the KP
__export__ = [
    # mediated
    'urlopen',
    # export all of the none-URLopener related stuff
    'ftperrors',
    'quote',
    'quote_plus',
    'unquote',
    'unquote_plus',
    'urlencode',
    'basejoin',
    'unwrap',
    'splitattr',
    'splitgophertype',
    'splithost',
    'splitnport',
    'splitpasswd',
    'splitport',
    'splitquery',
    'splittag',
    'splittype',
    'splituser',
    'splitvalue',
    # export this local name
    'URLSecurityError'
    ]

# we probably don't want to give the KP access to all the parts of
# the object returned by urllib.urlopen, e.g. the raw socket
validBastionNames = {'read':1,
                     'readline':1,
                     'readlines':1,
                     'close':1,
                     'info':1,
                     'geturl':1,
                     }

class URLSecurityError(IOError):
    """The desired IO operation is disallowed by the security policy

    Implemented as a subclass of IOError so that existing code won't
    fail with uncaught exceptions -- and because this is a sensible
    interface for new code.
    """

class AccessChecker:
    path = pathhack.kos_root() + '/config/access.http.conf'

    def __init__(self, path=None):
        self.conf = access.LimitParser(path or self.path)

    def checkGET(self, url):
        host = urlparse.urlparse(url)[1]
        host, port = splitport(host)
        port = port or '80'
        if '.' not in host:
            raise URLSecurityError, "must use fully qualified domain name"
        if not self.conf.is_allowed_to('get', host, port):
            raise URLSecurityError, "Permission denied GET %s" % \
                  str((host, port))
        
    def checkPOST(self, url):
        host = urlparse.urlparse(url)[1]
        host, port = splitport(host)
        port = port or '80'
        if '.' not in host:
            raise URLSecurityError, "must use fully qualified domain name"
        if not self.conf.is_allowed_to('post', host, port):
            raise URLSecurityError, "Permission denied POST %s" % \
                  str((host, port))
        
_urlopen = None
_checker = None

def urlopen(fullurl, data=None):
    global _urlopen, _checker
    if not _urlopen:
        _urlopen = URLopener().open
    if not _checker:
        _checker = AccessChecker()

    # will raise URLSecurityError is this is not allowed
    if data is None:
        _checker.checkGET(fullurl)
    else:
        _checker.checkPOST(fullurl)
    
    obj = _urlopen(fullurl, data)
    # Use the sticky bastion because the info method can return an
    # instance of mimetools.Message.  Inspection shows that Message
    # and its base class rfc822.Message are both safe for KPs to use.
    return StickyBastion(obj, validBastionNames.has_key)
