"""Client of the IFinder plugin that looks for services at other KOSes.

Put a file in the KP's suitcase called "interface" that contains the
name of the interface you want to search for. The KP will report
either the list of KOSes the have the service or a message that the
IFinder didn't know anything about that interface. 

Typically, the first time you submit this KP the IFinder won't know
anything about your interface. But it will send a KP to find out about
it. So send your KP a little later and the KOS should have an answer.
"""

import time

class KP:

    def __init__(self):
        self.found = []

    def __main__(self, kos):
        iferrs = kos.all_exceptions("InterfaceAPI")
        IF = kos.lookup_service("InterfaceAPI.InterfaceFinder",
                                "InterfaceFinder").Open()
        interface = self.get_interface(kos)
        try:
            self.found = IF.Find(interface)
            print "interface %s found at KOSes" % (interface), self.found
        except iferrs.UnknownInterface:
            self.found = []
            print "interface unknown, try back later"
            
    def get_interface(self, kos):
        sfs = kos.get_suitcase()
        f = sfs.open('interface')
        buf = f.read()
        f.close()
        return buf[:-1]
