"""List the contents of the namespace at our KOS, indicating concindent
references and non-namespace terminals."""

import string
import nstools

class KP:

    def __main__(self, kos):
        ns = kos.get_namespace()
        self.listnamespace(ns)

    def listnamespace(self, ns, path="/", seen=None):
        """Print the paths in the namespace, and

         - in those cases where a path denotes a namespace already referred
           to by by another path higher at a higher level, indicate it like
           a symlink - with a "->"
         - in those cases where a path denotes a non-namespace object,
           indicate the object indented on the next line."""

        if seen is None:
            seen = {}
            descriptor = ns.GetDescriptor()
            reference = descriptor.reference()
            seen[reference] = path
        try:
            names = ns.List()
        except AttributeError:
            # ns not a namespace - treat as terminal.
            print path
            print " ", ns
            return

        names.sort()
        print pathjoin(path, '')

        todo = []
        for name in names:
            fullpath = pathjoin(path, name)
            descriptor = nstools.Lookup(ns, name)
            reference = descriptor.reference()
            if seen.has_key(reference):
                todo.append((0, fullpath, seen[reference]))
            else:
                seen[reference] = fullpath
                todo.append((1, fullpath, descriptor))
        for what, fullpath, extra in todo:
            if what == 0:
                print fullpath, "->", extra
            else:
                subns = extra.Open()
                self.listnamespace(subns, fullpath, seen)

def pathjoin(a, b):
    if b[:1] == '/': return b
    if a and a[-1] != '/': a = a + '/'
    return a+b
