#! /usr/bin/env python

"""Stress test the SuitcaseFilesystem.

Does not do extensive testing of the SFSPath or the SuitcaseFile.

Run this from the command line "python stress_suitcase.py". It will
print the initial state of the suitcase, do a bunch of tests, and
print the final state (which should match the initial state).

During the test cases, it will print 
    "Expected: blah  Raised: blah blah"
for apparently successful tests. That is, tests that raised an
exception as expected. Check the strings to make sure it is the
correct exception.
    "Error:"
will be printed when a test fails. This indicates a bug in the SFS.

Note: The current implementation doesn't grok weird pathes like
/../../.'. It will report an "Error:" here.

"""

import sys, os; sys.path.append(os.path.join(sys.path[0], "../common"))

import pathhack
import KPReprImpl
import Suitcase
import sys

# we can simulate migrate by creating a new suitcase from the repr

repr = KPReprImpl.NamedPart()
suitcase = Suitcase.SuitcaseFilesystem(repr)

def dup(src=repr):
    return Suitcase.SuitcaseFilesystem(src)

# testcases

# initial values -- assume the easy stuff works & check at the end
suitcase.mkdir('test')
suitcase.chdir('test')
f = suitcase.open('jeremy', 'w')
f.write('no cat')
f.close()
f = suitcase.open('guido', 'w')
f.write('Tom')
f.close()
suitcase.chdir('..')
suitcase.mkdir('nest1')
suitcase.mkdir('nest1/nest2')
f = suitcase.open('/nest1/nest2/egg', 'w')
f.write('nest... egg... get it?')
f.close()
suitcase.open('/test/deleteme', 'w').close()
suitcase.chdir('/')

print "Initial state of suitcase"

def show(arg, dir, files):
    print dir, "->", files

suitcase.path.walk('/', show, None)

for dir in ['/nest1', '/nest1/nest2', '/test', '/']:
    if not suitcase.path.exists(dir):
        print "Error: %s does not exist" % (dir)
        sys.exit(-1)
    if not suitcase.path.isdir(dir):
        print "Error: %s is not dir" % (dir)
        sys.exit(-1)

for file in ['/nest1/nest2/egg', '/test/jeremy', '/test/guido']:
    if not suitcase.path.exists(file):
        print "Error: %s does not exist" % (file)
        sys.exit(-1)
    if not suitcase.path.isfile(file):
        print "Error: %s is not file" % (file)
        sys.exit(-1)

# start with listdir & bad pathes of various sorts

print "\nStarting tests"

def dotry(cmd, expected, noerr):
    try:
        eval(cmd)
    except IOError, err:
        print "Expected: %s  Raised: %s" % (expected, err)
    else:
        print "Error:", noerr
        print "current suitcase:"
        suitcase.path.walk('/', show, None)
        sys.exit(-1)

suitcase.chdir('/')

# test some path stuff

try:
    suitcase.listdir(12.3)
except TypeError, err:
    print "Expected: TypeError  Received:", err
else:
    print "Error: got listing of directory 12.3 (not '12.3')"
try:
    if suitcase.listdir("/../../../") != suitcase.listdir('/'):
        print "Error: can't handle /../../../"
        # going to continue anyway
        # sys.exit(-1)
except IOError:
    print "Error: can't handle /../../../"

if suitcase.listdir("/test/../nest1/.") != suitcase.listdir('/nest1'):
    print "Error: can't handle /test/../nest1/."

# test mkdir

suitcase.mkdir('bogus')
suitcase.mkdir('/bogus/absolute')
dotry("suitcase.mkdir('bogus/morebogus/mostbogus')",
      "No such file",
      "made directory with no parent")
dotry("suitcase.mkdir('/nest1/nest2')",
      "dir exists",
      "made directory over top of existing dir")
suitcase.chdir("test")
dotry("suitcase.mkdir('../nest1/nest2/egg')",
      "file exists",
      "made dir over top of existing file")

dotry("suitcase.open('/test/jeremy', 'smash')",
      "bad mode",
      "opened a filed with mode 'smash'")
dotry("suitcase.open('/test/', 'r')",
      "opened a directory",
      "was able to open a directory")
dotry("suitcase.open('/test/roger', 'r')",
      "file doesn't exist",
      "opened non-existent file")
# should be able to open a file multiple times
f1 = suitcase.open('/test/deleteme', 'w')
f2 = suitcase.open('/test/deleteme', 'w')
f3 = suitcase.open('/test/deleteme', 'w')
f1.write('spam')
f2.write('spam spam')
f3.write('spam spam spam beans spam')
f3.close()
f2.close()
f1.close()

# explicitly skipping multiple read/writer tests

# test chdir
dotry("suitcase.chdir('/test/jeremy')",
      "can't cd to file",
      "cd to file")
dotry("suitcase.chdir('/bogus/morebogus')",
      "non-existent dir",
      "non-existent dir")

# test listdir
if suitcase.listdir() != suitcase.listdir('/test'):
    print "Error: listdir + chdir, bad interaction"
    sys.exit(-1)
dotry("suitcase.listdir('/test/jeremy')",
      "not a directory",
      "listdir of file")
dotry("suitcase.listdir('/test/nest/foo')",
      "no such file",
      "listdir of bogus path")

# test unlink
dotry("suitcase.unlink('/bogus/absolute')",
      "not a file",
      "unlinked a directory")
dotry("suitcase.unlink('/bogus/morebogus/mostbogus')",
      "no such file",
      "unlinked a non-existent file")
suitcase.unlink('/test/deleteme')

# test rmdir
dotry("suitcase.rmdir('/test/guido')",
      "not a dir",
      "rmdir a file")
dotry("suitcase.rmdir('/nest1/nest2')",
      "dir not empty",
      "rmdir a non-empty directory")
dotry("suitcase.rmdir('/quux/quam')",
      "no such file",
      "rmdir a non-existent dir")
suitcase.rmdir('/bogus/absolute')
suitcase.rmdir('/bogus')


print "testing completed\n"

suitcase.path.walk('/', show, None)
