"""Convert iluprof statistics to pstats.py format.

API.Object.Method, ClientP  -->
API[__skel].Object.Method

"""

import types
import string

StubStats = { 'total' : ('BeginCall', 'FinishCall'),
              'request' : ('BeginRequest', 'FinishRequest'),
              'reply' : ('GetReply', 'ReplyRead'),
              }

SkelStats = { 'total' : ('InputObjectID', ('FinishReply', 'NoReply')),
              'request' : ('InputObjectID', 'RequestRead'),
              'reply' : (('BeginReply', 'NoReply'),
                         ('FinishReply', 'NoReply')),
              }


def compute_times(how, times):
    def match(dir, pattern, event):
        if type(pattern) != types.TupleType:
            names = (pattern,)
        else:
            names = pattern
        if event[1] == dir:
            for name in names:
                if event[0] == name:
                    return event[2]
        return None
        
    begin, end = how
    tbegin = tend = None
    for event in times:
        if tbegin is None:
            tbegin = match(1, begin, event)
        if tend is None:
            tend = match(2, end, event)
            if tend:
                break
    try:
        return tend - tbegin
    except TypeError:
        # didn't find a tbegin or tend
        raise ValueError, ("missing timing info", how, times)
    
def make_name(client, object, method, sub=''):
    if sub:
        method = "%s.%s" % (method, sub)
    i = string.find(object, '.')
    api = object[:i]
    obj = object[i+1:]
    if not client:
        api = api + "__skel"
    return (api, obj, method)

class Stats:

    def __init__(self):
        self.stats = {}

    def add(self, name, itime, ctime, subfunc={}):
        try:
            nc, ns, it, ct, callees = self.stats[name]
        except KeyError:
            nc = 0
            ns = 0
            it = 0.0
            ct = 0.0
            callees = {}
        nc = nc + 1
        ns = ns + 1
        it = it + itime
        ct = ct + ctime
        for k, v in subfunc.items():
            if callees.has_key(k):
                callees[k] = callees[k] + v
            else:
                callees[k] = v
        self.stats[name] = (nc, ns, it, ct, callees)

def convert(entries):
    stats = Stats()
    for entry in entries:
        try:
            kind, (object, method, instance), timing = entry
        except (ValueError, TypeError):
            # ignore bugs created by the profiling code. sigh.
            continue
        if kind:
            how = StubStats
        else:
            how = SkelStats

        tot = compute_times(how['total'], timing)
        req = compute_times(how['request'], timing)
        try:
            rep = compute_times(how['reply'], timing)
        except ValueError:
            rep = 0.0

        itime = tot - (req + rep)
        name = make_name(kind, object, method)
        stats.add(name, itime, tot)

        req_name = make_name(kind, object, method, 'request')
        stats.add(req_name, req, req, {name:1})

        if rep:
            rep_name = make_name(kind, object, method, 'reply')
            stats.add(rep_name, rep, rep, {name:1})

    return stats.stats

if __name__ == "__main__":
    import sys
    import marshal

    src = open(sys.argv[1])
    marshal.load(src)         # ignore header
    lines = []
    while 1:
        try:
            line = marshal.load(src)
        except EOFError:
            break
        lines.append(line)
    stats = convert(lines)
    try:
        dest = sys.argv[2]
    except IndexError:
        dest = sys.argv[1] + '.prof'
    f = open(dest, 'w')
    marshal.dump(stats, f)
    f.close()
