"""A read_eval_print command loop within a KP - take a KP for a spin.

Very like the Python command loop, takes and executes commands from stdin,
within the context of a KP __main__ section.

        - Results are printed to stdout
        - EOF on stdin or exception SystemExit force exit
        - Exceptions are handled
        - multiline statements work (mostly).
"""

import sys, string

class KProot:

    def __main__(self, kos):
        self.kos = kos
        self.__get_identity()
        self.__interact()

    def __interact(self, input=None):
        """Read/eval/print loop.

        Prompt indicates our identity, exceptions are handled nicely, and
        SystemExit exception or EOF on stdin force us to exit."""

        # locals is a global instead of on KP's self so it doesn't get pickled.
        global local_context

        kos = self.kos

        # Initialize local_context only once.
        try:
            local_context
        except NameError:
            local_context = {'self': self, 'kos': kos}

        main_prompt, cont_prompt = '>>>', '...'
        prompt = main_prompt
        _ = None
        once_only = input and 1

        while 1:

            if not input:
                # Not a recursive one-shot - obtain input:
                self.__prompt(main_prompt)
                input = sys.stdin.readline()
            try:
                if not input:
                    raise SystemExit, "End of input"
                _ = eval_or_exec(input, globals(), local_context)
                if _ is not None:
                    print `_`
                    local_context['_'] = _
            except SystemExit, exc_value:
                print "%s, exiting..." % str(exc_value)
                sys.exit(exc_value)
            except SyntaxError, msg:
                if msg[0] == 'unexpected EOF while parsing':
                    # Accumulate until a blank line:
                    accume = input
                    while 1:
                        self.__prompt(cont_prompt)
                        input = sys.stdin.readline()
                        if not string.strip(input):
                            break
                        else:
                            accume = accume + input
                    # ... and recurse for one-shot evaluation:
                    _ = self.__interact(accume)
                else:
                    flag_syntax_error(msg)
            except:
                if sys.exc_value:
                    print "%s: %s" % (str(sys.exc_type),
                                          str(sys.exc_value))
                else:
                    print "%s" % str(sys.exc_type)

            if once_only:
                return _
            else:
                input = ''

    def __prompt(self, which):
        sys.stdout.write("KP %s %s " % (self.__identity, which))
        sys.stdout.flush()

    def __get_identity(self):
        """Create identity that identifies origin and current kp handles.

        In the original KP the identity is the same as kos.get_kphandle().
        In clones, it is the original identity plus "|" and the new
        kphandle, leaving off kos name if it's the same as original's."""
        handle, kos_name = self.kos.get_kphandle(), self.kos.get_kos_name()
        if not hasattr(self, 'original'):
            self.__identity = self.__original = handle
            self.__original_kos = kos_name
        else:
            self.__identity = self.__original + "|" + handle

class KP(KProot):
    pass

def eval_or_exec(line, globals, locals):
    """Eval line or, if syntax error, exec it, both wrt globals and locals."""
    try:
        return eval(line, globals, locals)
    except SyntaxError:
        exec line in globals, locals
        return None

def flag_syntax_error(msg):
    type, (f, l, c, t) = msg
    print "  Line %d:" % l
    print "   %s" % t[:-1]
    print "  %s^" % (" " * c)
    print "SyntaxError:", type
