#! /usr/bin/env python

"""Script to create a MIME file of type multipart/knowbot.

Usage: makemime [options] module ...

Options are:
-k class:      class to instantiate (default KP)
-m module:     main module (default first module argument)
-o file:       output file (default stdout)

File arguments:
module ...:    one or more Python modules
"""


# path hackery, no longer requires $PYTHONPATH to be set properly
import sys, os; sys.path.insert(0, os.path.join(sys.path[0], "../../"))

# Standard python imports
import os
import sys
import string
import getopt

# More imports
from koe.common import kptools


# Local exception
Error = "makemime.Error"


# Main program
def main():
    ofile = "-"
    knowbot_class = kptools.DEFAULT_KP_CLASS
    main_module = None
    try:
        opts, args = getopt.getopt(sys.argv[1:], "k:m:o:")
    except getopt.error, msg:
        usage(msg)
    for o, a in opts:
        if o == '-k':
            knowbot_class = a
        if o == '-m':
            main_module = a
        if o == '-o':
            ofile = a
    if not args:
        usage("at least one module argument required")
    try:
        repr = kptools.Representation(args[1:], main=args[0],
                                      klass=knowbot_class)
    except (IOError, kptools.LoadingError), msg:
        error(msg)
    metadata = repr.GetMetadata()
    mimedata = repr.FreezeDry()
    if ofile and ofile != "-":
        ofp = open(ofile, "w")
    else:
        ofp = sys.stdout
    ofp.write(mimedata)
    if ofp is not sys.stdout:
        ofp.close()


def usage(msg):
    sys.stdout = sys.stderr
    print "Usage error:", msg
    for line in string.splitfields(__doc__, '\n')[1:]:
        print line
    sys.exit(2)


def error(msg):
    sys.stdout = sys.stderr
    print "Error:", msg
    sys.exit(1)


if __name__ == '__main__':
    main()

