#! /usr/bin/env python

"""Namespace robustness test.

This file works in conjunction with nsrebind2.py, in the same
directory.  Start this program up first (there must be a Worldroot
running).  This binds a couple of contexts into the /tools directory.

Next start the nsrebind2.py program and watch as it continually lists
a context resolved in the nsrebind1.py process.  At any time you can
kill the nsrebind1.py process and if you restart it before the next
listing, you will see the new nstools.Context object re-resolve the
path to get the new back-end object.

"""

import sys, os; sys.path.insert(0, os.path.join(sys.path[0], '../../'))
from koe.common import nstools
import ilutools
from koe.common import NamespaceImpl
from koe.interfaces.stubs Namespace, Namespace__skel



class NullContext(Namespace__skel.Context):
    def Lookup1(self, component):
        raise Namespace.ResolutionError, component

    def List(self):
        return []

    def Bind(self, name, desc):
        raise Namespace.BindError, name

    def Unbind(self, name):
        raise Namespace.BindError, name

    def GetDescriptor(self):
        return {'typename': 'Namespace.Context',
                'reference': self.IluSBH(),
                }


class StaticContext(NullContext):
    def __init__(self, *names):
        self.__bindings = {}
        for name in names:
            self.__bindings[name] = NullContext().GetDescriptor()

    def Lookup1(self, component):
        try:
            return self.__bindings[component]
        except KeyError:
            raise Namespace.ResolutionError, component

    def List(self):
        return self.__bindings.keys()



def main():
    two = StaticContext('three', 'four', 'five')
    one = NamespaceImpl.Context()
    one.Bind('two', two.GetDescriptor())

    ns = nstools.WorldOpen('tools')
    ns.Bind('one', one.GetDescriptor())

    print 'Subcontext started'
    ilutools.RunInterruptableMainLoop()
    print 'Subcontext stopped'

    ns.Unbind('one')



if __name__ == '__main__':
    main()
