"""KP construction dialog."""

import copy
import Dialog
import os
import string
import sys
import Tkinter ; Tk = Tkinter
from koe.common import tktools
from koe.common import kptools
from koe.common import physfonts

from LaunchDialog import Caller, KernelChooser, NO_KERNEL_FOUND
from LaunchDialog import RSLoadFileDialog, RSSaveFileDialog
from LaunchDialog import make_group, GROUP_LABEL_FONT

FONT_MODULE = physfonts.TT_R_M
FONT_MODULE_MAIN = physfonts.TT_B_M

IOMECH_KOS = 'kos-console'
IOMECH_NULL = 'null'
IOMECH_LAUNCHER = 'launcher'

VALID_IOMECHS = (IOMECH_KOS, IOMECH_NULL, IOMECH_LAUNCHER)



class BuilderDialog:

    __main_class_str = "KP"
    __xheaders = None

    def __init__(self, master, app, launcher, filename, kosname):
        self.__app = app
        self.__launcher = launcher
        self.__kernel = kosname
        ioservers = {'stdin': IOMECH_LAUNCHER,
                     'stdout': IOMECH_LAUNCHER,
                     'stderr': IOMECH_LAUNCHER}
        self.__ioservers = ioservers
        top = self.__top = Tk.Toplevel(master)
        top.iconname("KP Composer")
        self.__set_file(filename)
        self.__main_class = Tk.StringVar(top)
        self.__metadata = {}
        self.__modules = {}
        modname, ext = '', None
        if filename:
            modname, ext = os.path.splitext(os.path.basename(filename))
        self.__main_module = modname
        if ext == ".tmpl":
            self.__load_template(filename)
        elif ext == ".mime":
            self.__main_module = ''
        elif modname:
            self.__modules[modname] = filename
        self.__main_class.set(self.__main_class_str)
        self.create_widgets(master, kosname)
        if self.__main_module:
            self.select_module(self.__main_module)
        else:
            self.__class_entry['state'] = Tk.DISABLED
            self.__class_entry['foreground'] = self.__disabled_foreground
            self.__set_main_button['state'] = Tk.DISABLED
        if ext == ".mime":
            self.warning("Not Yet Implemented",
                         "Loading MIME files into the KP Composer is not"
                         " yet supported.")

    def create_widgets(self, master, kosname):
        botfr = Tk.Frame(self.__top, borderwidth='1m')
        botfr.pack(side=Tk.BOTTOM, fill=Tk.X)
        topfr = Tk.Frame(self.__top, borderwidth='1m')
        topfr.pack(expand=1, fill=Tk.BOTH)

        buttons = []
        btn = Tk.Button(botfr, text="Submit", command=self.__submit_cmd)
        btn.pack(side=Tk.LEFT)
        buttons.append(btn)
        btn = Tk.Button(botfr, text="Save", command=self.__save_cmd)
        btn.pack(side=Tk.LEFT, padx='1m')
        buttons.append(btn)
        btn = Tk.Button(botfr, text="Save As...", command=self.__save_as_cmd)
        btn.pack(side=Tk.LEFT)
        buttons.append(btn)
        btn = Tk.Button(botfr, text="Close", command=self.__close_cmd)
        self.__disabled_foreground = btn['disabledforeground']
        btn.pack(side=Tk.RIGHT)
        buttons.append(btn)

        apply(tktools.unify_button_widths, tuple(buttons))

        # modules editor:
        modulefr = make_group(topfr, "modules", "Modules:",
                              fill=Tk.BOTH, expand=1, side=Tk.LEFT)
        modulebotfr = Tk.Frame(modulefr)
        modulebotfr.pack(fill=Tk.X, side=Tk.BOTTOM)
        moduletopfr = Tk.Frame(modulefr)
        moduletopfr.pack(expand=1, fill=Tk.BOTH)
        modlistbox, fr = self.__make_list_box(moduletopfr)
        rightfr = Tk.Frame(moduletopfr)
        rightfr.pack(side=Tk.RIGHT, fill=Tk.Y)
        entry, frame, label = tktools.make_labeled_form_entry(
            rightfr, "Module", labelwidth=7, entrywidth=16)
        entry.bind("<Return>", self.__set_modulename_cmd)
        self.__module_entry = entry
        entry, frame, label = tktools.make_labeled_form_entry(
            rightfr, "Class", labelwidth=7, entrywidth=16)
        entry.bind("<Return>", self.__set_classname_cmd)
        entry['textvariable'] = self.__main_class
        self.__class_entry = entry
        self.__class_label = label
        button = Tk.Button(rightfr, text="Make Main Module",
                           command=self.__set_main_cmd, state=Tk.DISABLED)
        button.pack(side=Tk.BOTTOM)
        self.__set_main_button = button
        fr = Tk.Frame(modulefr)
        fr.pack(fill=Tk.X)
        Tk.Label(fr, text="File:").pack(side=Tk.LEFT)
        label = self.__module_filename = Tk.Label(fr, font=physfonts.BODY_R_M,
                                                  anchor=Tk.W)
        label.pack(side=Tk.LEFT, fill=Tk.X, expand=1)
        Tk.Frame(modulefr, height='1m').pack()
        self.__modlistbox = modlistbox
        #
        newmodulebtn = Tk.Button(modulebotfr, text="Add",
                                 command=self.__add_module_cmd)
        newmodulebtn.pack(side=Tk.LEFT)
        delmodulebtn = Tk.Button(modulebotfr, text="Remove",
                                 command=self.__delete_module_cmd)
        delmodulebtn.pack(side=Tk.RIGHT)
        tktools.unify_button_widths(newmodulebtn, delmodulebtn)
        self.__delmodulebtn = delmodulebtn
        self.__build_modules_list()

        # just a spacer
        Tk.Frame(topfr, width=8).pack(side=Tk.LEFT)

        # stream editor:
        rightfr = Tk.Frame(topfr)
        rightfr.pack(expand=1, fill=Tk.Y)
        streamfr = make_group(rightfr, "streams", "I/O Services:",
                              fill=Tk.BOTH, expand=1)
        self.__make_stream_widgets(streamfr, "stdin", "Standard Input:")
        self.__make_stream_widgets(streamfr, "stdout", "Standard Output:")
        self.__make_stream_widgets(streamfr, "stderr", "Standard Error:")

        # destination chooser:
        Tk.Frame(rightfr, height=8).pack()
        destfr = make_group(rightfr, "kernel", "Destination kernel:",
                            fill=Tk.X)
        Tk.Label(destfr, text="Kernel: ").pack(side=Tk.LEFT)
        kc = KernelChooser(destfr, self.__pick_kernel, kosname)
        kc.pack(side=Tk.LEFT, fill=Tk.BOTH)
        if kosname != NO_KERNEL_FOUND:
            kc.require(kosname)
        #
        self.__module_filename['text'] = string.lowercase
        top = self.__top
        cmd = self.__close_cmd
        top.protocol("WM_DELETE_WINDOW", cmd)
        top.bind("<Alt-W>", cmd)
        top.bind("<Alt-w>", cmd)
        top.update_idletasks()
        reqwidth = top.winfo_reqwidth()
        reqheight = top.winfo_reqheight()
        strwidth = self.__module_filename.winfo_reqwidth()
        self.__module_filename['text'] = ''
        top.update_idletasks()
        top.minsize(reqwidth, reqheight)
        perchar = strwidth / len(string.lowercase)
        self.__filename_perchar = perchar

    def __make_list_box(self, master):
        from TextListbox import TextListbox
        hbar, vbar, fr = tktools.make_scrollbars(
            master, 0, 1, 1, name="listbox")

        box = TextListbox(fr, name="listbox", height=1, width=30,
                          exportselection=0, font=FONT_MODULE, takefocus=0,
                          yscrollcommand=vbar.set
                          )
        box.pack(expand=1, fill=Tk.BOTH, side=Tk.LEFT)
        vbar['command'] = box.yview
        box.bind("<Button-1>", self.__modlist_click)
        box.bind("<ButtonRelease-1>", self.__modlist_click)
        fr.pack(expand=1, fill=Tk.BOTH, side=Tk.LEFT)
        return box, fr

    def __make_stream_widgets(self, frame, stream, label):
        f = Tk.Frame(frame)
        f.pack(fill=Tk.X)
        Tk.Label(f, text=label, font=GROUP_LABEL_FONT, anchor=Tk.W
                 ).pack(fill=Tk.X)
        var = Tk.StringVar(self.__top)
        var.set(self.__ioservers[stream])
        Tk.Radiobutton(f, anchor=Tk.W, text="KOS ",
                       variable=var, value=IOMECH_KOS,
                       command=Caller(self.__pick_iomech, stream, IOMECH_KOS)
                       ).pack(fill=Tk.X, side=Tk.LEFT)
        Tk.Radiobutton(f, anchor=Tk.W, text="Window ",
                       variable=var, value=IOMECH_LAUNCHER,
                       command=Caller(self.__pick_iomech, stream, IOMECH_LAUNCHER)
                       ).pack(fill=Tk.X, side=Tk.LEFT)
        Tk.Radiobutton(f, anchor=Tk.W, text="Null",
                       variable=var, value=IOMECH_NULL,
                       command=Caller(self.__pick_iomech, stream, IOMECH_NULL)
                       ).pack(fill=Tk.X, side=Tk.LEFT)

    def __add_module_cmd(self, event=None):
        ModuleDialog(self.__top, self, self.__app)

    def add_module(self, modname, filename):
        if self.__modules.has_key(modname):
            raise ValueError, "Module name %s already in use." % modname
        self.__modules[modname] = filename
        if len(self.__modules) == 1:
            self.__main_module = modname
        self.__modlistbox.select_clear(0, Tk.END)
        self.__build_modules_list()
        self.select_module(modname)

    def select_module(self, modname):
        mods = self.__modules.keys()
        mods.sort()
        self.__modlistbox.select_clear(0, Tk.END)
        self.__modlistbox.select_set(mods.index(modname))
        filename = self.__modules[modname]
        fnwidth = self.__module_filename.winfo_width()
        numchars = int(fnwidth / self.__filename_perchar)
        need_ellipsis = 0
        while len(filename) > numchars and os.sep in filename:
            pos = string.index(filename, os.sep)
            filename = filename[pos + 1:]
            need_ellipsis = 1
        if len(filename) > numchars:
            filename = "..." + filename[- numchars:]
        elif need_ellipsis:
            filename = ".../" + filename
        self.__module_filename['text'] = filename
        self.__module_entry['state'] = Tk.NORMAL
        self.__module_entry.delete(0, Tk.END)
        self.__module_entry.insert(0, modname)
        if modname == self.__main_module:
            self.__class_entry['state'] = Tk.NORMAL
            self.__class_entry['foreground'] = "black"
            self.__set_main_button['state'] = Tk.DISABLED
        else:
            self.__class_entry['state'] = Tk.DISABLED
            self.__class_entry['foreground'] = self.__disabled_foreground
            self.__set_main_button['state'] = Tk.NORMAL

    def __modlist_click(self, event):
        try:
            entry = self.__modlistbox.get(Tk.At(event.x, event.y))[0]
        except (Tk.TclError, IndexError):
            try:
                entry = self.__modlistbox.get(Tk.END)[0]
            except (Tk.TclError, IndexError):
                return
        modname = string.split(entry)[0]
        self.select_module(modname)

    def __delete_module_cmd(self, event=None):
        modname, index = self.__which_module()
        if not modname:
            self.warning("No Module Selected",
                         "You must select a module before attempting "
                         "a deletion.")
            return
        del self.__modules[modname]
        self.__modlistbox.select_clear(0, Tk.END)
        self.__modlistbox.delete(index)
        if index >= len(self.__modules):
            index = len(self.__modules) - 1
        if index >= 0:
            mods = self.__modules.keys()
            mods.sort()
            self.select_module(mods[index])
            if len(self.__modules) == 1 \
               and self.__main_module != self.__modules.keys()[0]:
                self.__set_main_cmd()
        if not self.__modules:
            self.__delmodulebtn['state'] = Tk.DISABLED

    def __set_main_cmd(self):
        modname, index = self.__which_module()
        if not modname:
            self.warning("Module Error",
                         "Select a module before attempting to the"
                         " main module of the KP.")
            return
        self.__main_module = modname
        self.__build_modules_list()
        self.select_module(modname)

    def __set_modulename_cmd(self, event):
        origname, index = self.__which_module()
        newname = string.strip(self.__module_entry.get())
        if newname == origname:
            return
        if not newname:
            self.__module_entry.delete(0, Tk.END)
            self.__module_entry.insert(0, origname)
            return
        if self.__modules.has_key(newname):
            self.warning("Module Error",
                         "Module name %s already in use." % newname)
            return
        if origname != self.__main_module and not check_identifier(newname):
            self.warning("Module Error",
                         "Illegal character in module name.")
            self.__module_entry.delete(0, Tk.END)
            self.__module_entry.insert(0, origname)
            return
        self.__modules[newname] = self.__modules[origname]
        del self.__modules[origname]
        self.__modlistbox.select_clear(0, Tk.END)
        self.__build_modules_list()
        if self.__main_module == origname:
            self.__main_module = newname
        self.select_module(newname)

    def __set_classname_cmd(self, event):
        origname = self.__main_class_str
        newname = string.strip(self.__main_class.get())
        if newname == origname:
            return
        if not newname:
            self.__class_entry.delete(0, Tk.END)
            self.__class_entry.insert(0, self.__main_class_str)
            return
        if not check_identifier(newname):
            self.warning("Class Error",
                         "Illegal character in class name.")
            self.__class_entry.delete(0, Tk.END)
            self.__class_entry.insert(0, origname)
            return
        self.__main_class_str = newname
        self.__module_entry.focus_set()

    def __which_module(self):
        # return name and index of currently selected module, or (None, None)
        results = self.__modlistbox.curselection()
        if results:
            s = self.__modlistbox.get(results[0])[0]
            modname = string.split(s)[0]
            return modname, results[0]
        return None, None

    __mainindex = None
    def __build_modules_list(self):
        mods = self.__modules.items()
        mods.sort()
        lines = []
        index = 0
        mainindex = -1
        for mod, filename in mods:
            basename = os.path.basename(filename)
            lines.append("%s from %s" % (mod, basename))
            if mod == self.__main_module:
                mainindex = index
            index = index + 1
        lines.insert(0, Tk.END)
        self.__modlistbox.delete(0, Tk.END)
        apply(self.__modlistbox.insert, tuple(lines))
        # set font for main module:
        self.__mainindex = mainindex
        if mainindex != -1:
            self.__modlistbox.itemconfig(mainindex, font=FONT_MODULE_MAIN)
        if self.__modules:
            self.__delmodulebtn['state'] = Tk.NORMAL
        else:
            self.__delmodulebtn['state'] = Tk.DISABLED

    def __pick_iomech(self, stream, iomech):
        self.__ioservers[stream] = iomech

    def __pick_kernel(self, kosname):
        self.__kernel = kosname

    def __close_cmd(self, event=None):
        self.__top.destroy()

    def __submit_cmd(self, event=None):
        if not self.__modules:
            self.warning("Representation Error",
                         "All KPs must have at least one module.")
        elif not self.__main_module:
            self.warning("Representation Error",
                         "Main module is not specified!")
        else:
            repr, io = self.__build_repr()
            if not repr: return
            repid = repr.GetMetadata().GetValue("reporting-id")
            reporter = self.get_launcher()
            kosname = self.__kernel
            _repid, kpid = repr.submit(kosname=kosname, repid=repid,
                                       reporter=reporter)
            # _repid should be the same as repid
            self.__app.select_kp(repid)
            kpinfo = self.__app.fetch_kpinfo(repid)
            kpinfo.set_ancestors(root=repid)
            if io: kpinfo.set_console(io)

    def __load_template(self, filename):
        try:
            fp = open(filename)
        except IOError, err:
            self.file_error(filename, err)
            self.__close_cmd()
            return
        import KPTemplate
        mheaders, theaders, self.__xheaders = KPTemplate.parse(fp)
        fp.close()
        if mheaders.has_key("knowbot-class"):
            self.__main_class_str = mheaders["knowbot-class"]
            del mheaders["knowbot-class"]
        if mheaders.has_key("main-module"):
            self.__main_module = mheaders["main-module"]
            del mheaders["main-module"]
        else:
            self.__main_module = ''
        self.__metadata = mheaders
        #
        if theaders.has_key("kernel"):
            kernel = theaders["kernel"]
            if kernel == NO_KERNEL_FOUND or not kernel:
                kernel = self.__kernel
            self.__kernel = kernel
        for stream, iotype in self.__ioservers.items():
            key = "stream-" + stream
            if theaders.has_key(key):
                iotype = theaders[key]
                if iotype not in VALID_IOMECHS:
                    iotype = IOMECH_LAUNCHER
                self.__ioservers[stream] = iotype
        if not theaders.has_key("modules"):
            return
        modules = string.split(theaders["modules"])
        basedir = os.path.dirname(filename)
        altdir = basedir
        if theaders.has_key("modules-base"):
            altdir = theaders["modules-base"] or basedir
        for modname in modules:
            if theaders.has_key("module-" + modname):
                modfile = theaders["module-" + modname]
            else:
                modfile = modname + ".py"
            if os.path.isfile(os.path.join(basedir, modfile)):
                modfile = os.path.normpath(
                    os.path.join(basedir, modfile))
            elif altdir and os.path.isfile(os.path.join(altdir, modfile)):
                modfile = os.path.normpath(
                    os.path.join(altdir, modfile))
            elif altdir == basedir:
                self.warning("KP Specification Error",
                             "Could not locate module %s in %s."
                             % (modname, basedir))
                modfile = ''
            else:
                self.warning("KP Specification Error",
                             "Could not locate module %s in %s or %s."
                             % (modname, basedir, altdir))
                modfile = ''
            self.__modules[modname] = modfile

    def __save_as_cmd(self):
        ext = "tmpl"
        if self.__sourcefile:
            xxx, ext = os.path.splitext(self.__sourcefile)
        fp, filename = self.__get_file(ext)
        if fp:
            xxx, ext = os.path.splitext(filename)
            if ext == ".mime":
                self.__write_mime(fp)
            else:
                self.__write_template(fp, filename)
            self.__set_file(filename)

    def __save_cmd(self):
        junk, ext = os.path.splitext(self.__sourcefile)
        if ext == ".py" or not self.__sourcefile:
            self.__save_as_cmd()
            return
        try:
            fp = open(self.__sourcefile, 'w')
        except IOError, err:
            self.file_error(self.__sourcefile, err)
            self.__save_as_cmd()
        else:
            if ext == ".mime":
                self.__write_mime(fp)
            else:
                self.__write_template(fp, self.__sourcefile)

    def __write_template(self, fp, filename):
        from KPTemplate import DESCRIPTOR_START
        write = fp.write
        write("# This file was generated by %s.\n\n"
              % os.path.basename(sys.argv[0]))
        write("main-module: %s\n" % self.__main_module)
        write("knowbot-class: %s\n" % self.__main_class_str)
        write("\n")
        write(DESCRIPTOR_START + "kernel: %s\n" % self.__kernel)
        write("\n")
        basedir = os.path.dirname(filename)
        write(DESCRIPTOR_START + "modules-base: %s\n" % basedir)
        write(DESCRIPTOR_START + "modules: %s\n"
              % string.join(self.__modules.keys()))
        for modname, modfile in self.__modules.items():
            moddir = os.path.dirname(modfile)
            reldir = get_relative_dir(moddir, basedir)
            modfile = os.path.join(reldir, os.path.basename(modfile))
            write(DESCRIPTOR_START + "module-%s: %s\n" % (modname, modfile))
        write("\n")
        for stream, type in self.__ioservers.items():
            write(DESCRIPTOR_START + "stream-%s: %s\n" % (stream, type))
        if self.__xheaders:
            write("\n# tool-specific headers\n\n")
            for name, headers in self.__xheaders.items():
                for key, value in headers.items():
                    write("%s%s--%s: %s\n"
                          % (DESCRIPTOR_START, name, key, value))
                write("\n")
        if self.__metadata:
            write("\n# additional metadata values;\n")
            write("# these should be passed directly into the KP\n\n")
            for item in self.__metadata.items():
                write("%s: %s\n" % item)
        write("\n# add additional metadata values here\n")
        fp.close()

    def __write_mime(self, fp):
        # use a bogus prefix to avoid becoming visible in the UI.
        prefix = "*MIME*-" + self.__main_module
        # this can lose console objects, but does clean up the widgets
        # a little.
        repr, io = self.__build_repr(prefix=prefix)
        if not repr: return
        fp.write(repr.FreezeDry())
        fp.close()
        if io: io.destroy()

    def __set_file(self, filename):
        if filename:
            self.__app.set_directory(os.path.dirname(filename))
        self.__sourcefile = filename
        self.__update_title()

    def file_error(self, filename, (errno, msg), what=''):
        self.warning("File Error",
                     "Could not open %s%s: %s (errno=%d)."
                     % (os.path.basename(filename), what, msg, errno))

    def warning(self, title, text, bitmap='error'):
        Dialog.Dialog(self.__top, title="KP Composer: " + title,
                      text=text,
                      default=0, strings=("OK",), bitmap=bitmap)

    def __get_file(self, filetype):
        if filetype[0] == ".":
            filetype = filetype[1:]
        if filetype == "py":
            filetype = "tmpl"
        while 1:
            filename = self.__get_filename(filetype)
            if not filename:
                return None, None       # <Cancel>
            try:
                fp = open(filename, "w")
            except IOError, err:
                self.file_error(filename, err, " for writing")
            else:
                return fp, filename

    def __get_filename(self, filetype):
        fd = SaveKPDialog(self.__top, filetype)
        if self.__main_module:
            default = "%s.%s" % (self.__main_module, filetype)
        else:
            default = ''
        filename = fd.go(dir_or_file=self.__app.get_directory(),
                         pattern="*." + filetype, default=default)
        return filename

    def __update_title(self):
        title = "KP Composer"
        if self.__sourcefile:
            title = "%s: %s" % (title, os.path.basename(self.__sourcefile))
        self.__top.title(title)

    def __build_repr(self, prefix=None):
        mheaders = copy.copy(self.__metadata)
        mheaders["knowbot-class"] = self.__main_class_str
        mheaders["main-module"] = self.__main_module
        theaders = {"kernel": self.__kernel,
                    "modules": string.join(self.__modules.keys())}
        for stream, type in self.__ioservers.items():
            theaders["stream-" + stream] = type
        # still need modules-base and module-* headers...
        xheaders = copy.copy(self.__xheaders)
        launcher = self.get_launcher()
        repid = launcher.generate_id(prefix or self.__main_module)
        repr = kptools.Representation()
        io = self.__handle_stdio(repid, repr)
        for modname, filename in self.__modules.items():
            repr.add_module(modname, open(filename).read())
        mheaders["reporting-id"] = repid
        #
        return repr, io

    def __handle_stdio(self, repid, repr):
        need_io = stdin = stdout = stderr = 0
        io = None
        need_io = IOMECH_LAUNCHER in (self.__ioservers['stdin'],
                                      self.__ioservers['stdout'],
                                      self.__ioservers['stderr'])
        if need_io:
            io, stdin, stdout, stderr = self.get_launcher().make_kp_console(
                repid, self.__ioservers['stdin'] == IOMECH_LAUNCHER,
                self.__ioservers['stdout'] == IOMECH_LAUNCHER,
                self.__ioservers['stderr'] == IOMECH_LAUNCHER)
        if self.__ioservers['stdin'] == IOMECH_NULL:
            stdin = 'null'
        if self.__ioservers['stdout'] == IOMECH_NULL:
            stdout = 'null'
        if self.__ioservers['stderr'] == IOMECH_NULL:
            stderr = 'null'
        meta = repr.GetMetadata()
        if stdin:
            meta.SetValue('stdin-server', stdin)
        if stdout:
            meta.SetValue('stdout-server', stdout)
        if stderr:
            meta.SetValue('stderr-server', stderr)
        #
        return io

    def get_launcher(self):
        if not self.__launcher:
            self.__launcher = self.__app.get_launcher(withdrawn=1)
        return self.__launcher


class ModuleDialog(RSLoadFileDialog):

    title = "KP Module Selection Dialog"

    def __init__(self, master, builder, app):
        self.__app = app
        self.__builder = builder
        RSLoadFileDialog.__init__(self, master)
        tktools.unify_button_widths(self.ok_button, self.filter_button,
                                    self.cancel_button)
        tktools.set_transient(self.top, master)
        self.set_filter(app.get_directory(), "*.py")
        self.filter_command()

    def ok_command(self):
        filename = self.selection.get()
        if not os.path.exists(filename):
            self.__builder.warning("Module Error",
                                   "Modules may only be added to the KP"
                                   " from existing files.")
            return
        if not os.path.isfile(filename):
            self.__builder.warning("File Error",
                                   "Selected name is not an existing file.")
            return
        try:
            fp = open(filename)
        except IOError, err:
            self.__builder.file_error(filename, err)
            return
        fp.close()
        modname = os.path.splitext(os.path.basename(filename))[0]
        try:
            self.__builder.add_module(modname, filename)
        except ValueError, msg:
            self.__builder.warning("Module Error", msg)
            return
        self.__builder = None
        self.cancel_command()

    def cancel_command(self):
        filename = self.selection.get()
        if filename:
            self.__app.set_directory(os.path.dirname(filename))
        self.top.destroy()


class SaveKPDialog(RSSaveFileDialog):

    title = "Save KP File Selection Dialog"

    def __init__(self, master, filetype, *args, **kw):
        apply(RSSaveFileDialog.__init__, (self, master) + args, kw)
        self.__do_tk_magic(master, filetype)
        dir, pat = self.get_filter()
        self.set_filter(dir, "*." + filetype)
        self.filter_command()

    def __do_tk_magic(self, master, filetype):
        self.ok_button['text'] = "Save"
        self.add_filetype_selector(
            [("Template", "tmpl"), ("MIME", "mime")], filetype)
        tktools.set_transient(self.top, master)


def check_identifier(name):
    # `name' must be non-empty
    legalchars = string.letters + string.digits + "_"
    for c in name:
        if c not in legalchars:
            return 0
    return 1


def get_relative_dir(dirname, relative_to):
    """Get relative path to `dirname' from `relative_to'."""
    dirpath = string.splitfields(dirname, os.sep)
    relative_to = string.splitfields(relative_to, os.sep)
    changed = 0
    while dirpath and relative_to and dirpath[0] == relative_to[0]:
        del dirpath[0]
        del relative_to[0]
        changed = changed + 1
    if changed < 2:
        return dirname
    dirpath = (len(relative_to) * [os.pardir]) + dirpath
    return string.joinfields(dirpath, os.sep)
