"""Window for showinf detail information for a single KP.

These windows can support only a single KP at a time, but output for many
KPs can be supported in round-robin fashion.  The object defined here provides
a 'fill-in form' interface for KP status information.
"""


import os

from FormWindow import FormWindow, BOLD_FONT
from FormWindow import LABEL, STRING, INT, REAL, DATETIME


class DetailWindow(FormWindow):
    def __init__(self, master, **kw):
        apply(FormWindow.__init__, (self, master, _detail_form), kw)
        w = self.get_window()
        w["background"] = "#cccccc"
        w.pack(expand = 1, fill = "both")

    def __setitem__(self, name, value):
        if name == "corefile":
            self["corefile-label"] = (value and "Core file:") or None
            if value and len(value) > 5:
                if value[:5] == "file:":
                    value = value[5:]
                    cwd = os.getcwd()
                    if len(value) > (len(cwd) + len(os.sep)) \
                       and value[:len(cwd)+len(os.sep)] == cwd+os.sep:
                        value = value[len(cwd)+len(os.sep):]
        elif name == "terminate-time":
            self["terminate-label"] = (value and "Termination time:") or None
        FormWindow.__setitem__(self, name, value)

D = (20,)                               # temp., for readability of the table
L = (-10,)
R = (10,)
#  This is the form data we use to drive the FormWindow object:
#
#    type      x    y  name                     value
_detail_form = \
[   (LABEL,  135,  24, None,                    "ID:",  {"text": "stuff"}),
    (STRING,   R,  (), "reporting-id",          None,   {"font": BOLD_FONT}),
    (LABEL,    L,   D, None,                    "Status:"),
    (STRING,   R,  (), "current-status",        None),
    (LABEL,    L,   D, None,                    "Location:"),
    (STRING,   R,  (), "kos",                   None),
    (LABEL,    L,   D, None,                    "Arrived:"),
    (DATETIME, R,  (), "most-recent-move",      None),
    (LABEL,    L,   D, None,                    "Remote ID:"),
    (STRING,   R,  (), "kos-id",                None),
    (LABEL,    L,   D, None,                    "Hops:"),
    (INT,      R,  (), "hop-count",             None),
    (LABEL,    L,   D, None,                    "Launch time:"),
    (DATETIME, R,  (), "launch-time",           None),
    (LABEL,    L,   D, "terminate-label",       None),
    (DATETIME, R,  (), "terminate-time",        None),
    (LABEL,    L,   D, "corefile-label",        None),
    (STRING,   R,  (), "corefile",              None),
    ]
del D, L, R


if __name__ == "__main__":
    import Tkinter
    root = Tkinter.Tk()
    root.withdraw()
    top = Tkinter.Toplevel()
    top.protocol("WM_DELETE_WINDOW", root.quit)
    dw = DetailWindow(top)
    dw["reporting-id"] = "reporting-id"
    dw["no such field"] = "dummy-value"
    dw["hop-count"] = 5
    dw["corefile"] = "test value"
    root.mainloop()
