"""Tk output window supporting multiple interleaved streams."""

import Tkinter
from koe.common import tktools
from Exposable import ExposableToplevel

class Console(ExposableToplevel):
    def __init__(self, master, title="Console", iconname="Console",
                 class_="Console", wrap=Tkinter.CHAR, **kw):
        self.__buttons = []
        self.__expose_tags = {}
        kw["class_"] = class_
        kw["exposed"] = 0
        apply(ExposableToplevel.__init__, (self, master), kw)
        self.title(title)
        self.iconname(iconname)
        x, topframe, botframe = tktools.make_double_frame(self)
        self.__control_frame = botframe
        hbar = (wrap == Tkinter.NONE) and 1 or 0
        self.__text, frame = tktools.make_text_box(
            topframe, hbar=hbar, height=12, takefocus=0)
        self.__text.configure(wrap=wrap, setgrid=1)
        self.add_button(Tkinter.Button(
            botframe, text="Dismiss", command=self.__dismiss_cmd))
        self.add_button(Tkinter.Button(
            botframe, text="Clear", command=self.clear))
        self.__text['state'] = Tkinter.DISABLED

    def get_control_frame(self):
        return self.__control_frame

    def add_button(self, button):
        buttons = self.__buttons
        buttons.append(button)
        if len(buttons) % 2 == 0:
            button.pack(side=Tkinter.RIGHT, padx="1m")
        else:
            button.pack(side=Tkinter.RIGHT)
        if len(buttons) >= 2:
            apply(tktools.unify_button_widths, tuple(buttons))

    def expose_on_tag(self, tag, flag=1):
        if flag:
            self.__expose_tags[tag] = tag
        elif self.__expose_tags.has_key(tag):
            del self.__expose_tags[tag]

    def insert(self, string, tags=()):
        self.__text['state'] = Tkinter.NORMAL
        self.__text.insert(Tkinter.END, string, tags)
        self.__text['state'] = Tkinter.DISABLED
        for tag in tags:
            if self.__expose_tags.has_key(tag) and not self.exposed_p():
                self.expose()
        if self.exposed_p():
            self.__text.yview_pickplace(Tkinter.END)

    def tag_configure(self, tag, **kw):
        apply(self.__text.tag_config, (tag,), kw)

    def clear(self, event=None):
        self.__text['state'] = Tkinter.NORMAL
        self.__text.delete('1.0', Tkinter.END)
        self.__text['state'] = Tkinter.DISABLED

    def __dismiss_cmd(self, event=None):
        self.withdraw()


class TaggedWriter:
    def __init__(self, console, tags=()):
        self.__console = console
        self.__tags = tags

    def get_tags(self):
        return self.__tags

    def set_tags(self, tags):
        self.__tags = tags

    def close(self):
        self.flush()

    def flush(self):
        pass

    def isatty(self):
        return 0

    def write(self, string):
        self.__console.insert(string, self.__tags)

    def writelines(self, lines):
        import string
        self.write(string.joinfields(lines, ""))
