"""Graphical reporting station log monitor."""

import Tkinter ; Tk = Tkinter ; del Tkinter

import ilu, os, string, types
import Dialog
import signal
from koe.common import tktools

import GuiTrackingInfo
from koe.interfaces.stubs import KOS
import KPList
from koe.common import pathhack
from koe.common import physfonts
from koe.interfaces.stubs import MonitorAPI
from koe.interfaces.stubs import MonitorAPI__skel
from koe.common import nstools
from koe.common import RS

from Exposable import ExposableToplevel
from LaunchDialog import LaunchDialog
from LaunchDialog import EXPOSE_ON_OUTPUT, EXPOSE_ON_ERROR, EXPOSE_ON_REQUEST
from LaunchDialog import make_group, receiver_names
from DetailWindow import DetailWindow


OUTSTANDING_KPS_WARNING = 'There are still active KPs which may' \
			  ' need to report to this station. ' \
			  ' Quitting will cause failures for the' \
			  ' outstanding KPs.'

KP_LIST_FONT = physfonts.BODY_R_M
STATUS_FONT = physfonts.BODY_R_M


class GuiRSWatcher(MonitorAPI__skel.MonitorServer):

    __curr_kp = None
    __kill_signal = signal.SIGTERM

    def __init__(self, master=None, application=0, iconic=0, name=None):
        self.__master = master
        self.__application = application
        self.create_widgets(iconic)
        self.set_kplist_mode(KPList.CURRENT, KPList.HISTORIC)
        self.__name = name
        self.publish()

    ######################################################################
    #
    # Published MonitorAPI (external) methods.
    #
    ######################################################################

    __published = 0
    def RequestReporter(self, prefix, ioserver):
        if not self.__published:
            raise KOS.AuthorizationError
        launcher = self.get_launcher(withdrawn=1)
        repid = launcher.generate_id(prefix)
        repsbh = launcher.IluSBH()
        if ioserver:
            io, stdout, stderr = launcher.make_kp_console(repid)
            kpinfo = self.fetch_kpinfo(repid)
            kpinfo.set_console(io)
        else:
            stdout = stderr = ''
        return MonitorAPI.ReporterInfo(repsbh, repid, stdout, stderr)

    ##############
    #
    #  Internal publication controls.

    def get_publication_name(self):
        return self.__name

    __descriptor = None
    def publish(self):
        name = self.get_publication_name()
        if self.__published or not name:
            return
        try:
            ns = nstools.WorldOpen("tools", "Namespace.Context")
        except nstools.BadPathError:
            self.warning("Publication Error",
                         "Could not locate tools namespace.")
            return
        if name in ns.List():
            try:
                obj = nstools.WorldOpen("tools/" + name)
            except nstools.BadPathError:
                ns.Unbind(name)
            else:
                # bound name is "live"
                self.warning("Publication Error",
                             "Publication name \"%s\" already in use"
                             " by another process - not publishing." % name)
                return
        if not self.__descriptor:
            desc = nstools.Descriptor(reference=self.IluSBH(),
                                      typename="MonitorAPI.MonitorServer")
            self.__descriptor = desc
        ns.Bind(name, self.__descriptor)
        self.__published = 1

    def published_p(self):
        return self.__published

    def withdraw(self):
        name = self.get_publication_name()
        if not (self.__published and name):
            return
        try:
            ns = nstools.WorldOpen("tools", "Namespace.Context")
        except nstools.BadPathError:
            pass
        else:
            if self.__name in ns.List():
                ns.Unbind(name)
        self.__published = 0

    ######################################################################
    #
    # Application-specific (internal) methods.
    #
    ######################################################################

    def create_widgets(self, iconic):
	# toss the dummy default root:
        if not Tk._default_root:
            Tk.Tk(className=self.__class__.__name__).withdraw()
        frame = self.__frame = Tk.Toplevel(self.__master, class_="Monitor")
        if iconic:
            self.iconify()
        frame.title("KP Monitoring Tool")
        frame.iconname("KP Monitor")
        frame.protocol("WM_DELETE_WINDOW", self.__quit)

        self._create_menubar()
        self._create_statusbar()

        bigbox = Tk.Frame(frame, borderwidth=2)
        self._create_kplist(bigbox)
        self._create_detail(bigbox)
        bigbox.pack(expand = 1, fill = Tk.BOTH)

    def _create_menubar(self):
        frame = self.__frame
        self.__buttonbar = Tk.Frame(frame, borderwidth=2, relief=Tk.FLAT)
        self.__buttonbar.pack(fill=Tk.X)
        bbar = self.__buttonbar

        self.add_command(bbar, "Launch...", self.show_launcher, "L")
        self.add_command(bbar, "Compose...", self.show_composer, "C")
        self.add_command(bbar, "Options...", self.show_options, "O")

        if self.__application:
            # Do this only if we're an application.
            self.add_command(bbar, "Quit", self.__quit, "Q")

    __ac_buttons = None
    def add_command(self, bbar, label, command, letter=None):
        if self.__ac_buttons is None:
            self.__ac_buttons = {}
        if self.__ac_buttons.has_key(str(bbar)):
            buttongroup = self.__ac_buttons[str(bbar)]
        else:
            buttongroup = []
            self.__ac_buttons[str(bbar)] = buttongroup
        button = Tk.Button(bbar, text=label, command=command)
        button.pack(side=Tk.LEFT, expand=1, fill=Tk.X)
        buttongroup.append(button)
        apply(tktools.unify_button_widths, tuple(buttongroup))
        if letter:
            for letter in (letter, string.lower(letter)):
                for pattern in "<Alt-%s>", "<Meta-%s>":
                    key = pattern % letter
                    self.__frame.bind(key, command)

    def _create_kplist(self, parent):
        self.__kplist = KPList.KPList(parent,
                                      clear_handler=self.clear_selection)
        self.__kplist.pack(side=Tk.LEFT)
        self.__kplist.bind("<Button-1>", self.__pick_kp)
        self.__kplist.bind("<Double-Button-1>", self.__pick_kp_io)
        self.__kplist["font"] = KP_LIST_FONT

    def _create_detail(self, frame):
        fr = Tk.Frame(frame)
        fr.pack(side=Tk.RIGHT, fill=Tk.BOTH, expand=1)
        # button frame first, since we want to pack it first.
        # This helps prevent resize weirdness.
        bbar = Tk.Frame(fr)
        bbar.pack(side=Tk.BOTTOM, fill=Tk.X)
        self.add_command(bbar, "Kill KP", self.kill_kp)
        self.add_command(bbar, "Show I/O...", self.show_io_box, "I")
        self.add_command(bbar, "Traceback...", self.show_traceback, "T")
        self.__detail = DetailWindow(fr, width=300)
        self.__detail.pack()
        self.__kp_buttonset = str(bbar)
        self.__update_kp_buttons(None)

    def _create_statusbar(self):
        self._statusStr = Tk.StringVar()
        self._statusStr.set('')
        fr = Tk.Frame(self.__frame, borderwidth=2, relief=Tk.FLAT)
        fr.pack(fill=Tk.X, side=Tk.BOTTOM)
        self._statusBar = Tk.Label(fr,
                                   anchor = Tk.W,               # TEXT anchor!
                                   justify = Tk.LEFT,
                                   textvariable = self._statusStr,
                                   borderwidth=1, relief=Tk.SUNKEN,
                                   font=STATUS_FONT)
        self._statusBar.pack(fill=Tk.X)

    #======================================================================
    #
    # General commands
    #
    #======================================================================

    def show_composer(self, event=None):
        kosnames = receiver_names()
        if kosnames:
            receiver = kosnames[0]
        else:
            import socket
            receiver = socket.gethostname()
        import Builder
        Builder.BuilderDialog(self.__frame, self, None, '', receiver)

    def show_launcher(self, event=None):
        self.get_launcher().expose()

    def show_options(self, event=None):
        self.get_options_dialog().expose()

    #======================================================================
    #
    # KP-specific commands
    #
    #======================================================================

    def kill_kp(self, event=None):
        repid = self.__kplist.get_reporting_id()
        if not repid:
            self.warning("No KP Selected",
                         "Select a KP before requesting termination.")
            return
        kpinfo = self.__kplist.get_kpinfo(repid)
        if kpinfo.get_completion_time():
            self.warning("KP Not Active",
                         "Selected KP is not active.")
            return
        kpinfo.set_cloning(0)
        kosname, kpid = kpinfo.get_location()
        receiver = nstools.WorldOpen("kos/%s/receiver" % kosname,
                                     "KOS.Receiver")
        try:
            receiver.KillKP(kpid, self.__kill_signal, "Killed by RS.")
        except (KOS.AuthorizationError, KOS.ResourceError), msg:
            self.warning("KP Termination Error", msg)
        else:
            self.set_status("Killed KP %s located at %s" % (repid, kosname))

    def kill_descendents(self, event=None):
        kpinfo = self.__kplist.get_kpinfo()
        if not kpinfo:
            self.warning("No KP Selected",
                         "Select a KP before requesting termination.")
            return
        targets = [kpinfo]
        num_killed = 0
        while targets:
            kpinfo = targets[0]
            del targets[0]
            if kpinfo.get_completion_time():
                kpinfo.set_cloning(0)
                kosname, kpid = kpinfo.get_location()
                receiver = nstools.WorldOpen("kos/%s/receiver" % kosname,
                                             "KOS.Receiver")
                try:
                    receiver.KillKP(kpid, self.__kill_signal, "Killed by RS.")
                except (KOS.AuthorizationError, KOS.ResourceError), msg:
                    self.warning("KP Termination Error", msg)
                else:
                    num_killed = num_killed + 1
            for repid in kpinfo.get_children():
                info = self.__kplist.get_kpinfo(repid)
                if info:
                    targets.append(info)
        self.set_status("Killed %d KPs" % num_killed)

    def show_io_box(self, event=None):
        kpinfo = self.__kplist.get_kpinfo()
        if kpinfo:
            console = kpinfo.get_console()
            if console:
                console.expose()
            else:
                self.warning("I/O Server Unavailable",
                             "No I/O server is available for this KP.")
        else:
            self.warning("No KP Selected",
                         "Select a KP before requesting the output console.")

    def show_traceback(self, event=None):
        repid = self.__kplist.get_reporting_id()        # actually need the ID
        if not repid:
            self.warning(
                "No KP Selected",
                "Select a KP before requesting traceback information.")
            return
        kpinfo = self.__kplist.get_kpinfo(repid)
        if not kpinfo.get_completion_time():
            self.warning("No Traceback Information",
                         "KP %s is still running." % repid)
        excp, value, tb = kpinfo.get_exception()
        if excp:
            from ExceptionDialog import ExceptionDialog
            ExceptionDialog(self.__frame, repid, excp, value, tb)
        else:
            self.warning("No Traceback Information",
                         "KP %s did not die of an exception." % repid)

    #======================================================================
    #
    # Misc. internal interfaces.
    #
    #======================================================================

    def kp_migrated(self, id, idTo, migratingTo):
        kpinfo = self.fetch_kpinfo(id)
        kpinfo.relocate(migratingTo, idTo)
        self.update_kp(kpinfo)

    def kp_terminated(self, id, reason, details, kprepr):
        kpinfo = self.fetch_kpinfo(id)
        if kpinfo and kpinfo.get_console():
            kpinfo.get_console().remove_kp(id)
        kpinfo.terminate(reason, details, kprepr)
        self.update_kp(kpinfo)

    def fetch_kpinfo(self, repid, create=1):
        kp = self.__kplist.get_kpinfo(repid)
        if kp is None and create:
            kp = GuiTrackingInfo.GuiTrackingInfo(repid, window=self.__detail)
            self.__kplist.insert_kp(kp)
        return kp

    __directory = os.path.join(pathhack.kos_root(), "examples")

    def get_directory(self):
        return self.__directory

    def set_directory(self, dir):
        self.__directory = dir

    def set_status(self, message):
        self._statusStr.set(message)

    def select_kp(self, repid):
        """Selects a single KP for presentation in the details area."""
        if self.__curr_kp and self.__curr_kp != repid:
            tracker = self.__kplist.get_kpinfo(self.__curr_kp)
            if tracker:
                tracker.disable_display()
        self.__curr_kp = repid
        self.__kplist.select_kp(repid)
        if repid:
            kpinfo = self.__kplist.get_kpinfo(repid)
            self.__update_kp_buttons(kpinfo)

    def update_kp(self, kpinfo):
        self.__kplist.update_kp(kpinfo)
        if self.__curr_kp == kpinfo.get_reporting_id():
            self.__update_kp_buttons(kpinfo)

    def __update_kp_buttons(self, kpinfo):
        buttons = self.__ac_buttons[self.__kp_buttonset]
        if kpinfo:
            buttons[0]["state"] = kpinfo.get_completion_time() \
                                  and Tk.DISABLED or Tk.NORMAL
            buttons[1]["state"] = kpinfo.get_console() \
                                  and Tk.NORMAL or Tk.DISABLED
            buttons[2]["state"] = kpinfo.get_exception()[2] \
                                  and Tk.NORMAL or Tk.DISABLED
        else:
            for b in buttons:
                b["state"] = Tk.DISABLED

    def clear_selection(self):
        if self.__curr_kp:
            kpinfo = self.__kplist.get_kpinfo(self.__curr_kp)
            kpinfo.disable_display()
            self.__detail.clear()
            self.__curr_kp = None
            self.__update_kp_buttons(None)

    __cloning_enabled = 1
    def set_cloning(self, flag):
        self.__cloning_enabled = flag and 1 or 0

    def cloning_p(self):
        return self.__cloning_enabled

    __exposure = EXPOSE_ON_OUTPUT
    def get_exposure(self):
        return self.__exposure

    def set_exposure(self, exposure):
        self.__exposure = exposure

    def set_killsignal(self, signal):
        self.__kill_signal = signal

    def set_kplist_mode(self, left, right):
        self.__kplist.config_radios(left, right)
        self.__kplist.select_list(left)

    __warned = 0
    def warning(self, title, text, bitmap=Dialog.DIALOG_ICON):
        self.__warned = 1
        self.set_status("")
        Dialog.Dialog(None, title=title, text=text, bitmap=bitmap,
                      default=0, strings=("OK",))

    def missing_worldroot(self):
        self.warning("Missing Worldroot",
                     "The Worldroot is missing;"
                     " please start the Worldroot server!")

    __launcher = None
    __launcher_exposed = 0
    def get_launcher(self, withdrawn=0):
        """Return the launcher object, creating it if needed."""
        self.__warned = 0
        if not self.__launcher:
            self.__launcher = LaunchDialog(self, self.__frame)
        if withdrawn:
            self.__launcher.withdraw()
        elif not self.__launcher_exposed:
            if not self.__warned:
                # if self.warning() was called, it's too late!
                tktools.set_transient(self.__launcher.top, self.__frame,
                                      expose=0)
            self.__launcher_exposed = 1
        return self.__launcher

    __options = None
    def get_options_dialog(self):
        if not self.__options:
            self.__options = OptionsDialog(self.__frame, self)
        return self.__options

    def iconify(self):
        if self.__application:
            self.__frame.iconify()

    def deiconify(self):
        if self.__application:
            self.__frame.deiconify()

    def __pick_kp(self, event):
        try:
            repid = self.__kplist.get(Tk.At(event.x, event.y))
        except IndexError:
            return                      # list is empty
        self.select_kp(repid)

    def __pick_kp_io(self, event):
        try:
            repid = self.__kplist.get(Tk.At(event.x, event.y))
        except IndexError:
            return                      # list is empty
        self.select_kp(repid)
        self.show_io_box()

    def __quit(self, event = None):
        close = 1
        if self.__kplist.get_ids(KPList.ACTIVE):
            # Bring up dialog warning the user before going down:
            d = Dialog.Dialog(
                None, title='Active Knowbots!',
                text=OUTSTANDING_KPS_WARNING, bitmap=Dialog.DIALOG_ICON,
                default=0, strings=('Cancel', 'Quit anyway'))
            close = d.num
        if close:
            import ilu_tk
            ilu_tk.ExitMainLoop()


class OptionsDialog(ExposableToplevel):

    def __init__(self, master, app, title="Launcher Options", **kw):
        self.__app = app
        self.__exposure_mode = Tk.StringVar()
        self.__exposure_mode.set(EXPOSE_ON_OUTPUT)
        self.__kplist_mode = Tk.StringVar()
        self.__kplist_mode.set("current / historic")
        self.__killsignal = Tk.StringVar()
        self.__killsignal.set("TERM")
        self.__cloning = Tk.BooleanVar()
        self.__cloning.set(1)
        self.__published = Tk.BooleanVar()
        self.__published.set(app.published_p())
        apply(ExposableToplevel.__init__, (self, master), kw)
        self.title(title)
        self.__create_widgets()
        tktools.set_transient(self, master)
        self.minsize(self.winfo_reqwidth(), self.winfo_reqheight())

    def __create_widgets(self):
        fr, topfr, botfr = tktools.make_double_frame(self)
        # option controls:
        options_frame = Tk.Frame(topfr)
        options_frame.pack(expand=1, fill=Tk.BOTH)
        lframe = make_group(options_frame, "listmodes", "List mode:",
                            side=Tk.LEFT)
        Tk.Frame(options_frame, width=8).pack(side=Tk.LEFT)
        rframe = make_group(options_frame, "consolemodes", "KP I/O console:",
                            side=Tk.RIGHT)
        self.__create_kplist_mode_controls(lframe)
        self.__create_exposure_mode_controls(rframe)
        Tk.Frame(topfr, height=8).pack()
        clframe = make_group(topfr, "options", "Miscellaneous options:",
                             fill=Tk.X)
        self.__create_misc_controls(clframe)
##      self.__create_killsignal_controls(clframe)
        # dialog controls:
        accept_btn = Tk.Button(botfr, text="OK", command=self.__accept_cmd)
        apply_btn = Tk.Button(botfr, text="Apply", command=self.__apply_cmd)
        cancel_btn = Tk.Button(botfr, text="Cancel", command=self.__cancel_cmd)
        tktools.unify_button_widths(accept_btn, apply_btn, cancel_btn)
        accept_btn.pack(side=Tk.LEFT)
        apply_btn.pack(side=Tk.LEFT, padx='1m')
        cancel_btn.pack(side=Tk.RIGHT)

    def __create_kplist_mode_controls(self, frame):
        value1 = "%s / %s" % (KPList.ACTIVE, KPList.COMPLETED)
        value2 = "%s / %s" % (KPList.CURRENT, KPList.HISTORIC)
        self.add_radios(frame, self.__kplist_mode,
                        [(string.capwords(value1), value1),
                         (string.capwords(value2), value2)])

    def __create_exposure_mode_controls(self, frame):
        self.add_radios(frame, self.__exposure_mode,
                        [("Expose I/O on output", EXPOSE_ON_OUTPUT),
                         ("Expose I/O on error", EXPOSE_ON_ERROR),
                         ("Expose I/O on request", EXPOSE_ON_REQUEST)])

    def __create_killsignal_controls(self, frame):
        fr = Tk.Frame(frame)
        fr.pack(anchor=Tk.W)
        Tk.Label(fr, text="Kill with signal: ").pack(side=Tk.LEFT)
        signals = self.__signals = {}
        for v in dir(signal):
            if v[:3] == "SIG" and v[3] != "_":
                signals[v[3:]] = getattr(signal, v)
        items = map(None, signals.values(), signals.keys())
        items.sort()
        names = []
        for sig, name in items:
            names.append(name)
        obox = apply(Tk.OptionMenu, tuple([fr, self.__killsignal] + names))
        obox["width"] = reduce(max, map(len, names), 1)
        obox.pack(side=Tk.LEFT)

    def __create_misc_controls(self, frame):
        Tk.Checkbutton(frame, text="Enable cloning",
                       variable=self.__cloning, anchor=Tk.W
                       ).pack(fill=Tk.X)
        if not self.__app.get_publication_name():
            return
        Tk.Checkbutton(frame, text="Publish client interface",
                       variable=self.__published, anchor=Tk.W
                       ).pack(fill=Tk.X)

    def add_radios(self, frame, variable, pairs):
        radios = []
        for (text, value) in pairs:
            rb = Tk.Radiobutton(frame, anchor=Tk.W,
                                text=text, value=value, variable=variable)
            rb.pack(fill=Tk.X)
            radios.append(rb)
        return radios

    def __accept_cmd(self, event=None):
        self.__apply_cmd(event)
        self.withdraw()

    def __apply_cmd(self, event=None):
        [left, x, right] = string.split(self.__kplist_mode.get())
        self.__app.set_kplist_mode(left, right)
        self.__app.set_exposure(self.__exposure_mode.get())
##      signal = self.__signals[self.__killsignal.get()]
##      self.__app.set_killsignal(signal)
        self.__app.set_cloning(self.__cloning.get())
        if self.__published.get():
            self.__app.publish()
        else:
            self.__app.withdraw()
        self.__published.set(self.__app.published_p())

    def __cancel_cmd(self, event=None):
        self.withdraw()
