"""TrackingInfo with automatic display updating."""


from TrackingInfo import TrackingInfo, LocationError


class GuiTrackingInfo(TrackingInfo):
    """Version of TrackingInfo with hooks to update a graphical display."""
    def __init__(self, repid, window=None, display=None):
        TrackingInfo.__init__(self, repid)
        self.__window = window
        if display and window:
            self.enable_display(window)

    __console = None
    def get_console(self):
        return self.__console

    def set_console(self, console):
        if self.__console is not console:
            if self.__console:
                self.__console.remove_kp(self.get_reporting_id())
            if console:
                console.add_kp(self.get_reporting_id())
        self.__console = console

    __display = 0
    def enable_display(self, window = None):
        if window:
            self.__window = window
        self.__display = self.__window and 1 or 0
        if self.__display:
            self.update_display()

    def disable_display(self):
        self.__display = 0

    def update_display(self):
        window = self.__window
        window["current-status"] = self._status
        try:
            kos, kpid, time = self.get_lastmove()
        except LocationError:
            kos, kpid, time = None, None, None
        #
        window["kos"] = kos
        window["kos-id"] = kpid
        window["most-recent-move"] = time
        window["hop-count"] = self.get_hopcount()
        window["corefile"] = self.get_corefile()
        window["reporting-id"] = self.get_reporting_id()
        window["launch-time"] = self.get_launchtime()
        window["terminate-time"] = self.get_completion_time()

    def set_corefile(self, fileName):
        TrackingInfo.set_corefile(self, fileName)
        if self.__display:
            self.__window["corefile"] = self.get_corefile()

    def relocate(self, newKosAddress, newKosID):
        TrackingInfo.relocate(self, newKosAddress, newKosID)
        if self.__display:
            self.update_display()

    def terminate(self, *args):
        apply(TrackingInfo.terminate, (self,) + args)
        if self.__display:
            self.__window["corefile"] = self.get_corefile()
            self.__window["terminate-time"] = self.get_completion_time()
