#! /usr/bin/env python
"""GUI launcher / reporting station script.

Usage:  rs.py [-n|--name NAME]

Where:
    -n | --name NAME    means to bind the public interface as tools/NAME;
                        default is not to bind into the namespace.

"""

import sys, os; sys.path.insert(0, os.path.join(sys.path[0], "../../"))
from koe.common import pathhack

import Tkinter
tk = Tkinter.Tk(className="Launcher")
tk.withdraw()

from koe.common import nstools
import getopt
import ilu_tk
from koe.common import ilutools                   # for exception handler only
import TrackingInfo

from GuiRSWatcher import GuiRSWatcher

UNBOUND_INSTRUCTIONS = """: old monitor still alive -- exiting.

Run without -n|--name option to skip binding in the namespace.

"""

iconic = 0
myname = None

if len(sys.argv) > 1:
    try:
        optlist, args = getopt.getopt(sys.argv[1:], "in:", ["iconic", "name="])
    except getopt.error:
        sys.stderr.write(__doc__)
        sys.exit(2)
    if args:
        os.stderr.write(
            os.path.basename(sys.argv[0]) + ": too many arguments\n\n")
        sys.stderr.write(__doc__)
        sys.exit(2)
    for opt, val in optlist:
        if opt in ("-n", "--name"):
            myname = val
        elif opt in ("-i", "--iconic"):
            iconic = 1

try:
    rsw = GuiRSWatcher(tk, application=1, iconic=iconic, name=myname)
except nstools.NoWorldrootError, msg:
    print "If at first you don't succeed"
    print "The worldroot is what I need"
    print msg
    sys.exit(1)

TrackingInfo.set_core_path(os.path.join(pathhack.kos_root(), "var"))

try:
    ilu_tk.RunMainLoop()
finally:
    rsw.withdraw()
