#! /usr/bin/env python

"""The KOE Visualizer utility.

The Visualizer provides a graphical view of the KSSs and KPs
accessible via the Namespace.  It displays animations as KPs migrate
from host-to-host, clone themselves, etc.

Options:

    -h (--help)  Print usage message and exit.

    -n (--name)  Optional name to bind Visualizer under Worldroot's
                 tools/ directory.  This must be unique, and if not
                 provided, the Visualizer does not bind a name to
                 itself in the namespace.

"""

import sys, os; sys.path.insert(0, os.path.join(sys.path[0], "../../"))
import getopt
import ilu
from koe.common import nstools
from koe.tools.visu import Controller

ilu.SetFDBudget(2 * ilu.GetFDBudget())


def usage(status):
    print __doc__
    sys.exit(status)


def main():
    # find the worldroot
    try:
        worldns = nstools.GetWorldroot()
    except nstools.NoWorldrootError, msg:
        print 'Try as I can, try as I must'
        print 'Seems the worldroot has bit the dust:'
        print msg
        sys.exit(-1)
    #
    # process command line options
    #
    name = None
    quiet = None
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'qhn:',
                                   ['help', 'name=', 'quiet'])
    except getopt.error, msg:
        print msg
        usage(-1)
    if args:
        usage(-1)
    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage(0)
        elif opt in ('-n', '--name'):
            name = val
        elif opt in ('-q', '--quiet'):
            quiet = 1

    c = Controller.Controller(name, quiet)
    c.start()
    try:
        c.close()
    except (ilu.IluGeneralError, ilu.IluProtocolError):
        pass

    

if __name__ == "__main__":
    main()
